#-------------------------------------------------------------------------------->
# Author: Vladimir Yankov (vladimir.yankov@gmail.com)
# Conducts a set of counter-factual exercises using the structural estimates of the
# search cost distribution in section 5.
#-------------------------------------------------------------------------------->
# Clear workspace
rm(list = ls())
#-------------------------------------------------------------------------------->
# Folders----
final_data_dir<- "./Data_InSearchOfRiskFreeAsset/str_est/"
tables_dir ="./InSearchRisklessAsset/Results/Tables/"
figures_dir = "./InSearchRisklessAsset/Results/Figures/"
code_proj_dir = "./InSearchRisklessAsset/Code/StructuralEstimation/functions/"
counterfactuals.results.dir <- "./Data_InSearchOfRiskFreeAsset/str_est/counterfactuals/"
#-------------------------------------------------------------------------------->
# Libraries----
source(paste0(code_proj_dir,"functions.R"))
source(paste0(code_proj_dir,"functions_equilibrium.R"))
require(nleqslv)
require(rootSolve)
require(lmomco)
require(GoFKernel)
# Set multi-threading
setMKLthreads(18)
#-------------------------------------------------------------------------------->
# Load data----
setwd(final_data_dir)
load(file = "F_xi_qvec_msa_y.rda")
load(file =paste0(final_data_dir,"F_xi.rda"))
# Market concentration measures
load(file = "sod_msa.rda")
load(file = "may9_final.rda")
time_deposits_hhi = may9[,list(time_less100K_hhi= 100*hhi(time_less100K),
                               time_less250K_hhi= 100*hhi(time_less250K)),by="date"]
time_deposits_hhi = time_deposits_hhi[order(time_deposits_hhi$date),]
time_deposits_hhi[time_deposits_hhi$time_less100K_hhi==0,time_less100K_hhi:=NA]
time_deposits_hhi[time_deposits_hhi$time_less250K_hhi==0,time_less250K_hhi:=NA]
time_deposits_hhi[,year:=year(date)]
time_deposits_hhi = time_deposits_hhi[,list(time_less100K_hhi=mean(time_less100K_hhi),
                                            time_less250K_hhi = mean(time_less250K_hhi)), by = "year"]
time_deposits_hhi[,small_time_hhi:= time_less100K_hhi]
time_deposits_hhi[is.na(time_less100K_hhi),small_time_hhi:= time_less250K_hhi]

sod_msa_agg = data.table(sod_msa)
sod_msa_agg[,bank_nmarkets:=length(unique(msa_code)),by = c("id_rssd_th","year")]
sod_msa_agg[,nbanks_agg:= as.numeric(length(unique(id_rssd_th))), by = "year"]
msa_codes.est = unique(F_xi_qvec_msa_y$msa_code)
sod_msa_agg = sod_msa_agg[!is.na(sod_msa_agg$msa_code) & sod_msa_agg$msa_code %in% msa_codes.est,
                          list(deposits = as.numeric(sum(dep_br,na.rm=T)),
                               hhi_deposits = hhi(dep_br),
                               nbanks = length(unique(id_rssd_th)),
                               bank_nmarkets = mean(bank_nmarkets,na.rm=T),
                               nbanks_agg =mean(nbanks_agg)),
                          by = c("msa_code","year")]
sod_agg = sod_msa_agg[,list(nbanks_wa = weighted.mean(nbanks,w=deposits,na.rm=T),
                            nbanks_med = median(nbanks, na.rm=T),
                            nbanks_agg = median(nbanks_agg),
                            bank_nmarkets = mean(bank_nmarkets,na.rm=T),
                            hhi_med = 100*median(hhi_deposits,na.rm=T),
                            hhi_wa = 100*weighted.mean(hhi_deposits,w=deposits,na.rm=T)
), by = "year"]

sod_agg = sod_agg[order(sod_agg$year)]
sod_agg = merge(sod_agg,time_deposits_hhi, by = "year", all.x=T)
names(sod_agg)
sod_agg = sod_agg[sod_agg$year <= 2016 & sod_agg$year>1996,]

#-------------------------------------------------------------------------------->
# Counterfactuals----
#-------------------------------------------------------------------------------->
#___Pass-through----
qvec_delta_m = qvec_msa_m
dstart = as.Date(c("1999-06-30","2004-06-30","2015-12-16"))
dates = unique(qvec_delta_m$date)
date_axis = unique(qvec_agg_m$date)
date_axis = date_axis[which(month(date_axis)==6)]
date_axis2 = seq(min(date_axis), max(date_axis), by = 365)
date_axis2 = c(date_axis2,max(qvec_agg_m$date))

qvec_delta_m[,qsum:= sum(q_k*qvec), by=c("msa_code","date")]
qvec_delta_m[,qsum_med:= sum(q_k*qvec*(1/2)^(q_k-1)), by=c("msa_code","date")]
qvec_delta_m[,qsum_p25:= sum(q_k*qvec*(1/4)^(q_k-1)), by=c("msa_code","date")]
qvec_delta_m[,qsum_p75:= sum(q_k*qvec*(3/4)^(q_k-1)), by=c("msa_code","date")]
qvec_delta_m[,qsum_p5:= sum(q_k*qvec*(0.05)^(q_k-1)), by=c("msa_code","date")]
qvec_delta_m[,qsum_p95:= sum(q_k*qvec*(0.95)^(q_k-1)), by=c("msa_code","date")]


qvec_delta_m[,q1:= qvec[q_k==1], by=c("msa_code","date")]
qvec_delta_m[,q1_qsum:=q1/qsum, by=c("msa_code","date")]
qvec_delta_m[,q1_qsum_p5:=q1/qsum_p5, by=c("msa_code","date")]
qvec_delta_m[,q1_qsum_med:=q1/qsum_med, by=c("msa_code","date")]
qvec_delta_m[,q1_qsum_p25:=q1/qsum_p25, by=c("msa_code","date")]
qvec_delta_m[,q1_qsum_p75:=q1/qsum_p75, by=c("msa_code","date")]
qvec_delta_m[,q1_qsum_p95:=q1/qsum_p95, by=c("msa_code","date")]


summary(qvec_delta_m$q1_qsum)
summary(qvec_delta_m$q1_qsum_med)
summary(qvec_delta_m$q1_qsum_p25)


qvec_summ_agg = qvec_delta_m[,list(q1_qsum=median(q1_qsum,na.rm=T),
                                   q1_qsum_med=median(q1_qsum_med,na.rm=T),
                                   q1_qsum_p5=median(q1_qsum_p5,na.rm=T),
                                   q1_qsum_p25=median(q1_qsum_p25,na.rm=T),
                                   q1_qsum_p75=median(q1_qsum_p75,na.rm=T),
                                   q1_qsum_p95=median(q1_qsum_p95,na.rm=T)
),
by = "date"]
qvec_summ_agg=qvec_summ_agg[order(qvec_summ_agg$date),]
# pass-through
qvec_summ_agg[,pt_p5  := 1-q1_qsum_p5]
qvec_summ_agg[,pt_p50 := 1-q1_qsum_med]
qvec_summ_agg[,pt_p25 := 1-q1_qsum_p25]
qvec_summ_agg[,pt_p75 := 1-q1_qsum_p75]
qvec_summ_agg[,pt_max := 1-q1_qsum]
summary(qvec_summ_agg[qvec_summ_agg$date<="2008-01-01",list(pt_p25,pt_p50,pt_p75,pt_max)])

pdf(file=paste0(figures_dir,"PassThrough.pdf"),height=10,width=16)
with(qvec_summ_agg, plot(date, 1-q1_qsum_p5,
                         cex=2,
                         cex.axis=2,
                         cex.lab=2,
                         col = "red",
                         xlab ="",
                         xaxt = "n",
                         ylab = "",
                         type ="l",
                         lwd=2,
                         ylim = c(0,1)))
axis.Date(side = 1, qvec_summ_agg$date, at =date_axis2 ,
          format = "%Y", las = 2, cex.axis =2)
par(new=T)
with(qvec_summ_agg, lines(date, 1-q1_qsum_med, col = "black", type ="l",lwd=2, ylim = c(0,1)))
with(qvec_summ_agg, lines(date, 1-q1_qsum_p25, col = "blue", type ="l",lwd=2, ylim = c(0,1)))
with(qvec_summ_agg, lines(date, 1-q1_qsum_p75, col = "blue", type ="l",lwd=2, ylim = c(0,1)))
with(qvec_summ_agg, lines(date, 1-q1_qsum_p95, col = "red", type ="l",lwd=2, ylim = c(0,1)))
with(qvec_summ_agg, lines(date, 1-q1_qsum, col = "purple", type ="l",lwd=2, ylim = c(0,1)))

abline(v=dstart, col ="darkgray",lty=2, lwd=3)
text(x =dstart[1], y = 0.01,pos = 4, col ="slateblue4",
     labels =TeX(paste0("$\\leftarrow$",dstart[1])),
     cex = 2)
text(x =dstart[2], y = 0.24,pos = 4, col ="slateblue4",
     labels =TeX(paste0("$\\leftarrow$",dstart[2])),
     cex = 2)
text(x =dstart[3], y = 0.1,pos =2, col ="slateblue4",
     labels =TeX(paste0(dstart[3],"$\\rightarrow$")),
     cex = 2)
abline(h=c(0,0.2,0.4,0.6,0.8,1), col = "darkgray")

text(x =as.Date("2011-06-30"), y = 0.9,pos = 4, col ="purple",
     labels =TeX(paste0("$\\leftarrow$ Max.rate")),
     cex = 1.7)

text(x =as.Date("2011-06-30"), y = 0.75,pos = 4, col ="red",
     labels =TeX(paste0("$\\leftarrow$ z = 0.95")),
     cex = 1.7)

text(x =as.Date("2011-06-30"), y = 0.58,pos = 4, col ="blue",
     labels =TeX(paste0("$\\leftarrow$ z = 0.75")),
     cex = 1.7)

text(x =as.Date("2011-06-30"), y = 0.42,pos = 4, col ="black",
     labels =TeX(paste0("$\\leftarrow$ z = 0.50")),
     cex = 1.7)

text(x =as.Date("2011-06-30"), y = 0.28,pos = 4, col ="blue",
     labels =TeX(paste0("$\\leftarrow$ z = 0.25")),
     cex = 1.7)

text(x =as.Date("2011-06-30"), y = 0.07,pos = 4, col ="red",
     labels =TeX(paste0("$\\leftarrow$ z = 0.05")),
     cex = 1.7)

# legend("topright", c("Maximum rate","5/95","25/75","Median"),
#        ncol= 2,
#        col=c("purple","red","blue","black"),lty=1,lwd=2,bty="n",cex=2)


dev.off()


#-------------------------------------------------------------------------------->
#___ Welfare analysis----
#-------------------------------------------------------------------------------->
qvec_delta_m = merge(qvec_msa_m[, list(msa_code,date, qvec,nbanks, q_k, ks_p,dep_msa,
                                       R_t,R_max,R_range)],
                     F_xi_msa_m[,list(msa_code,date,deltak,q_k)], by = c("msa_code","date","q_k"))

qvec_delta_m[, R_min:=R_max-R_range]
msa = unique(qvec_delta_m$msa_code[qvec_delta_m$ks_p>0.05])

deposit_rates_m = data.table(deposit_rates_m)
Call_funding = data.table(Call_funding)
sumd_bank_msa = data.table(sumd_bank_msa)
# ----------------------------------------------------------------------------->
cluster = makeCluster(20, type = "SOCK")
#registerDoSNOW(cluster)
registerDoParallel(cl = cluster, cores =20)
#------------------------------------------------------------------------------>


## Loop over markets and dates
search_cost_m = foreach(m = msa, .combine = "rbind",
                        .packages = c("foreach","data.table","nleqslv"),
                        #.errorhandling = "remove",
                        .verbose = T)%dopar%{

                          qvec_m = qvec_delta_m[qvec_delta_m$msa_code==m,]
                          dates_m = unique(qvec_m$date)
                          dates_m = dates_m[dates_m %in% unique(deposits_msa_m$date[deposits_msa_m$msa_code==m])]
                          index_m = which(deposits_msa_m$msa_code==m)
                          deposit_rates_m = deposits_msa_m[index_m,]
                          ids_m = unique(deposit_rates_m$id_rssd_hd_off)


                          df_m =  foreach(d = dates_m, .combine = "rbind")%do%{
                            print(d)
                            data_m_t = qvec_m[qvec_m$date==d]
                            deposit_funding_m = Call_funding[Call_funding$id_rssd %in% ids_m &
                                                               quarter(Call_funding$date)==quarter(d) &
                                                               year(Call_funding$date)==year(d),
                                                             list(id_rssd,date,time_below100K_q,total_deposits)]
                            deposit_rates_m_t = deposit_rates_m[deposit_rates_m$date==d,]
                            deposit_funding_m[, share_time:=time_below100K_q/total_deposits]

                            ## need to update data for 2012-2016
                            if(year(d)<=2011){time_deposit_share_m_t = sumd_bank_msa[sumd_bank_msa$msa_code==m &
                                                                                       sumd_bank_msa$year==year(d),
                                                                                     list(id_rssd,share_msa_bank)]}

                            if(year(d)>2011){time_deposit_share_m_t = sumd_bank_msa[sumd_bank_msa$msa_code==m &
                                                                                      sumd_bank_msa$year==2011,
                                                                                    list(id_rssd,share_msa_bank)]}


                            ## assume proportionality of time deposits/total deposits and msa allocations
                            time_deposit_share_m_t = merge(time_deposit_share_m_t,deposit_funding_m, by = "id_rssd")
                            time_deposit_share_m_t[,time_deposits_msa:=share_msa_bank*share_time*total_deposits]
                            time_deposit_share_m_t$date=NULL
                            time_deposit_share_m_t = merge(time_deposit_share_m_t,deposit_rates_m_t,
                                                           by.y = "id_rssd_hd_off",
                                                           by.x= "id_rssd", all= F)
                            time_deposits_m = sum(time_deposit_share_m_t$time_deposits_msa,na.rm=T)

                            ## fit price distribution
                            rates = deposit_rates_m_t$apy_cd12/100+1
                            rates = rates[!is.na(rates)]
                            Nbanks = length(rates)
                            R_min = min(rates)
                            R_max = max(rates)
                            R_tilde = unique(data_m_t$R_t)
                            qvec = data_m_t$qvec
                            F_R_ecdf <- ecdf(rates)







                            ## average search costs


                            data_m_t$deltak_Rmin = 10000*data_m_t$deltak/phi_prime(data_m_t$R_min, sigma, beta, tau)
                            data_m_t[,d_delta := (deltak_Rmin+lag(deltak_Rmin,1))/2]
                            data_m_t[,sum_k:= (q_k-1)*qvec*d_delta]


                            total_search_cost = sum(data_m_t$sum_k,na.rm=T)

                            df = data.frame(msa = m,
                                            date = d,
                                            V_avg = V_all,
                                            V_R_t = V_R_t,
                                            V_delta = (V_R_t/V_all-1)*100,
                                            search_cost =total_search_cost,
                                            time_deposits_m = time_deposits_m)

                            return(df)}

                          return(df_m)
                        }
stopCluster(cluster)
#------------------------------------------------------------------------------>
#___Construct aggregate estimates of search costs and welfare------
#------------------------------------------------------------------------------>
save(search_cost_m,file=paste0(final_data_dir,"search_cost_agg.rda"))


## add other market characteristics
setwd(final_data_dir)
load("search_cost_agg.rda")
search_cost_m = merge(search_cost_m,
                      qvec_delta_m, by.y = c("msa_code","date"),
                      by.x = c("msa","date"))
search_cost_m = data.table(search_cost_m)
search_cost_m$year = year(search_cost_m$date)
search_cost_agg = search_cost_m[, list(search_cost = weighted.mean(search_cost,w=time_deposits_m,na.rm=T),
                                       spread_max = 100*weighted.mean(R_t-R_max,w=time_deposits_m,na.rm=T),
                                       spread_min = 100*weighted.mean(R_t-R_min,w=time_deposits_m,na.rm=T),
                                       R_t = weighted.mean(R_t-R_min,w=time_deposits_m,na.rm=T),
                                       V_avg = weighted.mean(V_avg,w=time_deposits_m,na.rm=T),
                                       V_R_t = weighted.mean(V_R_t,w=time_deposits_m,na.rm=T),
                                       V_delta = weighted.mean(V_delta,w=time_deposits_m,na.rm=T)),
                                by = "year"]
search_cost_agg = search_cost_agg[order(search_cost_agg$year),]



time_deposits_agg = Call_funding[,list(time_below100K_q = sum(time_below100K_q,na.rm=T)), by = "date"]
time_deposits_agg =time_deposits_agg[time_deposits_agg$date>"1997-01-01" &
                                       time_deposits_agg$date <"2017-01-01",]
time_deposits_agg$year = year(time_deposits_agg$date)
time_deposits_agg = time_deposits_agg[,list(time_below100K_q = mean(time_below100K_q,na.rm=T)), by="year"]


search_cost_agg = merge(search_cost_agg,time_deposits_agg, by = "year")
search_cost_agg[,search_cost_dollar:=time_below100K_q*search_cost/10000]



par(mar=4*c(1,1,0.3,1))

pdf(file=paste0(figures_dir,"search_cost_agg.pdf"), height=10, width = 16)

with(search_cost_agg, plot(year,search_cost,lwd=2, col = "black", type="l",
                           ylab = "",
                           cex=2,cex.lab =1.5,
                           cex.axis =2,
                           xaxt = "n",yaxt="n",
                           xlab=""))
axis(side = 1,at =1997:2016,cex.axis=1.5)
axis(side = 2,at =seq(5,25,length.out = 5), cex.axis=1.5)
grid(nx=NULL,ny =NULL , col = "gray")
par(new=T)
with(search_cost_agg, plot(year,search_cost_dollar/10^6,lwd=2,  type="l",
                           ylab = "", col = "blue",lty=2,
                           cex=2,cex.lab =1.5,
                           cex.axis =2,
                           xaxt = "n",yaxt="n",
                           xlab=""))
axis(side = 4,cex.axis=2)
#mtext(side = 4, line = 2, 'billion', cex = 1.5)
legend("topright",c("Basis points (left)","Dollar amount, billions (right)"),
       col = c("black","blue"),
       cex=1.5,
       lty = c(1,2), bty ="n")


dev.off()



pdf(file=paste0(figures_dir,"welfare_agg.pdf"), height=10, width = 16)

with(search_cost_agg, plot(year,V_delta,lwd=2, col = "black", type="l",
                           ylab = "", cex=2.5,cex.lab =2.5,cex.axis =2.5,
                           ylim = c(0,1),
                           xaxt="n",yaxt="n",
                           xlab=""))
axis(side = 1,at =1997:2016,cex.axis=1.7)
axis(side = 2,at =seq(0,1,length.out = 5),cex.axis=1.7)
par(new=T)
with(search_cost_agg, lines(year,spread_max,lwd=2, col = "blue",lty=2))
grid(nx=NULL,ny = NULL, col = "gray")


dev.off()



R_grid = seq(1.02,1.05,length.out = 1000)

plot(R_grid,h(R_grid, sigma = 1.27,beta = 0.99, tau=12))
plot(R_grid,phi(R_grid, sigma = 1.27,beta = 0.99, tau=1))

#-------------------------------------------------------------------------------->
#___ Stylized facts on industry concentration----
load(file = "/ofs/scratch-m1vly00/Data/SOD/sod_msa.rda")
load(file = "/ofs/scratch-m1vly00/Data/Y_9C/may9_final.rda")
time_deposits_hhi = may9[,list(time_less100K_hhi= 100*hhi(time_less100K),
                               time_less250K_hhi= 100*hhi(time_less250K)),by="date"]
time_deposits_hhi = time_deposits_hhi[order(time_deposits_hhi$date),]
time_deposits_hhi[time_deposits_hhi$time_less100K_hhi==0,time_less100K_hhi:=NA]
time_deposits_hhi[time_deposits_hhi$time_less250K_hhi==0,time_less250K_hhi:=NA]
time_deposits_hhi[,year:=year(date)]
time_deposits_hhi = time_deposits_hhi[,list(time_less100K_hhi=mean(time_less100K_hhi),
                                            time_less250K_hhi = mean(time_less250K_hhi)), by = "year"]
time_deposits_hhi[,small_time_hhi:= time_less100K_hhi]
time_deposits_hhi[is.na(time_less100K_hhi),small_time_hhi:= time_less250K_hhi]

sod_msa_agg = data.table(sod_msa)
sod_msa_agg[,bank_nmarkets:=length(unique(msa_code)),by = c("id_rssd_th","year")]
sod_msa_agg[,nbanks_agg:= as.numeric(length(unique(id_rssd_th))), by = "year"]
sod_msa_agg = sod_msa_agg[!is.na(sod_msa_agg$msa_code),
                          list(deposits = as.numeric(sum(dep_br,na.rm=T)),
                               hhi_deposits = hhi(dep_br),
                               nbanks = length(unique(id_rssd_th)),
                               bank_nmarkets = mean(bank_nmarkets,na.rm=T),
                               nbanks_agg =mean(nbanks_agg)),
                          by = c("msa_code","year")]
sod_agg = sod_msa_agg[,list(nbanks_wa = weighted.mean(nbanks,w=deposits,na.rm=T),
                            nbanks_med = median(nbanks, na.rm=T),
                            nbanks_agg = median(nbanks_agg),
                            bank_nmarkets = mean(bank_nmarkets,na.rm=T),
                            hhi_med = 100*median(hhi_deposits,na.rm=T),
                            hhi_wa = 100*weighted.mean(hhi_deposits,w=deposits,na.rm=T)
), by = "year"]

sod_agg = sod_agg[order(sod_agg$year)]
sod_agg = merge(sod_agg,time_deposits_hhi, by = "year", all.x=T)
names(sod_agg)
sod_agg = sod_agg[sod_agg$year <= 2016 & sod_agg$year>1996,]
dev.off()
par(mar=c(5,3,0.5,3))

pdf(file = paste0(figures_dir,"NBanks_msa.pdf"),width = w , height = w/h)

with(sod_agg, plot(year, nbanks_wa, ylim = c(0,80),
                   main = "",
                   xlab = "",
                   ylab = "",
                   type = "b",
                   pch = 1,
                   xaxt = "n",
                   cex = 2, cex.lab =1.7, cex.axis = 1.5,
                   #ylab = "Bank holding companies per market",
                   lwd=3, col = "red"))
axis(side = 1, sod_agg$year, at =sod_agg$year,cex.axis =1.7, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)

par(new=T)
with(sod_agg, lines(year, nbanks_med,
                    pch = 2,
                    type="b",lwd=3,col = "blue",lty=2))
par(new=T)
with(sod_agg, plot(year, nbanks_agg, type="b",col = "black",lty=2,
                   #ylab = "Aggregate number of bank holding companies",
                   xlab = "",
                   ylab = "",
                   pch = 3,lwd=3,
                   cex = 2, cex.lab =2, cex.axis = 1.7,
                   axes = FALSE, bty = "n"))

axis(side=4, at = pretty(range(sod_agg$nbanks_agg)), cex = 1.5)
legend("topright",c("Banks per market (deposit-weighted)",
                    "Banks per market (median)",
                    "Aggregate number of banks (right axis)"),
       col = c("red","blue","black"),
       bty = "n", cex  =1.2,lwd=3,
       lty = c(1,2,1),pch = c(1,2, 3))

dev.off()


sod_agg = sod_agg[order(sod_agg$year),]


pdf(file = paste0(figures_dir,"hhi_deposits.pdf"),width = w , height = w/h)

with(sod_agg, plot(year, hhi_wa, ylim = c(0,20),
                   main = "",
                   xlab = "",
                   ylab = "",
                   type = "b",
                   pch = 1,
                   xaxt = "n",
                   cex = 2, cex.lab =1.7, cex.axis = 1.5,
                   lwd=3, col = "red"))
axis(side = 1, sod_agg$year, at =sod_agg$year,cex.axis =1.7, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
par(new=T)
with(sod_agg, lines(year, hhi_med, type="b",
                    pch =2,
                    lwd=3,col = "blue",lty=2))
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
with(sod_agg, lines(year,small_time_hhi,
                    pch =3,

                    type="b", lwd=2,
                    ylim = c(0,20)))

abline(h=0)

legend("topleft",c("HHI all deposits (deposit-weighed)",
                   "HHI all deposits (median)",
                   "HHI insured time deposits"),
       col = c("red","blue","black"),
       bty = "n", cex  =1.2, lwd=3,
       lty = c(1,2,1),pch = c(1,2, 3))

dev.off()


# ----------------------------------------------------------------------------->
#_____ Measures of market concentration ----
dev.off()
par(mar=c(5,3,0.5,3))

pdf(file = paste0(figures_dir,"NBanks_msa.pdf"),width = 16 , height = 10)
with(sod_agg, plot(year, nbanks_wa, ylim = c(0,80),
                   main = "",
                   xlab = "",
                   ylab = "",
                   type = "b",
                   pch = 1,
                   xaxt = "n",
                   cex = 2, cex.lab =1.7, cex.axis = 1.5,
                   #ylab = "Bank holding companies per market",
                   lwd=3, col = "red"))
axis(side = 1, sod_agg$year, at =sod_agg$year,cex.axis =1.7, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)

par(new=T)
with(sod_agg, lines(year, nbanks_med,
                    pch = 2,
                    type="b",lwd=3,col = "blue",lty=2))
par(new=T)
with(sod_agg, plot(year, nbanks_agg, type="b",col = "black",lty=2,
                   #ylab = "Aggregate number of bank holding companies",
                   xlab = "",
                   ylab = "",
                   pch = 3,lwd=3,
                   cex = 2, cex.lab =2, cex.axis = 1.7,
                   axes = FALSE, bty = "n"))

axis(side=4, at = pretty(range(sod_agg$nbanks_agg)), cex = 1.5)
legend("topright",c("Banks per market (deposit-weighted)",
                    "Banks per market (median)",
                    "Aggregate number of banks (right axis)"),
       col = c("red","blue","black"),
       bty = "n", cex  =1.2,lwd=3,
       lty = c(1,2,1),pch = c(1,2, 3))

dev.off()


sod_agg = sod_agg[order(sod_agg$year),]


pdf(file = paste0(figures_dir,"hhi_deposits.pdf"),width = 10 , height = 16)
with(sod_agg, plot(year, hhi_wa, ylim = c(0,20),
                   main = "",
                   xlab = "",
                   ylab = "",
                   type = "b",
                   pch = 1,
                   xaxt = "n",
                   cex = 2, cex.lab =1.7, cex.axis = 1.5,
                   lwd=3, col = "red"))
axis(side = 1, sod_agg$year, at =sod_agg$year,cex.axis =1.7, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
par(new=T)
with(sod_agg, lines(year, hhi_med, type="b",
                    pch =2,
                    lwd=3,col = "blue",lty=2))
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
with(sod_agg, lines(year,small_time_hhi,
                    pch =3,

                    type="b", lwd=2,
                    ylim = c(0,20)))

abline(h=0)

legend("topleft",c("HHI all deposits (deposit-weighed)",
                   "HHI all deposits (median)",
                   "HHI insured time deposits"),
       col = c("red","blue","black"),
       bty = "n", cex  =1.2, lwd=3,
       lty = c(1,2,1),pch = c(1,2, 3))

dev.off()


# ----------------------------------------------------------------------------->
#_____ Counterfactuals: changes in number of competitors----
# market level data
F_xi_qvec_msa_2006 = F_xi_qvec_msa_y[F_xi_qvec_msa_y$year==2006,]
# aggregate data
F_xi_2006 = F_xi[F_xi$date=="2006-06-30"]
sigma = 1.276
tau = 1
beta = 0.99

##_____ 2006 data: weighted-mean market search cost: -----
hist(F_xi_qvec_msa_2006$R_min)
quantile(F_xi_qvec_msa_2006$R_min, prob = c(0.05,0.1,0.5))
mean(F_xi_qvec_msa_2006$R_min)
R_min.w = 1+1.5/100
hist(F_xi_qvec_msa_2006$R_max)
quantile(F_xi_qvec_msa_2006$R_max, prob = c(0.99,0.95))
R_max.w = quantile(F_xi_qvec_msa_2006$R_max, prob = 0.99)
R_tilde.w = as.numeric(quantile(F_xi_qvec_msa_2006$R_t, prob = 0.9))
xi = F_xi_2006$xi
delta1 = F_xi_2006$delta1_Rmin_wm[1]
xi_bps_Rmin = 10^4*F_xi_2006$xi/phi_prime(R_min.w)
q1 = weighted.mean(F_xi_qvec_msa_2006$qvec[F_xi_qvec_msa_2006$q_k==1],
                   w=F_xi_qvec_msa_2006$dep_msa[F_xi_qvec_msa_2006$q_k==1],
                   na.rm=T)
N = round(weighted.mean(F_xi_qvec_msa_2006$nbanks[F_xi_qvec_msa_2006$q_k==1],
                  w=F_xi_qvec_msa_2006$dep_msa[F_xi_qvec_msa_2006$q_k==1],
                  na.rm=T),0)
Fxi.w = F_xi_2006$Fxi_wm
Fxi_fit.w = splinefun(xi,
                      Fxi.w,
                        method = "monoH.FC")
Fxi_fit.w.vec= function(x){
  y = Fxi_fit.w(x)
  if(y>1){y=1}
  if(y<0){y=0}
  return(y)
}
Fxi_fit.w.vec = Vectorize(Fxi_fit.w.vec)
Fxi_fit.w.inv = inverse(Fxi_fit.w.vec, lower = min(xi), upper = max(xi))
Fxi_fit.w.inv = Vectorize(Fxi_fit.w.inv)
median.xi = Fxi_fit.w.inv(1/2)
plot(xi,Fxi_fit.w.vec(xi))
abline(v = median.xi, col = "red")
plot(xi_bps_Rmin,Fxi_fit.w.vec(xi))
abline(v = delta1, lwd=3,col = "red")
int_fun1.w = function(x){ x*(1- Fxi_fit.w.vec(x))}
int_fun2.w = function(x){ (1- Fxi_fit.w.vec(x))}


# compute variance/standard deviation of search cost
xi_sd.w = 10^4*sqrt(2*integrate(int_fun1.w,lower = 0, upper =10*max(xi) )$value-
               integrate(int_fun2.w,lower = 0, upper = 10*max(xi))$value^2)/phi_prime(R_min.w)
# compute mean
xi_mean.w = 10^4*integrate(int_fun2.w,lower = 0, upper =max(xi) )$value/phi_prime(R_min.w)

# median and interquartile range
xi_median = 10^4*Fxi_fit.w.inv(0.5)/phi_prime(R_min.w)
xi_p25 = 10^4*Fxi_fit.w.inv(0.25)/phi_prime(R_min.w)
xi_p75 = 10^4*Fxi_fit.w.inv(0.75)/phi_prime(R_min.w)


# equilibrium function
update_delta.w = function(delta){
  y = update_delta(delta,
                   R_min = R_min.w,
                   R_tilde = R_tilde.w,
                   Fxi_fit = Fxi_fit.w.vec)
  return(y)
}
equil_fun.w = function(delta){
  y = (delta-update_delta.w(delta))^2
  return(y)
}


# Use parallel computing to speed up computation
tictoc::tic()
registerDoParallel(30)
# loop over market structures with n = 2:60 competitors
market_sim_w2006 <- foreach(n = 2:60,.combine = "rbind",
                                 .errorhandling = "remove")%dopar%{
                                   N1 = n
                                   print(N1)
                                   delta0 = seq(from =xi[2], to=max(xi)*1.5, length.out = (n-1))
                                   if(n ==2){delta0 = max(xi)*1.5}
                                   delta0 = delta0[order(-delta0)]
                                   # critical search cost points
                                   # find fixed point
                                   delta_new.sol =nleqslv(delta0,equil_fun.w,
                                                          #method = "Newton",
                                                          #global = "pwldog",
                                                          control = list(trace=1, ftol = 1e-9))
                                   # Equilibrium search behavior
                                   delta_new=delta_new.sol$x
                                   qvec_new = qvec_delta(delta_new,Fxi_fit.w.vec)
                                   R_max_w.n = R_max_fn.solve(qvec_new,R_tilde.w,R_min.w)
                                   phi_Rmin = phi(R_min.w)
                                   # Equilibrium offer rate distribution
                                   F_R.w= function(R){return(F_R(R,qvec=qvec_new,
                                                                 R_min = R_min.w,
                                                                 R_tilde = R_tilde.w))}
                                   f_R.w = function(R){return(f_R(R,qvec=qvec_new,
                                                                  R_min = R_min.w,
                                                                  R_tilde = R_tilde.w))}
                                   F_R.w.inv = inverse(F_R.w, lower = R_min.w, upper = R_max_w.n)



                                   mean.fun.w = function(R){return(R*f_R.w(R))}
                                   mean.fun.w = Vectorize(mean.fun.w)
                                   sd.fun.w = function(R){return(R^2*f_R.w(R))}
                                   sd.fun.w = Vectorize(sd.fun.w)
                                   # end----

                                   R_mean.w = integrate(f =mean.fun.w,lower = R_min.w,upper = R_max_w.n)$value
                                   R_sd.w = sqrt(integrate(f =sd.fun.w,lower = R_min.w,upper = R_max_w.n)$value-
                                                   R_mean.w^2)
                                   R_median.w =F_R.w.inv(1/2)
                                   R_p25.w =F_R.w.inv(0.25)
                                   R_p75.w =F_R.w.inv(0.75)
                                   R_p5.w =F_R.w.inv(0.05)
                                   R_p95.w =F_R.w.inv(0.95)



                                   R_grid = seq(from=R_min.w,to=R_max_w.n, length.out = 200)
                                   plot(R_grid,F_R.w(R_grid),type = "b")
                                   plot(R_grid,f_R.w(R_grid),type = "b")
                                   abline(v = R_mean.w, col = "red")

                                   # consumer surplus
                                   cs.w= cs_exp(delta_new,R_min = R_min.w,
                                                   R_max = R_max_w.n,
                                                   R_tilde = R_tilde.w,
                                                   Fxi_fit = Fxi_fit.w.vec)

                                   # bank profits
                                   pi_exp.w = pi_exp(R=R_max_w.n,
                                                       qvec = qvec_new,
                                                       R_min =R_min.w,
                                                       R_max = R_max_w.n,
                                                       R_tilde = R_tilde.w)

                                   # export final data
                                   data_sim_m = data.frame(
                                     N1 = N1,
                                     q_k = 1:N1,
                                     R_tilde = R_tilde.w,
                                     R_max =R_max_w.n,
                                     R_min = R_min.w,
                                     pi =pi_exp.w,
                                     cs = cs.w,
                                     qvec = qvec_new,
                                     delta = c(delta_new,NA),
                                     xi = c(rev(delta_new),NA),
                                     xi_Rmin =10^4*c(rev(delta_new),NA)/phi_Rmin,
                                     R_mean.n = R_mean.w,
                                     R_median = R_median.w,
                                     R_p5 = R_p5.w,
                                     R_p95 = R_p95.w,
                                     R_p25 = R_p25.w,
                                     R_p75 = R_p75.w,
                                     R_sd.n = 10^4*R_sd.w,
                                     exit_deltaN = delta_new.sol$termcd
                                   )
                                   return(data_sim_m)
                                 }
stopImplicitCluster()
tictoc::toc()
# load(file = "/ofs/scratch-m1vly00/Data/Data_InSearchOfRiskFreeAsset/market_sim_w2006.60.rda")
market_sim_w2006 = data.table(market_sim_w2006)
with(market_sim_w2006[market_sim_w2006$q_k==1,], plot(N1, cs))
with(market_sim_w2006[market_sim_w2006$q_k==1,], plot(N1, pi))
with(market_sim_w2006[market_sim_w2006$q_k==1,], plot(N1, R_mean.n))
with(market_sim_w2006[market_sim_w2006$q_k==1,], plot(N1, qvec))
with(market_sim_w2006[market_sim_w2006$q_k==1,], plot(N1, R_sd.n/10^4))
with(market_sim_w2006[market_sim_w2006$q_k==1,], plot(N1, R_p5))
with(market_sim_w2006[market_sim_w2006$q_k==1,], plot(N1, R_p95))
with(market_sim_w2006[market_sim_w2006$q_k==1,], plot(N1, R_p95-R_p5))
with(market_sim_w2006[market_sim_w2006$q_k==1,], plot(N1, R_max))

# average number of firms sampled
market_sim_w2006[,n_avg:=sum(qvec*q_k,na.rm=T), by = "N1"]
summary(market_sim_w2006$n_avg)
market_sim_w2006[,N_max :=as.numeric(q_k==N1)]
market_sim_w2006[,cs.N2:=cs[N1==2][1]]
market_sim_w2006[,cs.pct:=cs/cs.N2]
market_sim_w2006[,pi.N2:=pi[N1==2][1]]
market_sim_w2006[,pi_total.pct:=N1*pi/(2*pi.N2)]
market_sim_w2006[,pi_per_bank.pct:=pi/(pi.N2)]


# expected offer rate given search----
registerDoParallel(36)
expected_R = foreach(N = 2:60,.combine = "rbind")%do%{
  print(N)
  data.N =market_sim_w2006[market_sim_w2006$N1==N,]
  F_R.w.N= function(R){return(F_R(R,qvec=data.N$qvec,
                                R_min = data.N$R_min[1],
                                R_tilde = data.N$R_tilde[1]))}
  f_R.w.N = function(R){return(f_R(R,qvec=data.N$qvec,
                                 R_min =  data.N$R_min[1],
                                 R_tilde = data.N$R_tilde[1]))}
  F_R.w.inv.N = inverse(F_R.w.N, lower = data.N$R_min[1], upper = data.N$R_max[1])

  expected_R.N <- foreach(k = 1:N,.combine = "sum")%dopar%{
    print(k)
    int_R_k = function(R){R*k*F_R.w.N(R)^(k-1)*f_R.w.N(R)}

    if(k < N){
      R_max_k = integrate(int_R_k, lower = R_min.w,upper = data.N$R_max[1] )$value
    }
    if(k == N){
    R_max_k = data.N$R_max[1]
      }
    return(R_max_k*data.N$qvec[data.N$q_k==k])
  }
  expected_R.N = data.frame(N = N,expected_R = expected_R.N,
                            R_max = data.N$R_max[1])
}

# average search costs
int_fun1.w = function(x){ x*(1- Fxi_fit.w.vec(x))}
int_fun2.w = function(x){ (1- Fxi_fit.w.vec(x))}

search_cost.w = foreach(N = 2:60,.combine = "rbind")%do%{
print(N)
data.N =market_sim_w2006[market_sim_w2006$N1==N,]
xi_random.Rmin.m = xi_random.Rmin
# change in the search cost distribution
Fxi_w = ecdf(xi_random.Rmin.m)
int_fun.w = function(x){ (1- Fxi_w(x))}

search_cost.N <- foreach(k = 1:N,.combine = "sum")%do%{
print(k)
if(k==N){
    delta_k = 0
    delta_k_1 = data.N$delta[data.N$q_k==(N-1)]
}
if(k>1 & k < N){
    delta_k = data.N$delta[data.N$q_k==k]
    delta_k_1 = data.N$delta[data.N$q_k==(k-1)]
}
if(k!=1){
xi_avg_k= 10^4*(k-1)*integrate(int_fun.w,lower = delta_k,
                               upper =delta_k_1)$value/phi_prime(data.N$R_min[1])
}
if(k==1){
  xi_avg_k = 0
}
return(xi_avg_k*data.N$qvec[data.N$q_k==k])
}
search_cost.n = data.frame(N = N, search_cost = search_cost.N)
return(search_cost.n)
}

search_cost.w = data.table(search_cost.w)
search_cost.w[,search_cost.pct:=search_cost/search_cost[N==2]]
with(search_cost.w, plot(N,search_cost))
setwd(final_data_dir)
save(search_cost.w, file = "search_cost.w.60.rda")
save(expected_R, file = "expected_R.60.rda")
#save(market_sim_w2006, file = "market_sim_w2006.60.rda")


#_____ Changes in the search cost distribution ----
names(F_xi_2006)
names(F_xi_qvec_msa_2006)
with(F_xi_qvec_msa_y[F_xi_qvec_msa_y$year==2007], plot(xi_mean,xi_sd))
summary(F_xi_qvec_msa_2006$xi_mean)
xi.Rmin= F_xi_2006$xi_Rmin
xi =F_xi_2006$xi
xi_factor = (xi.Rmin/xi)
xi_factor = max(xi_factor,na.rm=T)
xi_grid.Rmin = seq(0,max(xi.Rmin), length.out = 1000)
xi_grid = seq(0,max(xi), length.out = 1000)
Fxi_wm = F_xi_2006$Fxi_wm
Fxi.Rmin.spline = splinefun(xi.Rmin,
                        Fxi_wm,
                       method = "monoH.FC")
Fxi.Rmin= function(x){
  y = Fxi.Rmin.spline(x)
  if(y>1){y=1}
  if(y<0){y=0}
  return(y)
}
Fxi.Rmin = Vectorize(Fxi.Rmin)
Fxi.Rmin.inv = inverse.fun(Fxi.Rmin,lower = 0,upper = Inf)

Fxi.spline = splinefun(xi,Fxi_wm,
                            method = "monoH.FC")
Fxi= function(x){
  y = Fxi.spline(x)
  if(y>1){y=1}
  if(y<0){y=0}
  return(y)
}
Fxi = Vectorize(Fxi)
Fxi.inv = inverse.fun(Fxi,lower = 0,upper = Inf)

plot(xi_grid.Rmin,Fxi.Rmin(xi_grid.Rmin))
plot(xi_grid, Fxi(xi_grid))

xi_random.Rmin = random.function(n=10000,f = Fxi.Rmin,kind = "cumulative",
                                  lower = 0, upper = Inf)
summary(xi_random.Rmin)
xi_random = xi_random.Rmin/xi_factor
summary(xi_random)

##________ Changes in the mean keeping the variance fixed-----
delta_mu_xi = seq(0.5,-0.5,by = -0.05)
tictoc::tic()
registerDoParallel(30)
N = 15
delta0 = market_sim_w2006$delta[market_sim_w2006$N1==N]
delta0 = delta0[!is.na(delta0)]
search_cost_sim_w2006.mean <- foreach(mm = delta_mu_xi,.combine = "rbind",
                            .errorhandling = "remove")%dopar%{
                              print(mm)
                              xi_random.Rmin.m = xi_random.Rmin*(1+mm)
                              xi_random.m = xi_random*(1+mm)

                              # change in the search cost distribution
                              Fxi.m = ecdf(xi_random.m)
                              Fxi.m = Vectorize(Fxi.m)
                              Fxi.Rmin.m = ecdf(xi_random.Rmin.m)

                              # recompute the equilibrium
                              #delta0 = seq(from =xi[2], to=max(xi)*1.5, length.out = (N-1))
                              delta0 = delta0[order(-delta0)]

                              # equilibrium function
                              update_delta.m = function(delta){
                                y = update_delta(delta,
                                                 R_min = R_min.w,
                                                 R_tilde = R_tilde.w,
                                                 Fxi_fit = Fxi.m)
                                return(y)
                              }
                              equil_fun.m = function(delta){
                                y = (delta-update_delta.m(delta))^2
                                return(y)
                              }


                              # critical search cost points
                              # find fixed point
                              delta_new.sol =nleqslv(delta0,equil_fun.m,
                                                     #method = "Newton",
                                                     #global = "pwldog",
                                                     control = list(trace=1,btol = 1e-6, xtol = 1e-9,
                                                                    ftol = 1e-9))
                              # Equilibrium search behavior
                              delta_new=delta_new.sol$x
                              qvec_new = qvec_delta(delta_new,Fxi.m)
                              R_max_w.n = R_max_fn.solve(qvec_new,R_tilde.w,R_min.w)
                              phi_Rmin = phi(R_min.w)
                              # Equilibrium offer rate distribution
                              F_R.w= function(R){return(F_R(R,qvec=qvec_new,
                                                            R_min = R_min.w,
                                                            R_tilde = R_tilde.w))}
                              f_R.w = function(R){return(f_R(R,qvec=qvec_new,
                                                             R_min = R_min.w,
                                                             R_tilde = R_tilde.w))}
                              F_R.w.inv = inverse(F_R.w, lower = R_min.w, upper = R_max_w.n)



                              mean.fun.w = function(R){return(R*f_R.w(R))}
                              mean.fun.w = Vectorize(mean.fun.w)
                              sd.fun.w = function(R){return(R^2*f_R.w(R))}
                              sd.fun.w = Vectorize(sd.fun.w)
                              # end----

                              R_mean.w = integrate(f =mean.fun.w,lower = R_min.w,upper = R_max_w.n)$value
                              R_sd.w = sqrt(integrate(f =sd.fun.w,lower = R_min.w,upper = R_max_w.n)$value-
                                              R_mean.w^2)
                              R_median.w =F_R.w.inv(1/2)
                              R_p25.w =F_R.w.inv(0.25)
                              R_p75.w =F_R.w.inv(0.75)
                              R_p5.w =F_R.w.inv(0.05)
                              R_p95.w =F_R.w.inv(0.95)



                              R_grid = seq(from=R_min.w,to=R_max_w.n, length.out = 200)
                              plot(R_grid,F_R.w(R_grid),type = "b")
                              plot(R_grid,f_R.w(R_grid),type = "b")
                              abline(v = R_mean.w, col = "red")

                              # consumer surplus
                              cs.w= cs_exp(delta_new,R_min = R_min.w,
                                           R_max = R_max_w.n,
                                           R_tilde = R_tilde.w,
                                           Fxi_fit = Fxi.m)

                              # bank profits
                              pi_exp.w = pi_exp(R=R_max_w.n,
                                                qvec = qvec_new,
                                                R_min =R_min.w,
                                                R_max = R_max_w.n,
                                                R_tilde = R_tilde.w)

                              # export final data
                              data_sim_m = data.frame(
                                delta_mu = mm,
                                q_k = 1:N,
                                R_tilde = R_tilde.w,
                                R_max =R_max_w.n,
                                R_min = R_min.w,
                                pi =pi_exp.w,
                                cs = cs.w,
                                qvec = qvec_new,
                                delta = c(delta_new,NA),
                                xi = c(rev(delta_new),NA),
                                xi_Rmin =10^4*c(rev(delta_new),NA)/phi_Rmin,
                                R_mean.n = R_mean.w,
                                R_median = R_median.w,
                                R_p5 = R_p5.w,
                                R_p95 = R_p95.w,
                                R_p25 = R_p25.w,
                                R_p75 = R_p75.w,
                                R_sd.n = 10^4*R_sd.w,
                                exit_deltaN = delta_new.sol$termcd
                              )
                              return(data_sim_m)
                            }
stopImplicitCluster()
tictoc::toc()
setwd(final_data_dir)
expected_R.mean = foreach(mm = delta_mu_xi,.combine = "rbind")%do%{
  print(mm)
  data.N =search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$delta_mu==mm,]
  F_R.w.N= function(R){return(F_R(R,qvec=data.N$qvec,
                                  R_min = data.N$R_min[1],
                                  R_tilde = data.N$R_tilde[1]))}
  f_R.w.N = function(R){return(f_R(R,qvec=data.N$qvec,
                                   R_min =  data.N$R_min[1],
                                   R_tilde = data.N$R_tilde[1]))}
  F_R.w.inv.N = inverse(F_R.w.N, lower = data.N$R_min[1], upper = data.N$R_max[1])

  expected_R.N <- foreach(k = 1:N,.combine = "sum")%dopar%{
    print(k)
    int_R_k = function(R){R*k*F_R.w.N(R)^(k-1)*f_R.w.N(R)}

    if(k < N){
      R_max_k = integrate(int_R_k, lower = R_min.w,upper = data.N$R_max[1] )$value
    }
    if(k == N){
      R_max_k = data.N$R_max[1]
    }
    return(R_max_k*data.N$qvec[data.N$q_k==k])
  }
  expected_R.N = data.frame(delta_mu = mm,expected_R = expected_R.N,
                            R_max = data.N$R_max[1])
}

# average search costs

search_cost.mean = foreach(mm = delta_mu_xi,.combine = "rbind")%do%{
  print(mm)
  data.N =search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$delta_mu==mm,]
  N= length(data.N$q_k)
  xi_random.Rmin.m = xi_random.Rmin*(1+mm)
  # change in the search cost distribution
  Fxi.Rmin.m = ecdf(xi_random.Rmin.m)
  #Fxi.Rmin.m = Vectorize(Fxi.Rmin.m)
  plot(Fxi.Rmin.m)
  int_fun = function(x){return(1- Fxi.Rmin.m(x))}

  search_cost.N <- foreach(k = 1:N,.combine = "sum")%do%{
    print(k)
    if(k==N){
      delta_k = 0
      delta_k_1 = data.N$delta[data.N$q_k==(N-1)]
    }
    if(k>1 & k < N){
      delta_k = data.N$delta[data.N$q_k==k]
      delta_k_1 = data.N$delta[data.N$q_k==(k-1)]
    }
    if(k!=1){
    xi_avg_k= 10^4*(k-1)*integrate(int_fun,lower = delta_k,
                                   upper =delta_k_1)$value/phi_prime(data.N$R_min[1])
    }
    if(k==1){
      xi_avg_k = 0
    }
    return(xi_avg_k*data.N$qvec[data.N$q_k==k])
  }
  search_cost.n = data.frame(delta_mu_xi = mm, search_cost = search_cost.N)
  return(search_cost.n)
}

search_cost.mean = data.table(search_cost.mean)
with(search_cost.mean, plot(delta_mu_xi, search_cost))
table(search_cost_sim_w2006.mean$exit_deltaN)
save(search_cost_sim_w2006.mean, file = "search_cost_sim_w2006_mean.rda")
save(expected_R.mean, file= "expected_R_mean.rda")
save(search_cost.mean, file= "search_cost_mean.rda")


tictoc::tic()
registerDoParallel(30)
N = 15
delta0 = market_sim_w2006$delta[market_sim_w2006$N1==N]
delta0 = delta0[!is.na(delta0)]
search_cost.stats <- foreach(mm = delta_mu_xi,
                                      .errorhandling = "remove", .combine = "rbind")%dopar%{
                                        print(mm)
                                        xi_random.Rmin.m = xi_random.Rmin*(1+mm)

                                        Fxi.Rmin.m = ecdf(xi_random.Rmin.m)
                                        Fxi.Rmin.m = Vectorize(Fxi.Rmin.m)
                                        # summary statistics

                                       Fxi.Rmin.inv = inverse(Fxi.Rmin.m, lower = 0,
                                                              upper = max(xi_random.Rmin.m))
                                       Fxi.Rmin.inv = Vectorize(Fxi.Rmin.inv)
                                        int_fun1 = function(x){ x*(1- Fxi.Rmin.m(x))}
                                        int_fun2 = function(x){ (1- Fxi.Rmin.m(x))}


                                        xi_mean =  integrate(int_fun2,lower = 0,
                                                             upper = max(xi_random.Rmin.m))$value

                                        xi_sd = sqrt(2*integrate(int_fun1,lower = 0, upper =max(xi_random.Rmin.m))$value-
                                                       integrate(int_fun2,lower = 0,
                                                                 upper = max(xi_random.Rmin.m))$value^2)

                                        result.m = data.frame(
                                        xi_min = min(xi_random.Rmin.m),
                                        x_max  = max(xi_random.Rmin.m),
                                        xi_sd = xi_sd,
                                        xi_mean = xi_mean,
                                        delta_xi = mm)
                    return(result.m)
                                    }
stopImplicitCluster()
tictoc::toc()

xi_random.Rmin0 = xi_random.Rmin
grid_x0 = seq(min(xi_random.Rmin),max(xi_random.Rmin), length.out = 1000)
Fxi.Rmin0 = ecdf(xi_random.Rmin0)
Fxi.Rmin0 = Vectorize(Fxi.Rmin0)

plot(grid_x0,Fxi.Rmin0(grid_x0), type = "l", lwd =3, col = "black")
par(new=T)

search_cost.Fxi.mean <- for(mm in delta_mu_xi){
                              print(mm)
                              xi_random.Rmin.m = xi_random.Rmin*(1+mm)
                              grid_x = seq(min(xi_random.Rmin.m),max(xi_random.Rmin.m), length.out = 1000)

                              Fxi.Rmin.m = ecdf(xi_random.Rmin.m)
                              Fxi.Rmin.m = Vectorize(Fxi.Rmin.m)

                              if(mm<0){col = "red"}
                              if(mm==0){col = "black"}
                              if(mm>0){col = "blue"}

                              lines(grid_x,Fxi.Rmin.m(grid_x), type = "l", lwd =3, col = col)
                            }



##________ Mean-preserving spread (increase in the variance)----
# based interest rate equivalents

delta_std_xi = seq(0.5,-0.5,by = -0.05)
tictoc::tic()
registerDoParallel(30)
N = 15
delta0 = market_sim_w2006$delta[market_sim_w2006$N1==N]
delta0 = delta0[!is.na(delta0)]
search_cost_sim_w2006.std <- foreach(mm = delta_mu_xi,.combine = "rbind",
                                      .errorhandling = "remove")%dopar%{
                                        print(mm)
                                        xi_random.Rmin.m = xi_random.Rmin*(1+mm)
                                        xi_random.m = xi_random*(1+mm)

                                        # change in the search cost distribution
                                        Fxi.m = ecdf(xi_random.m)
                                        Fxi.m = Vectorize(Fxi.m)
                                        Fxi.Rmin.m = ecdf(xi_random.Rmin.m)

                                        # recompute the equilibrium
                                        #delta0 = seq(from =xi[2], to=max(xi)*1.5, length.out = (N-1))
                                        delta0 = delta0[order(-delta0)]

                                        # equilibrium function
                                        update_delta.m = function(delta){
                                          y = update_delta(delta,
                                                           R_min = R_min.w,
                                                           R_tilde = R_tilde.w,
                                                           Fxi_fit = Fxi.m)
                                          return(y)
                                        }
                                        equil_fun.m = function(delta){
                                          y = (delta-update_delta.m(delta))^2
                                          return(y)
                                        }


                                        # critical search cost points
                                        # find fixed point
                                        delta_new.sol =nleqslv(delta0,equil_fun.m,
                                                               #method = "Newton",
                                                               #global = "pwldog",
                                                               control = list(trace=1, ftol = 1e-9))
                                        # Equilibrium search behavior
                                        delta_new=delta_new.sol$x
                                        qvec_new = qvec_delta(delta_new,Fxi.m)
                                        R_max_w.n = R_max_fn.solve(qvec_new,R_tilde.w,R_min.w)
                                        phi_Rmin = phi(R_min.w)
                                        # Equilibrium offer rate distribution
                                        F_R.w= function(R){return(F_R(R,qvec=qvec_new,
                                                                      R_min = R_min.w,
                                                                      R_tilde = R_tilde.w))}
                                        f_R.w = function(R){return(f_R(R,qvec=qvec_new,
                                                                       R_min = R_min.w,
                                                                       R_tilde = R_tilde.w))}
                                        F_R.w.inv = inverse(F_R.w, lower = R_min.w, upper = R_max_w.n)



                                        mean.fun.w = function(R){return(R*f_R.w(R))}
                                        mean.fun.w = Vectorize(mean.fun.w)
                                        sd.fun.w = function(R){return(R^2*f_R.w(R))}
                                        sd.fun.w = Vectorize(sd.fun.w)
                                        # end----

                                        R_mean.w = integrate(f =mean.fun.w,lower = R_min.w,upper = R_max_w.n)$value
                                        R_sd.w = sqrt(integrate(f =sd.fun.w,lower = R_min.w,upper = R_max_w.n)$value-
                                                        R_mean.w^2)
                                        R_median.w =F_R.w.inv(1/2)
                                        R_p25.w =F_R.w.inv(0.25)
                                        R_p75.w =F_R.w.inv(0.75)
                                        R_p5.w =F_R.w.inv(0.05)
                                        R_p95.w =F_R.w.inv(0.95)



                                        R_grid = seq(from=R_min.w,to=R_max_w.n, length.out = 200)
                                        plot(R_grid,F_R.w(R_grid),type = "b")
                                        plot(R_grid,f_R.w(R_grid),type = "b")
                                        abline(v = R_mean.w, col = "red")

                                        # consumer surplus
                                        cs.w= cs_exp(delta_new,R_min = R_min.w,
                                                     R_max = R_max_w.n,
                                                     R_tilde = R_tilde.w,
                                                     Fxi_fit = Fxi_)

                                        # bank profits
                                        pi_exp.w = pi_exp(R=R_max_w.n,
                                                          qvec = qvec_new,
                                                          R_min =R_min.w,
                                                          R_max = R_max_w.n,
                                                          R_tilde = R_tilde.w)

                                        # export final data
                                        data_sim_m = data.frame(
                                          delta_mu = mm,
                                          q_k = 1:N,
                                          R_tilde = R_tilde.w,
                                          R_max =R_max_w.n,
                                          R_min = R_min.w,
                                          pi =pi_exp.w,
                                          cs = cs.w,
                                          qvec = qvec_new,
                                          delta = c(delta_new,NA),
                                          xi = c(rev(delta_new),NA),
                                          xi_Rmin =10^4*c(rev(delta_new),NA)/phi_Rmin,
                                          R_mean.n = R_mean.w,
                                          R_median = R_median.w,
                                          R_p5 = R_p5.w,
                                          R_p95 = R_p95.w,
                                          R_p25 = R_p25.w,
                                          R_p75 = R_p75.w,
                                          R_sd.n = 10^4*R_sd.w,
                                          exit_deltaN = delta_new.sol$termcd
                                        )
                                        return(data_sim_m)

                                      }
stopImplicitCluster()
tictoc::toc()
setwd(final_data_dir)
save(search_cost_sim_w2006.std, file = "search_cost_sim_w2006_std.rda")




#----------------------------------------------------------------------------->
#________ Plot results from counter-factuals----
#----------------------------------------------------------------------------->
# load(file = "market_sim_w2006.prelim.rda")
# load(file = "search_cost.w.rda")
# load(file = "expected_R.rda")
# Changes in the number of competitors----
dev.off()
par(mar=c(3,3,0.5,5))
pdf(file = paste0(figures_dir,"n_banks_impact_q1_qN.pdf"),
    width = 16 , height = 10)
with(market_sim_w2006[market_sim_w2006$q_k==1],
     plot(N1, qvec,
          main = "",
          xlab = "",
          ylab = "",
          type = "l",
          #pch = 1,
          ylim = c(0,0.62),
          xaxt = "n",
          cex = 2, cex.lab =2, cex.axis = 2,
          #ylab = "Bank holding companies per market",
          lwd=3, col = "red"))
par(new=T)
with(market_sim_w2006[market_sim_w2006$q_k==N1 & market_sim_w2006$N1>2,],
     lines(N1, qvec,
           lwd=3, col = "blue"))
with(market_sim_w2006[market_sim_w2006$q_k==2],
     lines(N1, qvec,
           lwd=3, col = "purple"))
axis(side = 1, 2:60, at =2:60,cex.axis =2, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
# par(new=T)
# with(search_cost.w,
#      plot(N, search_cost, type = "l",
#           main = "",
#           xlab = "",
#           ylab = "",
#           ylim = c(0,100),
#           cex = 2, cex.lab =2, cex.axis = 2,
#           axes = FALSE, bty = "n",
#           lwd=3, col = "black"))
# axis(side=4, at = pretty(range(search_cost.w$search_cost)), cex = 2, cex.axis=2)

legend("topright", c(TeX("$q_1$"),
                     TeX("$q_2$"),
                     TeX("$q_N$")),
       col = c("red","purple","blue"),
       bty = "n", cex  =2,lwd=3,
       lty = 1)

dev.off()

par(mar=c(3,3,0.5,5))
pdf(file = paste0(figures_dir,"n_banks_impact_search_costs.pdf"),
    width = 16 , height = 10)
with(market_sim_w2006[market_sim_w2006$q_k==1],
     plot(N1, n_avg,
          main = "",
          xlab = "",
          ylab = "",
          type = "l",
          #pch = 1,
          #ylim = c(0,0.7),
          xaxt = "n",
          cex = 2, cex.lab =2, cex.axis = 2,
          #ylab = "Bank holding companies per market",
          lwd=3, col = "red"))
axis(side = 1, 2:60, at =2:60,cex.axis =2, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
par(new=T)
with(search_cost.w,
     plot(N, search_cost, type = "l",
          main = "",
          xlab = "",
          ylab = "",
          ylim = c(0,25),
          cex = 2, cex.lab =2, cex.axis = 2,
          axes = FALSE, bty = "n",
          lwd=3, col = "black"))
axis(side=4, at = c(0,2,pretty(range(search_cost.w$search_cost))), cex = 2, cex.axis=2)

legend("bottomright", c("Banks sampled (left)",
                    "Total search costs (right)"),
       col = c("red","black"),
       bty = "n", cex  =2,lwd=3,
       lty = 1)

dev.off()

expected_R$expected_R
with(market_sim_w2006[market_sim_w2006$q_k==1], plot(R_max))
with(market_sim_w2006[market_sim_w2006$q_k==1], plot(R_p95-R_p5))
with(market_sim_w2006[market_sim_w2006$q_k==1], summary(R_p95-R_p5))
with(market_sim_w2006[market_sim_w2006$q_k==1], plot(cs.pct))
pmax(market_sim_w2006$cs.pct[market_sim_w2006$q_k==1])
market_sim_w2006$cs.pct[market_sim_w2006$q_k==1]

par(mar=c(3,3,0.5,5))

pdf(file = paste0(figures_dir,"n_banks_impact_cs_profits.pdf"),
    width = 16 , height = 10)
with(market_sim_w2006[market_sim_w2006$q_k==1],
     plot(N1, cs.pct,
          main = "",
          xlab = "",
          ylab = "",
          type = "l",
          #pch = 1,
          ylim = c(1,2.5),
          xaxt = "n",
          cex = 2, cex.lab =2, cex.axis = 2,
          #ylab = "Bank holding companies per market",
          lwd=3, col = "red"))
axis(side = 1, 2:60, at =2:60,cex.axis =2, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
abline(h=1)
par(new=T)
with(market_sim_w2006[market_sim_w2006$q_k==1],
     plot(N1, pi_total.pct,
          main = "",
          xlab = "",
          ylab = "",
          ylim = c(0,1),
          cex = 2, cex.lab =2, cex.axis = 2,
          axes = FALSE, bty = "n",type="l",
          lwd=3, col = "blue"))
par(new=T)
with(market_sim_w2006[market_sim_w2006$q_k==1],
     lines(N1, pi_per_bank.pct, col="blue",lty=2, lwd=3))

axis(side=4, at = seq(0,1,by =0.1), cex = 2, cex.axis=2)

legend(x=40,y = 0.3,c("Consumer surplus",
                      "Total bank profits",
                      "Profits per bank"),
       col = c("red","blue","blue"),
       bty = "n", cex  =2,lwd=3,
       lty = c(1,1,2))

dev.off()

pdf(file = paste0(figures_dir,"n_banks_impact_rates.pdf"),
    width = 16 , height = 10)

with(market_sim_w2006[market_sim_w2006$q_k==1],
     plot(N1, 100*(R_max-1),
          main = "",
          xlab = "",
          ylab = "",
          type = "l",
          ylim = c(1,6.5),
          xaxt = "n",
          cex = 2, cex.lab =2, cex.axis = 2,
          #ylab = "Bank holding companies per market",
          lwd=3, col = "purple"))
axis(side = 1, 2:60, at =2:60,cex.axis =2, las = 2)
abline(h = 100*(market_sim_w2006$R_min[1]-1),lwd=3, col= "green")
par(new=T)
with(market_sim_w2006[market_sim_w2006$q_k==N1],
     lines(N1, 100*(R_p95-1),
           lwd=1, col = "darkgray"))

with(market_sim_w2006[market_sim_w2006$q_k==N1],
     lines(N1, 100*(R_p5-1),
           lwd=1, col = "darkgray"))
with(market_sim_w2006[market_sim_w2006$q_k==N1],
     polygon(c(N1, rev(N1)),
             c(100*(R_p95-1),rev(100*(R_p5-1))),
             col = "grey", border = "grey70"))

with(market_sim_w2006[market_sim_w2006$q_k==N1],
     lines(N1, 100*(R_p25-1),
           #cex = 2, cex.lab =1.7, cex.axis = 1.5,
           #axes = FALSE, bty = "n",
           lwd=1, col = "blue"))
with(market_sim_w2006[market_sim_w2006$q_k==N1],
     lines(N1, 100*(R_p75-1),
           #cex = 2, cex.lab =1.7, cex.axis = 1.5,
           #axes = FALSE, bty = "n",
           lwd=1, col = "blue"))

with(market_sim_w2006[market_sim_w2006$q_k==N1],
     polygon(c(N1, rev(N1)),
             c(100*(R_p75-1),rev(100*(R_p25-1))),
             col = "grey37", border = "grey37"))

with(expected_R,
     lines(N, 100*(expected_R-1),
           lwd=3, col = "black"))

legend("topleft", c("Average expected rate given search",
                    TeX("$R_{max}$"),
                    TeX("$R_{min}$"),
                    "25/75th",
                    "5/95th"
),
cex = 1.7,
col = c("black","purple","green",
        "grey37",
        "grey70"),
lwd = c(3,3,3,10,10), bty = "n",
# bg = NULL,
#box.lwd = 0,
lty = 1)


dev.off()


# Changes in the search cost distribution----
search_cost_sim_w2006.mean = data.table(search_cost_sim_w2006.mean)
search_cost_sim_w2006.mean[,ind_N:=as.numeric(q_k==max(q_k))]
table(search_cost_sim_w2006.mean$delta_mu)
table(search_cost_sim_w2006.mean$exit_deltaN[search_cost_sim_w2006.mean$delta_mu==-0.5])
table(search_cost_sim_w2006.mean$exit_deltaN[search_cost_sim_w2006.mean$delta_mu==0.5])
table(search_cost_sim_w2006.mean$exit_deltaN[search_cost_sim_w2006.mean$delta_mu==-0.4])
table(search_cost_sim_w2006.mean$exit_deltaN[as.character(search_cost_sim_w2006.mean$delta_mu)==-0.4])


dev.off()
par(mar=c(5,5,0.5,0.1))
pdf(file = paste0(figures_dir,"search_cost_mu_q1_qN.pdf"),
    width = 16 , height = 10)
with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$q_k==1,],
     plot(delta_mu, qvec,
          main = "",
          xlab = "",
          ylab = "",
          type = "l",
          #pch = 1,
          ylim = c(0,0.7),
          xaxt = "n",
          cex = 2, cex.lab =2, cex.axis = 2,
          #ylab = "Bank holding companies per market",
          lwd=3, col = "red"))
axis(side = 1, (1+delta_mu_xi), at =delta_mu_xi,cex.axis =2, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
abline(v=0, lwd = 4, col = "darkgray")
par(new=T)
with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$q_k==2,],
     lines(delta_mu, qvec,
           lwd=3, col = "purple"))
with(search_cost_sim_w2006.mean[which(search_cost_sim_w2006.mean$q_k==15),],
     lines(delta_mu, qvec,
           lwd=3, col = "blue"))
legend("topleft", c(TeX("$q_1$"),
                     TeX("$q_2$"),
                     TeX("$q_N$")),
       col = c("red","purple","blue"),
       bty = "n", cex  =2,lwd=3,
       lty = 1)
dev.off()
search_cost_sim_w2006.mean$qvec[which(search_cost_sim_w2006.mean$q_k==15)]

par(mar=c(5,5,0.5,0.1))
pdf(file = paste0(figures_dir,"search_cost_mu_rates.pdf"),
    width = 16 , height = 10)
with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$q_k==1],
     plot(delta_mu, 100*(R_max-1),
          main = "",
          xlab = "",
          ylab = "",
          type = "l",
          ylim = c(1,6.5),
          xaxt = "n",
          cex = 2, cex.lab =2, cex.axis = 2,
          #ylab = "Bank holding companies per market",
          lwd=3, col = "purple"))
axis(side = 1, (1+delta_mu_xi), at =delta_mu_xi,cex.axis =2, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
abline(v=0, lwd = 4, col = "darkgray")
par(new=T)

with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$ind_N==1],
     lines(delta_mu, 100*(R_p95-1),
           lwd=1, col = "darkgray"))

with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$ind_N==1],
     lines(delta_mu, 100*(R_p5-1),
           lwd=1, col = "darkgray"))
with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$ind_N==1],
     polygon(c(delta_mu, rev(delta_mu)),
             c(100*(R_p95-1),rev(100*(R_p5-1))),
             col = "grey", border = "grey70"))

with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$ind_N==1],
     lines(delta_mu, 100*(R_p25-1),
           #cex = 2, cex.lab =1.7, cex.axis = 1.5,
           #axes = FALSE, bty = "n",
           lwd=1, col = "blue"))
with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$ind_N==1],
     lines(delta_mu, 100*(R_p75-1),
           #cex = 2, cex.lab =1.7, cex.axis = 1.5,
           #axes = FALSE, bty = "n",
           lwd=1, col = "blue"))

with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$ind_N==1],
     polygon(c(delta_mu, rev(delta_mu)),
             c(100*(R_p75-1),rev(100*(R_p25-1))),
             col = "grey37", border = "grey37"))

with(expected_R.mean,
     lines(delta_mu, 100*(expected_R-1),
           lwd=3, col = "black"))

legend("topright", c("Average expected rate given search",
                    TeX("$R_{max}$"),

                    "25/75th",
                    "5/95th"
),
cex = 1.7,
col = c("black","purple",
        "grey37",
        "grey70"),
lwd = c(3,3,10,10), bty = "n",
# bg = NULL,
#box.lwd = 0,
lty = 1)


dev.off()


search_cost_sim_w2006.mean[,cs.pct:= cs/cs[delta_mu==0]]
search_cost_sim_w2006.mean[,pi_total.pct:= pi/pi[delta_mu==0]]

par(mar=c(3,3,0.5,0.5))

pdf(file = paste0(figures_dir,"search_cost_mu_cs_profits.pdf"),
    width = 16 , height = 10)
with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$ind_N==1],
     plot(delta_mu, cs.pct,
          main = "",
          xlab = "",
          ylab = "",
          type = "l",
          #pch = 1,
          ylim = c(0.2,1.7),
          xaxt = "n",
          cex = 2, cex.lab =2, cex.axis = 2,
          #ylab = "Bank holding companies per market",
          lwd=3, col = "red"))
axis(side = 1, (1+delta_mu_xi), at =delta_mu_xi,cex.axis =2, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
abline(v=0, lwd = 4, col = "darkgray")
par(new=T)

with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$ind_N==1],
     lines(delta_mu, pi_total.pct,
          main = "",
          xlab = "",
          ylab = "",
          #ylim = c(0,1),
          cex = 2, cex.lab =2, cex.axis = 2,
          #axes = FALSE,
          #bty = "n",
          type="l",
          lwd=3, col = "blue"))
par(new=T)
with(market_sim_w2006[market_sim_w2006$q_k==1],
     lines(N1, pi_per_bank.pct, col="blue",lty=2, lwd=3))


legend("topleft",c("Consumer surplus",
                      "Total bank profits"),
       col = c("red","blue"),
       bty = "n", cex  =2,lwd=3,
       lty = c(1,1))

dev.off()


search_cost_sim_w2006.mean[,n_avg:=sum(qvec*q_k,na.rm=T), by = "delta_mu"]
summary(search_cost_sim_w2006.mean$n_avg)

par(mar=c(5,5,0.5,5))
pdf(file = paste0(figures_dir,"search_cost_mu_search_cost.pdf"),
    width = 16 , height = 10)
with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$ind_N==1],
     plot(delta_mu, n_avg,
          main = "",
          xlab = "",
          ylab = "",
          type = "l",
          #pch = 1,
          #ylim = c(0,0.7),
          xaxt = "n",
          cex = 2, cex.lab =2, cex.axis = 2,
          #ylab = "Bank holding companies per market",
          lwd=3, col = "red"))
axis(side = 1, (1+delta_mu_xi), at =delta_mu_xi,cex.axis =2, las = 2)
grid(ny = NULL, nx = NULL, col ="darkgray",lwd=2)
abline(v=0, lwd = 4, col = "darkgray")
par(new=T)
with(search_cost.mean,
     plot(delta_mu_xi, search_cost, type = "l",
          main = "",
          xlab = "",
          ylab = "",
          ylim = c(10,16),
          cex = 2, cex.lab =2, cex.axis = 2,
          axes = FALSE, bty = "n",
          lwd=3, col = "black"))
axis(side=4, at = c(10,pretty(range(search_cost.mean$search_cost))), cex = 2, cex.axis=2)

legend("bottomright", c("Banks sampled (left)",
                    "Total search costs (right)"),
       col = c("red","black"),
       bty = "n", cex  =2,lwd=3,
       lty = 1)

dev.off()



with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$q_k==1,],
     plot(delta_mu,qvec))


with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$q_k==2,],
     plot(delta_mu,qvec))



with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$q_k==2,],
     plot(delta_mu,cs))

with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$q_k==2,],
     plot(delta_mu,R_median))

with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$q_k==2,],
     plot(delta_mu,pi))

with(search_cost_sim_w2006.mean[search_cost_sim_w2006.mean$q_k==2,],
     plot(delta_mu,R_mean.n))





# ----------------------------------------------------------------------------->


