#------------------------------------------------------------------------------>
# Name: iMN_analysis.R
# Last date modified: May 2015
# Description: Analysis on bank-affiiated money market mutual funds
#------------------------------------------------------------------------------>
rm(list = ls())
#------------------------------------------------------------------------------>
# Set directories
#------------------------------------------------------------------------------>
root_dir <- "./Data/"
code_dir = "./Programs/"
figures_dir = "./Results/Figures/"
figures_dir = "/InSearchRisklessAsset/Results/Figures/"
tables_dir = "./InSearchRisklessAsset/Results/Tables/"
#------------------------------------------------------------------------------>
# Load libraries----
#------------------------------------------------------------------------------>>
require(foreign)
require(Matrix)
require(Rcpp)
library(boot, lib.loc = lib_loc)
library(MASS)
library(iterators, lib.loc = lib_loc)
library(foreach, lib.loc = lib_loc)
library(snow, lib.loc = lib_loc)
library(doSNOW, lib.loc = lib_loc)
library(lubridate, lib.loc = lib_loc)
library(zoo, lib.loc = lib_loc)
library(plyr, lib.loc = lib_loc)
require(reshape2)
library(bdsmatrix, lib.loc = lib_loc)
library(Formula, lib.loc = lib_loc)
library(sandwich, lib.loc = lib_loc)
library(plm, lib.loc = lib_loc)
library(lubridate, lib.loc = lib_loc)
library(data.table, lib.loc = lib_loc)
require(stargazer)
#------------------------------------------------------------------------------>
# Define functions ----
#------------------------------------------------------------------------------>
as.numeric.factor <- function(x){as.numeric(levels(x))[x]}
as.character.factor <- function(x){as.character(levels(x))[x]}
source(paste0(code_dir,"Tools/PlottingFunctions.R"))
source(paste0(code_dir,"Tools/CollapseFunctions.R"))

agg_to_nb_sponsor = function(x, cond){
  y <-  x[cond ,
          list(aum = sum(aum_fund, na.rm = T),
               APY = weighted.mean(APY_mmf, w = aum_fund, na.rm = T),
               flow = weighted.mean(flow, w = aum_fund, na.rm = T),
               flow_abs = sum(flow_abs, na.rm = T),
               expense = weighted.mean(expense, w = aum_fund, na.rm = T),
               maturity = weighted.mean(maturity, w = aum_fund, na.rm = T),
               maturity_7d = weighted.mean(maturity_7d, w = aum_fund, na.rm = T),
               min_inv = weighted.mean(min_inv, w = aum_fund, na.rm = T),
               # asset allocation
               ABCP = weighted.mean(ABCP, w = aum_fund, na.rm = T),
               DBO = weighted.mean(DBO, w = aum_fund, na.rm = T),
               FRNS = weighted.mean(FRNS, w = aum_fund, na.rm = T),
               CP_tier1 = weighted.mean(CP_tier1, w = aum_fund, na.rm = T),
               Repos = weighted.mean(Repos, w = aum_fund, na.rm = T),
               FBO = weighted.mean(FBO, w = aum_fund, na.rm = T),
               CP_tier2  = weighted.mean(CP_tier2, w = aum_fund, na.rm = T),
               TD = weighted.mean(TD, w = aum_fund, na.rm = T),
               USOther = weighted.mean(USOther, w = aum_fund, na.rm = T),
               USTreasury = weighted.mean(USTreasury, w = aum_fund, na.rm = T),
               age = weighted.mean(age, w = aum_fund, na.rm = T),
               # fund counts
               n_distr = length(unique(FundDistributor)),
               n_PortMan = length(unique(PortfolioManager)),
               n_cust = length(unique(custodial_name)),
               n_advisers = length(unique(FundAdviser)),
               n_funds = length(unique(id_fund)),
               n_mcfunds = length(unique(id_mcfund)),
               n_complex = length(unique(id_complex))),
          by = c("id_complex","FundComplex","date")]
  return(y)}

agg_to_bank_sponsor <- function(x, cond){
  y <- x[cond,
         list(aum = sum(aum_fund, na.rm = T),
              APY = weighted.mean(APY_mmf, w = aum_fund, na.rm = T),
              flow = weighted.mean(flow, w = aum_fund, na.rm = T),
              flow_abs = sum(flow_abs, na.rm = T),
              expense = weighted.mean(expense, w = aum_fund, na.rm = T),
              maturity = weighted.mean(maturity, w = aum_fund, na.rm = T),
              maturity_7d = weighted.mean(maturity_7d, w = aum_fund, na.rm = T),
              min_inv = weighted.mean(min_inv, w = aum_fund, na.rm = T),
              # asset allocation
              ABCP = weighted.mean(ABCP, w = aum_fund, na.rm = T),
              DBO = weighted.mean(DBO, w = aum_fund, na.rm = T),
              FRNS = weighted.mean(FRNS, w = aum_fund, na.rm = T),
              CP_tier1 = weighted.mean(CP_tier1, w = aum_fund, na.rm = T),
              Repos = weighted.mean(Repos, w = aum_fund, na.rm = T),
              FBO = weighted.mean(FBO, w = aum_fund, na.rm = T),
              CP_tier2  = weighted.mean(CP_tier2, w = aum_fund, na.rm = T),
              TD = weighted.mean(TD, w = aum_fund, na.rm = T),
              USOther = weighted.mean(USOther, w = aum_fund, na.rm = T),
              USTreasury = weighted.mean(USTreasury, w = aum_fund, na.rm = T),
              age = weighted.mean(age, w = aum_fund, na.rm = T),
              # fund counts
              n_distr = length(unique(FundDistributor)),
              n_PortMan = length(unique(PortfolioManager)),
              n_cust = length(unique(custodial_name)),
              n_advisers = length(unique(FundAdviser)),
              n_funds = length(unique(id_fund)),
              n_mcfunds = length(unique(id_mcfund)),
              n_complex = length(unique(id_complex))),
         by = c("id_rssd_th","date")]
  return(y)}

agg_stat = function(x, cond){
  y <-  x[cond ,
          list(aum = sum(aum_fund, na.rm = T),
               APY = weighted.mean(APY_mmf, w = aum_fund, na.rm = T),
               flow = weighted.mean(flow, w = aum_fund, na.rm = T),
               flow_abs = sum(flow_abs, na.rm = T),
               expense = weighted.mean(expense, w = aum_fund, na.rm = T),
               maturity = weighted.mean(maturity, w = aum_fund, na.rm = T),
               maturity_7d = weighted.mean(maturity_7d, w = aum_fund, na.rm = T),
               min_inv = weighted.mean(min_inv, w = aum_fund, na.rm = T),
               # asset allocation
               ABCP = weighted.mean(ABCP, w = aum_fund, na.rm = T),
               DBO = weighted.mean(DBO, w = aum_fund, na.rm = T),
               FRNS = weighted.mean(FRNS, w = aum_fund, na.rm = T),
               CP_tier1 = weighted.mean(CP_tier1, w = aum_fund, na.rm = T),
               Repos = weighted.mean(Repos, w = aum_fund, na.rm = T),
               FBO = weighted.mean(FBO, w = aum_fund, na.rm = T),
               CP_tier2  = weighted.mean(CP_tier2, w = aum_fund, na.rm = T),
               TD = weighted.mean(TD, w = aum_fund, na.rm = T),
               USOther = weighted.mean(USOther, w = aum_fund, na.rm = T),
               USTreasury = weighted.mean(USTreasury, w = aum_fund, na.rm = T),
               age = weighted.mean(age, w = aum_fund, na.rm = T),
               # fund counts
               n_banks = length(unique(id_rssd_th)),
               n_distr = length(unique(FundDistributor)),
               n_PortMan = length(unique(PortfolioManager)),
               n_cust = length(unique(custodial_name)),
               n_advisers = length(unique(FundAdviser)),
               n_funds = length(unique(id_fund)),
               n_mcfunds = length(unique(id_mcfund)),
               n_complex = length(unique(id_complex))),
          by = c("date")]
  return(y)}

distr_time <- function(x, var, w = "aum_fund", by = "date"){
  eval(parse(text = paste0("y = x[, list(expense_med = median(expense, na.rm =T),"
                           ,var,"_wm = weighted.mean(expense, w =", w, ", na.rm = T),"
                           ,var,"_p1= quantile(expense, prob= 0.01, na.rm = T),"
                           ,var, "_p5= quantile(expense, prob= 0.05, na.rm = T),"
                           ,var, "_p25= quantile(expense, prob= 0.25, na.rm = T),"
                           ,var,"_p75= quantile(expense, prob= 0.75, na.rm = T),"
                           ,var,"_p95= quantile(expense, prob= 0.95, na.rm = T),"
                           ,var,"_p99= quantile(expense, prob= 0.99, na.rm = T)),
                           by =",by,"]")))
  y = y[order(y$date), ]
  y = as.data.frame(y)
  return(y)
}

# plotting function -- plot distribution

plot_ExpDistr <- function(x,ylab = "", xlab ="", main = ""){
  with(x, plot(date,expense_p99, col = "grey", lwd = 2,type = "l", ylim = c(0,2),
                         ylab = ylab,
                         xlab = xlab,
                         main= main))
  par(new = T)
  with(x, lines(date,expense_p1, col = "grey", lwd = 2,type = "l" ))
  with(x, lines(date,expense_p5, col = "red", lwd = 2,type = "l" ))
  with(x, lines(date,expense_p95, col = "red", lwd = 2,type = "l" ))
  with(x, lines(date,expense_p25, col = "black", lwd = 1,type = "l" ))
  with(x, lines(date,expense_p75, col = "black", lwd = 1,type = "l" ))
  with(x, lines(date,expense_wm, col = "blue", lwd = 2,type = "l" ))
  #axis(1, 1:10)
  legend("topright", c("p1/99", "p5/95", "p25/75", "mean(w)"),
         lty=c(1,1,1,1),cex = 0.5, col = c("grey", "red", "black", "blue"))

  grid(nx = 10, ny = 10, col = "lightgray", lty = "dotted",
       lwd = par("lwd"), equilogs = TRUE)

}


#------------------------------------------------------------------------------>
setwd(root_dir)
#------------------------------------------------------------------------------>
load(file = "./iMN/iMN.rda" )
#------------------------------------------------------------------------------>
iMN = as.data.table(iMN)

# Time series variation of the distribution of expense ratios
Expense_all = distr_time(iMN, var = "expense", w = "aum_fund", by = "date")
Expense_inst_all = distr_time(iMN[iMN$InvestorType =="Institutional",], var = "expense", w = "aum_fund", by = "date")
Expense_inst_prime = distr_time(iMN[iMN$CategoryHistorical =="Prime Institutional",], var = "expense", w = "aum_fund", by = "date")
Expense_inst_gov = distr_time(iMN[iMN$CategoryHistorical =="Govt Institutional",], var = "expense", w = "aum_fund", by = "date")

Expense_ret_all = distr_time(iMN[iMN$InvestorType =="Retail",], var = "expense", w = "aum_fund", by = "date")
Expense_ret_prime = distr_time(iMN[iMN$CategoryHistorical =="Prime Retail",], var = "expense", w = "aum_fund", by = "date")
Expense_ret_gov = distr_time(iMN[iMN$CategoryHistorical =="Government Retail" ,], var = "expense", w = "aum_fund", by = "date")


## Bank affiliated funds----
Expense_bank = distr_time(iMN[!is.na(iMN$id_rssd_th),], var = "expense", w = "aum_fund", by = "date")
Expense_inst_bank = distr_time(iMN[!is.na(iMN$id_rssd_th) & iMN$InvestorType =="Institutional",],
                               var = "expense", w = "aum_fund", by = "date")
Expense_inst_prime_bank = distr_time(iMN[!is.na(iMN$id_rssd_th) & iMN$CategoryHistorical =="Prime Institutional",], var = "expense", w = "aum_fund", by = "date")
Expense_inst_gov_bank = distr_time(iMN[!is.na(iMN$id_rssd_th) & iMN$InvestorType =="Govt Institutional",], var = "expense", w = "aum_fund", by = "date")

Expense_ret_bank = distr_time(iMN[!is.na(iMN$id_rssd_th) & iMN$InvestorType =="Retail",], var = "expense", w = "aum_fund", by = "date")
Expense_ret_prime_bank = distr_time(iMN[!is.na(iMN$id_rssd_th) & iMN$CategoryHistorical =="Prime Retail",], var = "expense", w = "aum_fund", by = "date")
Expense_ret_gov_bank = distr_time(iMN[!is.na(iMN$id_rssd_th) & iMN$CategoryHistorical =="Government Retail" ,],
                                      var = "expense", w = "aum_fund", by = "date")

## non-bank funds
Expense_nonbank = distr_time(iMN[is.na(iMN$id_rssd_th),], var = "expense", w = "aum_fund", by = "date")
Expense_inst_nonbank = distr_time(iMN[is.na(iMN$id_rssd_th) & iMN$InvestorType =="Institutional",], var = "expense", w = "aum_fund", by = "date")
Expense_inst_prime_nonbank = distr_time(iMN[is.na(iMN$id_rssd_th) & iMN$CategoryHistorical =="Prime Institutional",], var = "expense", w = "aum_fund", by = "date")
Expense_inst_gov_nonbank = distr_time(iMN[is.na(iMN$id_rssd_th) & iMN$InvestorType =="Govt Institutional",], var = "expense", w = "aum_fund", by = "date")

Expense_ret_nonbank = distr_time(iMN[is.na(iMN$id_rssd_th) & iMN$InvestorType =="Retail",], var = "expense", w = "aum_fund", by = "date")
Expense_ret_prime_nonbank = distr_time(iMN[is.na(iMN$id_rssd_th) & iMN$CategoryHistorical =="Prime Retail",], var = "expense", w = "aum_fund", by = "date")
Expense_ret_gov_nonbank = distr_time(iMN[is.na(iMN$id_rssd_th) & iMN$InvestorType =="Government Retail" ,], var = "expense", w = "aum_fund", by = "date")


Expense_wm = data.frame(date = Expense_all$date,
                        expense_all = Expense_all[,c("expense_wm")],
                        expense_bank = Expense_inst_bank[,c("expense_wm")],
                        expense_inst = Expense_inst_all$expense_wm,
                        expense_ret = Expense_ret_all$expense_wm,
                        expense_inst_prime = Expense_inst_prime$expense_wm,
                        expense_ret_prime = Expense_ret_prime$expense_wm,
                        expense_inst_gov = Expense_inst_gov$expense_wm,
                        expense_ret_gov = Expense_ret_gov$expense_wm,
                        expense_inst_bank  = Expense_inst_bank$expense_wm,
                        expense_inst_nonbank  = Expense_inst_nonbank$expense_wm,
                        expense_ret_bank  = Expense_ret_bank$expense_wm,
                        expense_ret_nonbank  = Expense_ret_nonbank$expense_wm,
                        expense_inst_prime_bank = Expense_inst_prime_bank$expense_wm,
                        expense_inst_prime_nonbank = Expense_inst_prime_nonbank$expense_wm)

# By Distribution channel
# Direct (No Load)
Expense_ret_direct = distr_time(iMN[iMN$PrimaryDistributionChannels =="Direct (No Load)" &
                 iMN$InvestorType =="Retail",], var = "expense", w = "aum_fund", by = "date")




# Expense ratio for a point in time by distribution channel----
## 2005
iMN = data.table(iMN)
Inst_expense_byDistr = iMN[iMN$InvestorType =="Institutional" & iMN$date == "2005-12-27",
                           list(expense = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                n_funds = length(unique(id_fund)),
                                aum = sum(aum_fund, na.rm = T)/10^6),
                           by ="PrimaryDistributionChannels" ]


Ret_expense_byDistr = iMN[iMN$InvestorType =="Retail" & iMN$date == "2005-12-27",
                          list(expense = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                               n_funds = length(unique(id_fund)),
                               aum = sum(aum_fund, na.rm = T)/10^6),
                          by ="PrimaryDistributionChannels" ]

print(Inst_expense_byDistr)
print(Ret_expense_byDistr)

stargazer(Ret_expense_byDistr[order(-Ret_expense_byDistr$aum),], summary=FALSE)
stargazer(Inst_expense_byDistr[order(-Inst_expense_byDistr$aum),], summary=FALSE)


# Expense rations by distributional channel and bank-affiliation----
iMN=data.table(iMN)
iMN[, bank_ind := (!is.na(iMN$id_rssd_th))]
iMN$PrimaryDistributionChannels[iMN$PrimaryDistributionChannels %in% c("Broker (Full Service)",
                                                                       "Broker (Discount)")] = "Broker"

Retail_expense_bank_byDistr_2006 = iMN[iMN$InvestorType =="Retail" & iMN$date == "2006-06-27" &
                                         iMN$bank_ind==T,
                          list(bank_n_funds = length(unique(id_fund)),
                               bank_aum = sum(aum_fund, na.rm = T)/10^6,
                               bank_expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                               bank_expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                               bank_expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                          by ="PrimaryDistributionChannels" ]


Retail_expense_nonbank_byDistr_2006 = iMN[iMN$InvestorType =="Retail" & iMN$date == "2006-06-27" &
                                            iMN$bank_ind==F,
                                       list(n_funds = length(unique(id_fund)),
                                            aum = sum(aum_fund, na.rm = T)/10^6,
                                            expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                            expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                                            expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                                       by ="PrimaryDistributionChannels" ]

Retail_expense_byaffiliation_2006 = iMN[iMN$InvestorType =="Retail" & iMN$date == "2006-06-27",
                                       list(total_n_funds = length(unique(id_fund)),
                                            total_aum = sum(aum_fund, na.rm = T)/10^6,
                                            total_expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                            total_expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                                            total_expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                                       by ="bank_ind" ]


Retail_expense_distr_2006 = merge(Retail_expense_bank_byDistr_2006,Retail_expense_nonbank_byDistr_2006,
                                  by = "PrimaryDistributionChannels", all = F)


Retail_expense_distr_2006=Retail_expense_distr_2006[order(-Retail_expense_distr_2006$bank_aum),]

Retail_expense_distr_total_2006 = data.frame(PrimaryDistributionChannels="Total",
                                             bank_n_funds =Retail_expense_byaffiliation_2006$total_n_funds[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             bank_aum = Retail_expense_byaffiliation_2006$total_aum[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             bank_expense_wm = Retail_expense_byaffiliation_2006$total_expense_wm[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             bank_expense_p5 = Retail_expense_byaffiliation_2006$total_expense_p5[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             bank_expense_p95 = Retail_expense_byaffiliation_2006$total_expense_p95[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             n_funds =Retail_expense_byaffiliation_2006$total_n_funds[Retail_expense_byaffiliation_2006$bank_ind==F],
                                             aum = Retail_expense_byaffiliation_2006$total_aum[Retail_expense_byaffiliation_2006$bank_ind==F],
                                             expense_wm = Retail_expense_byaffiliation_2006$total_expense_wm[Retail_expense_byaffiliation_2006$bank_ind==F],
                                             expense_p5 = Retail_expense_byaffiliation_2006$total_expense_p5[Retail_expense_byaffiliation_2006$bank_ind==F],
                                             expense_p95 = Retail_expense_byaffiliation_2006$total_expense_p95[Retail_expense_byaffiliation_2006$bank_ind==F])

Retail_expense_distr_2006 = rbind(Retail_expense_distr_2006,
                                  Retail_expense_distr_total_2006)
stargazer(Retail_expense_distr_2006, summary = F, rownames = F, digits = 0)


# institutional funds----
Inst_expense_bank_byDistr_2006 = iMN[iMN$InvestorType =="Institutional"
                                     & iMN$date == "2006-06-27" &
                                         iMN$bank_ind==T,
                                       list(bank_n_funds = length(unique(id_fund)),
                                            bank_aum = sum(aum_fund, na.rm = T)/10^6,
                                            bank_expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                            bank_expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                                            bank_expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                                       by ="PrimaryDistributionChannels" ]


Inst_expense_nonbank_byDistr_2006 = iMN[iMN$InvestorType =="Institutional" & iMN$date == "2006-06-27" &
                                            iMN$bank_ind==F,
                                          list(n_funds = length(unique(id_fund)),
                                               aum = sum(aum_fund, na.rm = T)/10^6,
                                               expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                               expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                                               expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                                          by ="PrimaryDistributionChannels" ]

Inst_expense_byaffiliation_2006 = iMN[iMN$InvestorType =="Institutional" & iMN$date == "2006-06-27",
                                        list(total_n_funds = length(unique(id_fund)),
                                             total_aum = sum(aum_fund, na.rm = T)/10^6,
                                             total_expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                             total_expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                                             total_expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                                        by ="bank_ind" ]

Inst_expense_distr_total_2006 = data.frame(PrimaryDistributionChannels="Total",
                                           bank_n_funds =Inst_expense_byaffiliation_2006$total_n_funds[Inst_expense_byaffiliation_2006$bank_ind==T],
                                           bank_aum = Inst_expense_byaffiliation_2006$total_aum[Inst_expense_byaffiliation_2006$bank_ind==T],
                                           bank_expense_wm = Inst_expense_byaffiliation_2006$total_expense_wm[Inst_expense_byaffiliation_2006$bank_ind==T],
                                           bank_expense_p5 = Inst_expense_byaffiliation_2006$total_expense_p5[Inst_expense_byaffiliation_2006$bank_ind==T],
                                           bank_expense_p95 = Inst_expense_byaffiliation_2006$total_expense_p95[Inst_expense_byaffiliation_2006$bank_ind==T],
                                           n_funds =Inst_expense_byaffiliation_2006$total_n_funds[Inst_expense_byaffiliation_2006$bank_ind==F],
                                           aum = Inst_expense_byaffiliation_2006$total_aum[Inst_expense_byaffiliation_2006$bank_ind==F],
                                           expense_wm = Inst_expense_byaffiliation_2006$total_expense_wm[Inst_expense_byaffiliation_2006$bank_ind==F],
                                           expense_p5 = Inst_expense_byaffiliation_2006$total_expense_p5[Inst_expense_byaffiliation_2006$bank_ind==F],
                                           expense_p95 = Inst_expense_byaffiliation_2006$total_expense_p95[Inst_expense_byaffiliation_2006$bank_ind==F])

Retail_Inst_expense_distr_2006 = rbind(Retail_expense_distr_2006,
                                       Inst_expense_distr_total_2006)



stargazer(Retail_Inst_expense_distr_2006, summary = F, rownames = F, digits = 0)


## Government funds----


RetailGov_expense_bank_byDistr_2006 = iMN[iMN$CategoryHistorical =="Government Retail"
                                          & iMN$date == "2006-06-27" &
                                         iMN$BankAffiliated==T,
                                       list(bank_n_funds = length(unique(id_fund)),
                                            bank_aum = sum(aum_fund, na.rm = T)/10^6,
                                            bank_expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                            bank_expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                                            bank_expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                                       by ="PrimaryDistributionChannels" ]


RetailGov_expense_nonbank_byDistr_2006 = iMN[iMN$CategoryHistorical =="Government Retail"
                                             & iMN$date == "2006-06-27" &
                                               iMN$BankAffiliated==F,
                                          list(n_funds = length(unique(id_fund)),
                                               aum = sum(aum_fund, na.rm = T)/10^6,
                                               expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                               expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                                               expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                                          by ="PrimaryDistributionChannels" ]

RetailGov_expense_byaffiliation_2006 = iMN[iMN$CategoryHistorical =="Government Retail" & iMN$date == "2006-06-27",
                                        list(total_n_funds = length(unique(id_fund)),
                                             total_aum = sum(aum_fund, na.rm = T)/10^6,
                                             total_expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                             total_expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                                             total_expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                                        by ="BankAffiliated" ]


RetailGov_expense_byaffiliation_2006 = iMN[iMN$CategoryHistorical =="Prime Retail" & iMN$date == "2006-06-27",
                                           list(total_n_funds = length(unique(id_fund)),
                                                total_aum = sum(aum_fund, na.rm = T)/10^6,
                                                total_expense_wm = round(100*weighted.mean(expense,w = aum_fund, na.rm = T),2),
                                                total_expense_p5 = quantile(100*expense, na.rm = T, probs = 0.05),
                                                total_expense_p95 = quantile(100*expense, na.rm = T, probs = 0.95)),
                                           by ="BankAffiliated" ]


RetailGov_expense_distr_2006 = merge(RetailGov_expense_bank_byDistr_2006,RetailGov_expense_nonbank_byDistr_2006,
                                  by = "PrimaryDistributionChannels", all = F)


RetailGov_expense_distr_2006=RetailGov_expense_distr_2006[order(-Retail_expense_distr_2006$bank_aum),]

RetailGov_expense_distr_total_2006 = data.frame(PrimaryDistributionChannels="Total",
                                             bank_n_funds =Retail_expense_byaffiliation_2006$total_n_funds[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             bank_aum = Retail_expense_byaffiliation_2006$total_aum[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             bank_expense_wm = Retail_expense_byaffiliation_2006$total_expense_wm[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             bank_expense_p5 = Retail_expense_byaffiliation_2006$total_expense_p5[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             bank_expense_p95 = Retail_expense_byaffiliation_2006$total_expense_p95[Retail_expense_byaffiliation_2006$bank_ind==T],
                                             n_funds =Retail_expense_byaffiliation_2006$total_n_funds[Retail_expense_byaffiliation_2006$bank_ind==F],
                                             aum = Retail_expense_byaffiliation_2006$total_aum[Retail_expense_byaffiliation_2006$bank_ind==F],
                                             expense_wm = Retail_expense_byaffiliation_2006$total_expense_wm[Retail_expense_byaffiliation_2006$bank_ind==F],
                                             expense_p5 = Retail_expense_byaffiliation_2006$total_expense_p5[Retail_expense_byaffiliation_2006$bank_ind==F],
                                             expense_p95 = Retail_expense_byaffiliation_2006$total_expense_p95[Retail_expense_byaffiliation_2006$bank_ind==F])

Retail_expense_distr_2006 = rbind(Retail_expense_distr_2006,
                                  Retail_expense_distr_total_2006)
stargazer(Retail_expense_distr_2006, summary = F, rownames = F, digits = 0)





# Plot the time series variation of expense ratios

pdf(file = paste0(figures_dir, "ExpenseRatio_Distr.pdf"), width = 10, height = 7)
# All Funds
plot_ExpDistr(Expense_all, ylab = "Percent of AUM", main = "Expense ratio: All Funds")
plot_ExpDistr(Expense_bank, ylab = "Percent of AUM", main = "Expense ratio: All Bank Funds")

# Institutional funds
plot_ExpDistr(Expense_inst_all, ylab = "Percent of AUM", main = "Expense ratio: Institutional Funds")
plot_ExpDistr(Expense_inst_bank, ylab = "Percent of AUM", main = "Expense ratio: Institutional Bank Funds")
plot_ExpDistr(Expense_inst_nonbank, ylab = "Percent of AUM", main = "Expense ratio: Institutional Non-bank Funds")




# Retail funds
plot_ExpDistr(Expense_ret_all, ylab = "Percent of AUM", main = "Expense ratio: Retail Funds")
plot_ExpDistr(Expense_ret_bank, ylab = "Percent of AUM", main = "Expense ratio: Retail Bank Funds")
plot_ExpDistr(Expense_ret_nonbank, ylab = "Percent of AUM", main = "Expense ratio: Retail Non-bank Funds")



# Government funds
plot_ExpDistr(Expense_inst_gov, ylab = "Percent of AUM", main = "Expense ratio: Gov't Institutional Funds")
plot_ExpDistr(Expense_ret_gov, ylab = "Percent of AUM", main = "Expense ratio: Gov't Retail Funds")

# Prime funds
plot_ExpDistr(Expense_inst_prime, ylab = "Percent of AUM", main = "Expense ratio: Prime Institutional Funds")
plot_ExpDistr(Expense_ret_prime, ylab = "Percent of AUM", main = "Expense ratio: Prime Retail Funds")

# plot weighted averages

with(Expense_wm, plot(date, expense_all, type = "l", lwd = 2, ylim = c(0,1),
                      main = "Expense ratios: All funds (black),
                      Retail(green), Institutional (red) weighted average"))
par(new = T)
with(Expense_wm, lines(date, expense_ret, type = "l", col = "green", lwd = 2, ylim = c(0,1)))
with(Expense_wm, lines(date, expense_inst, type = "l",col = "red", lwd = 2, ylim = c(0,1)))
grid(nx = 10, ny = 10, col = "lightgray", lty = "dotted",
     lwd = par("lwd"), equilogs = TRUE)

with(Expense_wm, plot(date, expense_inst_bank, type = "l", col = "green",
                      lwd = 2, ylim = c(0,1),
                      main = "Expense ratios: Institutional bank (green) non-bank (red)"))
par(new = T)
with(Expense_wm, lines(date, expense_inst_nonbank, type = "l",col = "red", lwd = 2, ylim = c(0,1)))
grid(nx = 10, ny = 10, col = "lightgray", lty = "dotted",
     lwd = par("lwd"), equilogs = TRUE)

with(Expense_wm, plot(date, expense_ret_bank, type = "l", col = "green",
                      lwd = 2, ylim = c(0,1),
                      main = "Expense ratios: Retail bank (green) non-bank (red)"))
par(new = T)
with(Expense_wm, lines(date, expense_ret_nonbank, type = "l",col = "red", lwd = 2, ylim = c(0,1)))
grid(nx = 10, ny = 10, col = "lightgray", lty = "dotted",
     lwd = par("lwd"), equilogs = TRUE)


with(Expense_wm, plot(date, expense_ret_prime, type = "l", col = "green",
                      lwd = 2, ylim = c(0,1),
                      main = "Expense ratios: Prime Retail (green), Institutional (red)"))
par(new = T)
with(Expense_wm, lines(date, expense_inst_prime, type = "l",col = "red", lwd = 2, ylim = c(0,1)))
grid(nx = 10, ny = 10, col = "lightgray", lty = "dotted",
     lwd = par("lwd"), equilogs = TRUE)


dev.off()






#------------------------------------------------------------------------------>


















