# ----------------------------------------------------------------------------->>
# Author: Vladimir Yankov
# The data construction code is partially based on
# ( https://github.com/ajdamico/usgsd/tree/master/Survey%20of%20Consumer%20Finances)
#
# ----------------------------------------------------------------------------->>
# Clear workspace
rm(list = ls())
# Set path to where the files will be stored
#work_dir <- setwd("F:/Dropbox/Data/SCF/")
work_dir <-"./Data/SCF/"
figures_dir <- "./InSearchRisklessAsset/Results/Figures/"
tables_dir <- "./InSearchRisklessAsset/Results/Tables/"
# ----------------------------------------------------------------------------->
# Load required libaries
require(foreign)   # load foreign package (converts data files into R)
library(survey)
require(spatstat)
require(stargazer)
require(foreach)
# ----------------------------------------------------------------------------->
# Load data----
setwd(work_dir)
load("SCF_select.rda")
# ----------------------------------------------------------------------------->


# ----------------------------------------------------------------------------->
# Summary statistics for the financial sophistication index SCF 2007----
# ----------------------------------------------------------------------------->
summary(scf2007$pca_f1)
summary(scf2007$pca_f2)

with(scf2007[scf2007$rep==3,], plot(pca_f1,pca_f2))


pdf(file = paste0(figures_dir,"FinSophIndex_pca.pdf"),
                  width = 16, height = 10)
with(pca_mix_results2007,
     plot(rsq1,rsq2, lwd = 0.5, xlab="",ylab ="", col = "white",
          xlim = c(0,0.6), ylim = c(0,0.6)))
par(new =T)
with(pca_mix_results2007,
     arrows(x0= rep(0,length(rsq1)),
            y0 = rep(0,length(rsq1)), x1 = rsq1, y1 = rsq2,
            code = 2, col = "darkgray",
            length = 0.1, angle =30))
with(pca_mix_results2007,text(rsq1,(rsq2+0.01),labels = var, cex = 0.7))
dev.off()


#--2007
scf2007 = data.table(scf2007)
scf2007$fs_groups = NA
scf2007$fs_groups[scf2007$pca_f1_above_q4==T]=1
scf2007$fs_groups[scf2007$pca_f1_q23==T]=2
scf2007$fs_groups[scf2007$pca_f1_below_q1==T]=3

scf2007[,share_income_fin :=(intdivinc+kginc)/norminc,]
scf2007$share_income_fin[is.infinite(scf2007$share_income_fin)]=0
scf2007$share_income_fin[is.na(scf2007$share_income_fin)]=0

scf2007$share_income_fin[scf2007$share_income_fin>1]=1
scf2007$share_income_fin[scf2007$share_income_fin< -1]= -1

summary(scf2007$share_income_fin)

scf2007[, fin_share:= fin/asset]
summary(scf2007$fin_share)
scf2007$fin_share[is.na(scf2007$fin_share)]=0


# share of retirement and life insurance
scf2007[, share_ret_li:= (retqliq+cashli)/fin]
scf2007$share_ret_li[is.na(scf2007$share_ret_li)]=0
summary(scf2007$share_ret_li)
scf2007[,share_liq :=liq/fin]
summary(scf2007$share_liq)
# deposits exceed the deposit insurance limit
scf2007[, deposits_aboveFDIC:= (deposits>100000)]
scf2007[, cd_aboveFDIC:= (cd_amt>100000)]

# household leverage
scf2007[, leverage:= debt/asset]
scf2007$leverage[is.infinite(scf2007$leverage)]=0
scf2007$leverage[is.na(scf2007$leverage)]=0
scf2007$leverage[scf2007$leverage>2]=NA
summary(scf2007$leverage)

ecfd_pca_f1 <- ecdf(scf2007$pca_f1) # construct ECDF of the FS index

finsoph_quartiles = scf2007[, list(
  fs_groups = ecfd_pca_f1(weighted.mean(pca_f1, w= wgt)),
  age = weighted.mean(age,w=wgt, na.rm = T),
  college_educ= 100*weighted.mean(edcl==4, w= wgt),
  income = round(weighted.mean(income,w = wgt),0),
  networth = round(weighted.mean(networth,w = wgt),0),

  leverage = 100*weighted.mean(leverage,w = wgt, na.rm = T),


  bankruptcy = 100*weighted.mean(bankruptcy, w = wgt),
  homeown = 100*weighted.mean(houses>0, w = wgt),

  fin_share = 100*weighted.mean(fin_share, w = wgt, na.rm =T),

  share_deposits = 100*weighted.mean(share_deposits[fin>0], w = wgt[fin>0], na.rm =T),
  share_mmf = 100*weighted.mean(share_mmf[fin>0], w = wgt[fin>0]),
  share_gov_sec = 100*weighted.mean(share_gov_sec[fin>0], w = wgt[fin>0]),
  share_liq = 100*weighted.mean(share_liq[fin>0], w = wgt[fin>0], na.rm =T),
  share_equity = 100*weighted.mean(share_equity[fin>0], w = wgt[fin>0], na.rm =T),
  share_risky= 100*weighted.mean(share_risky[fin>0], w = wgt[fin>0], na.rm =T),
  share_ret_li = 100*weighted.mean(share_ret_li[fin>0], w= wgt[fin>0], na.rm =T),

  hcds = 100*weighted.mean(hcds, w = wgt, na.rm = T),
  cd_joint = 100*weighted.mean(cd_joint[hcds==1], w = wgt[hcds==1], na.rm = T),
  deposits_aboveFDIC = 100*weighted.mean(deposits_aboveFDIC[deposits>0], w = wgt[deposits>0], na.rm = T),
  cd_aboveFDIC = 100*weighted.mean(cd_aboveFDIC[hcds==1], w = wgt[hcds==1], na.rm = T),

  hmmf = 100*weighted.mean(hmmf[fin>0], w = wgt[fin>0], na.rm= T),
  hnmmf = 100*weighted.mean(hnmmf[fin>0], w = wgt[fin>0], na.rm= T),

  share_income_fin = 100*weighted.mean(share_income_fin[fin>0], w= wgt[fin>0], na.rm = T),

  nbanks = weighted.mean(nbanks[fin>0], w = wgt[fin>0], na.rm =T),
  nfininst = weighted.mean(num_inst_inferred[fin>0], w = wgt[fin>0], na.rm =T),

  risk_high = 100*weighted.mean(risk_high[fin>0], w = wgt[fin>0], na.rm =T),
  budget_horizon_long = 100*weighted.mean(budget_horizon_long[fin>0], w = wgt[fin>0], na.rm =T),

  ishopgrdl= 100*weighted.mean(ishopgrdl[fin>0], w = wgt[fin>0], na.rm =T),
  bshopgrdl= 100*weighted.mean(bshopgrdl[fin>0], w = wgt[fin>0], na.rm =T),

  iinternet= 100*weighted.mean(iinternet[fin>0], w = wgt[fin>0], na.rm =T),
  binternet= 100*weighted.mean(binternet[fin>0], w = wgt[fin>0], na.rm =T),

  scf_under= 100*weighted.mean(scf_under, w = wgt, na.rm =T),

  ifinpro = 100*weighted.mean(ifinpro[fin>0],w = wgt[fin>0], na.rm =T),
  bfinpro = 100*weighted.mean(bfinpro[fin>0],w = wgt[fin>0], na.rm =T)
                                   ),
                            by ="fs_groups" ]
finsoph_quartiles = finsoph_quartiles[order(-finsoph_quartiles$fs_groups),]
finsoph_quartiles = round(t(finsoph_quartiles),1)
finsoph_quartiles = finsoph_quartiles[-1,]
print(finsoph_quartiles)
# select the group that owns a CD



finsoph_quartiles_cd = scf2007[scf2007$hcds==1, list(
  fs_groups = ecfd_pca_f1(weighted.mean(pca_f1, w= wgt)),
  age = weighted.mean(age,w=wgt, na.rm = T),
  college_educ= 100*weighted.mean(edcl==4, w= wgt),
  income = round(weighted.mean(income,w = wgt),0),
  networth = round(weighted.mean(networth,w = wgt),0),

  leverage = 100*weighted.mean(leverage,w = wgt, na.rm = T),


  bankruptcy = 100*weighted.mean(bankruptcy, w = wgt),
  homeown = 100*weighted.mean(houses>0, w = wgt),

  fin_share = 100*weighted.mean(fin_share, w = wgt, na.rm =T),

  share_deposits = 100*weighted.mean(share_deposits[fin>0], w = wgt[fin>0], na.rm =T),
  share_mmf = 100*weighted.mean(share_mmf[fin>0], w = wgt[fin>0]),
  share_gov_sec = 100*weighted.mean(share_gov_sec[fin>0], w = wgt[fin>0]),
  share_liq = 100*weighted.mean(share_liq[fin>0], w = wgt[fin>0], na.rm =T),
  share_equity = 100*weighted.mean(share_equity[fin>0], w = wgt[fin>0], na.rm =T),
  share_risky= 100*weighted.mean(share_risky[fin>0], w = wgt[fin>0], na.rm =T),
  share_ret_li = 100*weighted.mean(share_ret_li[fin>0], w= wgt[fin>0], na.rm =T),

  hcds = 100*weighted.mean(hcds, w = wgt, na.rm = T),
  cd_joint = 100*weighted.mean(cd_joint[hcds==1], w = wgt[hcds==1], na.rm = T),
  deposits_aboveFDIC = 100*weighted.mean(deposits_aboveFDIC[deposits>0], w = wgt[deposits>0], na.rm = T),
  cd_aboveFDIC = 100*weighted.mean(cd_aboveFDIC[hcds==1], w = wgt[hcds==1], na.rm = T),

  hmmf = 100*weighted.mean(hmmf[fin>0], w = wgt[fin>0], na.rm= T),
  hnmmf = 100*weighted.mean(hnmmf[fin>0], w = wgt[fin>0], na.rm= T),

  share_income_fin = 100*weighted.mean(share_income_fin[fin>0], w= wgt[fin>0], na.rm = T),

  nbanks = weighted.mean(nbanks[fin>0], w = wgt[fin>0], na.rm =T),
  nfininst = weighted.mean(num_inst_inferred[fin>0], w = wgt[fin>0], na.rm =T),

  risk_high = 100*weighted.mean(risk_high[fin>0], w = wgt[fin>0], na.rm =T),
  budget_horizon_long = 100*weighted.mean(budget_horizon_long[fin>0], w = wgt[fin>0], na.rm =T),

  ishopgrdl= 100*weighted.mean(ishopgrdl[fin>0], w = wgt[fin>0], na.rm =T),
  bshopgrdl= 100*weighted.mean(bshopgrdl[fin>0], w = wgt[fin>0], na.rm =T),

  iinternet= 100*weighted.mean(iinternet[fin>0], w = wgt[fin>0], na.rm =T),
  binternet= 100*weighted.mean(binternet[fin>0], w = wgt[fin>0], na.rm =T),

  scf_under= 100*weighted.mean(scf_under, w = wgt, na.rm =T),

  ifinpro = 100*weighted.mean(ifinpro[fin>0],w = wgt[fin>0], na.rm =T),
  bfinpro = 100*weighted.mean(bfinpro[fin>0],w = wgt[fin>0], na.rm =T)
                                   )]
finsoph_quartiles_cd = round(t(finsoph_quartiles_cd),1)
print(finsoph_quartiles_cd)


# all population
finsoph_quartiles_all = scf2007[, list(
  fs_groups = ecfd_pca_f1(weighted.mean(pca_f1, w= wgt)),
  age = weighted.mean(age,w=wgt, na.rm = T),
  college_educ= 100*weighted.mean(edcl==4, w= wgt),
  income = round(weighted.mean(income,w = wgt),0),
  networth = round(weighted.mean(networth,w = wgt),0),

  leverage = 100*weighted.mean(leverage,w = wgt, na.rm = T),


  bankruptcy = 100*weighted.mean(bankruptcy, w = wgt),
  homeown = 100*weighted.mean(houses>0, w = wgt),

  fin_share = 100*weighted.mean(fin_share, w = wgt, na.rm =T),

  share_deposits = 100*weighted.mean(share_deposits[fin>0], w = wgt[fin>0], na.rm =T),
  share_mmf = 100*weighted.mean(share_mmf[fin>0], w = wgt[fin>0]),
  share_gov_sec = 100*weighted.mean(share_gov_sec[fin>0], w = wgt[fin>0]),
  share_liq = 100*weighted.mean(share_liq[fin>0], w = wgt[fin>0], na.rm =T),
  share_equity = 100*weighted.mean(share_equity[fin>0], w = wgt[fin>0], na.rm =T),
  share_risky= 100*weighted.mean(share_risky[fin>0], w = wgt[fin>0], na.rm =T),
  share_ret_li = 100*weighted.mean(share_ret_li[fin>0], w= wgt[fin>0], na.rm =T),

  hcds = 100*weighted.mean(hcds, w = wgt, na.rm = T),
  cd_joint = 100*weighted.mean(cd_joint[hcds==1], w = wgt[hcds==1], na.rm = T),
  deposits_aboveFDIC = 100*weighted.mean(deposits_aboveFDIC[deposits>0], w = wgt[deposits>0], na.rm = T),
  cd_aboveFDIC = 100*weighted.mean(cd_aboveFDIC[hcds==1], w = wgt[hcds==1], na.rm = T),

  hmmf = 100*weighted.mean(hmmf[fin>0], w = wgt[fin>0], na.rm= T),
  hnmmf = 100*weighted.mean(hnmmf[fin>0], w = wgt[fin>0], na.rm= T),

  share_income_fin = 100*weighted.mean(share_income_fin[fin>0], w= wgt[fin>0], na.rm = T),

  nbanks = weighted.mean(nbanks[fin>0], w = wgt[fin>0], na.rm =T),
  nfininst = weighted.mean(num_inst_inferred[fin>0], w = wgt[fin>0], na.rm =T),

  risk_high = 100*weighted.mean(risk_high[fin>0], w = wgt[fin>0], na.rm =T),
  budget_horizon_long = 100*weighted.mean(budget_horizon_long[fin>0], w = wgt[fin>0], na.rm =T),

  ishopgrdl= 100*weighted.mean(ishopgrdl[fin>0], w = wgt[fin>0], na.rm =T),
  bshopgrdl= 100*weighted.mean(bshopgrdl[fin>0], w = wgt[fin>0], na.rm =T),

  iinternet= 100*weighted.mean(iinternet[fin>0], w = wgt[fin>0], na.rm =T),
  binternet= 100*weighted.mean(binternet[fin>0], w = wgt[fin>0], na.rm =T),

  scf_under= 100*weighted.mean(scf_under, w = wgt, na.rm =T),

  ifinpro = 100*weighted.mean(ifinpro[fin>0],w = wgt[fin>0], na.rm =T),
  bfinpro = 100*weighted.mean(bfinpro[fin>0],w = wgt[fin>0], na.rm =T)
  )]
finsoph_quartiles_all =t(finsoph_quartiles_all)
print(finsoph_quartiles_all)


finsoph_quartiles = cbind(finsoph_quartiles,finsoph_quartiles_cd)
finsoph_quartiles = cbind(finsoph_quartiles,finsoph_quartiles_all)


rownames(finsoph_quartiles)=
  c("FS index",
    "Age",
    "College educaton",
    "Income",
    "Net worth",
    "Leverage",
    "Bankruptcy",
    "Homeowner",

    "Fin.assets",
    "Share deposits",
    "Share MMMF",
    "Share Gov't bonds",
    "Share safe and liquid",
    "Share equity",
    "Share risky",
    "Share retirement and life ins.",
    "Own CDs",
    "CD owned jointly",
    "Deposits above FDIC limit",
    "CD above FDIC limit",
    "Own MMMFs",
    "Own Mutual funds",

    "Income from fin.assets",

    "Number of banks",
    "Number fin.institutions",
    "Take financial risks",
    "Budgeting horizon over 5 years",
    "Great deal shopping investment",
    "Great deal shopping borrowing",
    "Use internet investment",
    "Use internet borrowing",
    "Excellent SCF understanding",
    "Use fin.professional investment",
    "Use fin.professional borrowing")

finsoph_quartiles = round(finsoph_quartiles,1)
print(finsoph_quartiles)

# Create summary table for financial sophistication index-----
###-----> export in Latex
stargazer(finsoph_quartiles, type = "latex", summary = F, digits = 0)

# ----------------------------------------------------------------------------->
# Construct aggregate time series----
# ----------------------------------------------------------------------------->
years = c(1989,1992,1995,1998,2001,2004,2007,2010,2013)


scf_agg = foreach(y = years, .combine = "rbind")%do%{

  eval(parse(text = paste0("scf_select=scf", y)))

  scf_select$fs_groups = NA
  scf_select$fs_groups[scf_select$pca_f1_above_q4==T]=1
  scf_select$fs_groups[scf_select$pca_f1_q23==T | scf_select$pca_f1_below_q1==T ]=2
  #scf_select$fs_groups[scf_select$pca_f1_below_q1==T]=3


  scf_select = data.table(scf_select)
  if(length(scf_select$iinternet)==0){scf_select$iinternet=NA}
  if(length(scf_select$bfinpro)==0){scf_select$bfinpro = NA}
  if(length(scf_select$cd_instnum)==0){scf_select$cd_instnum = NA}
  if(length(scf_select$ishopgrdl)==0){scf_select$ishopgrdl = NA}


 data_y = scf_select[,list(num_inst = weighted.mean(num_inst,
                                                    w = wgt, na.rm = T),
                            num_inst_inferred = weighted.mean(num_inst_inferred,
                                                              w = wgt, na.rm = T),
                            hcds = weighted.mean(hcds,w = wgt, na.rm = T),
                            cd_instnum = weighted.mean(cd_instnum[hcds==1],
                                                       w = wgt[hcds==1], na.rm = T),

                            cd_instnum_ishopgrdl = weighted.mean(cd_instnum[hcds==1
                                                                           & ishopgrdl==1],
                                                      w = wgt[hcds==1
                                                              & ishopgrdl==1], na.rm = T),
                           cd_instnum_noshopgrdl = weighted.mean(cd_instnum[hcds==1
                                                                           & ishopgrdl!=1],
                                                                w = wgt[hcds==1
                                                                        & ishopgrdl!=1], na.rm = T),

                           cd_num_ishopgrdl = weighted.mean(cd_num[hcds==1
                                                                           & ishopgrdl==1],
                                                                w = wgt[hcds==1
                                                                        & ishopgrdl==1], na.rm = T),

                           cd_num_noshopgrdl = weighted.mean(cd_num[hcds==1
                                                                   & ishopgrdl!=1],
                                                            w = wgt[hcds==1
                                                                    & ishopgrdl!=1], na.rm = T),



                           ishopgrdl = weighted.mean(ishopgrdl,w = wgt, na.rm = T),




                            nbanks = weighted.mean(nbanks,
                                                  w = wgt, na.rm = T),

                            nbanks_ishopgrdl = weighted.mean(nbanks[ishopgrdl==1],
                                                  w = wgt[ishopgrdl==1], na.rm = T),

                            nbanks_notshopgrdl = weighted.mean(nbanks[ishopgrdl!=1],
                                                            w = wgt[ishopgrdl!=1], na.rm = T),

                           nbanks_ishopgrdl_med = median(nbanks[ishopgrdl==1]),
                           nbanks_notshopgrdl_med = median(nbanks[ishopgrdl!=1]),

                           nbanks_iinternet = weighted.mean(nbanks[iinternet==1],
                                                            w = wgt[iinternet==1], na.rm = T),

                           nbanks_noiinternet = weighted.mean(nbanks[iinternet!=1],
                                                            w = wgt[iinternet!=1], na.rm = T),

                           nbanks_internet_med = median(nbanks[iinternet==1]),
                           nbanks_notinternet_med = median(nbanks[iinternet!=1]),

                           age_cd = weighted.mean(age[hcds==1],
                                                  w = wgt[hcds==1], na.rm = T),


                           age_eq = weighted.mean(age[hequity==1],
                                                  w = wgt[hequity==1], na.rm = T),

                           age_mmf = weighted.mean(age[hmmf==1],
                                                  w = wgt[hmmf==1], na.rm = T),


                            iinternet = weighted.mean(iinternet,
                                                      w = wgt, na.rm = T),
                            ishopgrdl= weighted.mean(ishopgrdl,
                                                     w = wgt, na.rm = T),
                            bfinpro = weighted.mean(bfinpro,
                                                    w = wgt, na.rm = T)),
                     by = "fs_groups"]

  data_y$year = y

  return(data_y)
}


# -----> Plots distinguishing financial sophistication and search-----
par(mar=rep(2.5,4))

pdf(file=paste0(figures_dir,"scf_low_finsoph_search.pdf"), width=16,height=10)
with(scf_agg[scf_agg$fs_groups==2 & scf_agg$year>=1995,],
     plot(year, nbanks_notshopgrdl, type = "b",
          lwd=2, pch=1,
          cex = 2,
          cex.lab =2,
          cex.axis=2,
          ylim = c(1,2),
          xaxt = "n",
          main = "",
          ylab = "Number of banks",
          xlab = ""))
axis(side=1,at = year[year>=1995], labels =year[year>=1995],cex.axis=2)
par(new= T)
with(scf_agg[scf_agg$fs_groups==2,],
     lines(year, nbanks_ishopgrdl, col = "red", cex =2,
           lwd =2, pch=2, type="b"))

legend("topright", c("Shopping for return:","Great deal","Moderate or little"),
       col = c("white","red","black"), lty=1,pch=c(1,2),lwd=2,
       bty="n",cex=2)

dev.off()


pdf(file=paste0(figures_dir,"scf_high_finsoph_search.pdf"), width=16,height=10)
with(scf_agg[scf_agg$fs_groups==1 & scf_agg$year>=1995,],
     plot(year, nbanks_notshopgrdl, type = "b", lwd=2, pch=1,
          ylim = c(1,3),
          cex = 2,
          cex.lab =2,
          cex.axis=2,
          xaxt = "n",
          main = "",
          ylab = "Number of banks",
          xlab = ""))
axis(side=1,at = year[year>=1995], labels =year[year>=1995],cex.axis=2)
par(new= T)
with(scf_agg[scf_agg$fs_groups==1,],
     lines(year, nbanks_ishopgrdl, cex=2,
           col = "red", lwd =2, pch=2, type="b"))

# legend("topright", c("Shopping intensity:","Great deal","Low"),
#        col = c("white","red","black"), lty=c(1,2,2),pch=c(1,2),lwd=2,
#        bty="n",cex=1.5)

dev.off()


## examine shares of hh reporting preference for high shopping

with(scf_agg[scf_agg$fs_groups==1 & scf_agg$year>=1995,],
     plot(year, ishopgrdl, type = "b", lwd=2, pch=1,
          col = "green",
          ylim = c(0,0.4),
          cex = 2,
          cex.lab =2,
          cex.axis=2,
          xaxt = "n",
          main = "",
          ylab = "",
          xlab = ""))
axis(side=1,at = year[year>=1995], labels =year[year>=1995],cex.axis=2)
par(new= T)
with(scf_agg[scf_agg$fs_groups==2,],
     lines(year, ishopgrdl, cex=2,
           col = "red", lwd =2, pch=2, type="b"))
with(scf_agg[scf_agg$fs_groups==3,],
     lines(year, ishopgrdl, cex=2,
           col = "blue", lwd =2, pch=2, type="b"))



# number of cds

pdf(file=paste0(figures_dir,"scf_low_finsoph_cd_search.pdf"), width=16,height=10)
with(scf_agg[scf_agg$fs_groups==1 & scf_agg$year>=1995,],
     plot(year, cd_num_noshopgrdl,
          type = "b", lwd=2, pch=1,
          ylim = c(1,4),
          xaxt = "n",
          main = "",
          ylab = "Number of CDs",
          xlab = ""))
axis(side=1,at = year[year>=1995], labels =year[year>=1995],cex=1.5 )
par(new= T)
with(scf_agg[scf_agg$fs_groups==2,],
     lines(year, cd_num_ishopgrdl, col = "red", lwd =2, pch=2, type="b"))

# legend("topright", c("Shopping intensity:","Great deal","Low"),
#        col = c("white","red","black"), lty=c(1,2,2),pch=c(1,2),lwd=2,
#        bty="n",cex=1.5)

dev.off()

# number of banks cds

pdf(file=paste0(figures_dir,"scf_high_finsoph_cd_inst_search.pdf"), width=16,height=10)
with(scf_agg[scf_agg$fs_groups==1 & scf_agg$year>=1995,],
     plot(year, cd_instnum_noshopgrdl, type = "b", lwd=2, pch=1,
          ylim = c(1,2),
          xaxt = "n",
          main = "",
          ylab = "Number of banks",
          xlab = ""))
axis(side=1,at = year[year>=1995], labels =year[year>=1995],cex=1.5 )
par(new= T)
with(scf_agg[scf_agg$fs_groups==1,],
     lines(year, cd_instnum_ishopgrdl, col = "red", lwd =2, pch=2, type="b"))

# legend("topright", c("Shopping intensity:","Great deal","Low"),
#        col = c("white","red","black"), lty=c(1,2,2),pch=c(1,2),lwd=2,
#        bty="n",cex=1.5)

dev.off()


# shopping for rates, fin sophistication, and number of CD contracts
with(scf_agg[scf_agg$fs_groups==1 & scf_agg$year>=1995,],
     plot(year, cd_num_ishopgrdl, type = "b", lwd=2, pch=1,
          ylim = c(1.5,3.5),
          xaxt = "n",
          main = "",
          ylab = "Number of banks",
          xlab = ""))
par(new=T)
with(scf_agg[scf_agg$fs_groups==1 & scf_agg$year>=1995,],
     lines(year, cd_num_noshopgrdl, type = "b", lwd=2, pch=1, col = "red"))



with(scf_agg[scf_agg$fs_groups==2 & scf_agg$year>=1995,],
     plot(year, cd_num_ishopgrdl, type = "b", lwd=2, pch=1,
          ylim = c(1.5,3.5),
          xaxt = "n",
          main = "",
          ylab = "Number of banks",
          xlab = ""))
par(new=T)
with(scf_agg[scf_agg$fs_groups==2 & scf_agg$year>=1995,],
     lines(year, cd_num_noshopgrdl, type = "b", lwd=2, pch=1, col = "red"))




# shopping for rates and number of bank accounts

with(scf_agg[scf_agg$fs_groups==1 & scf_agg$year>=1995,],
     plot(year, cd_instnum_ishopgrdl, type = "b", lwd=2, pch=1,
          ylim = c(1,2),
          xaxt = "n",
          main = "",
          ylab = "Number of banks",
          xlab = ""))
par(new=T)
with(scf_agg[scf_agg$fs_groups==1 & scf_agg$year>=1995,],
     lines(year, cd_instnum_noshopgrdl, type = "b", lwd=2, pch=1, col = "red"))



with(scf_agg[scf_agg$fs_groups==2 & scf_agg$year>=1995,],
     plot(year, cd_num_ishopgrdl, type = "b", lwd=2, pch=1,
          ylim = c(1.5,3.5),
          xaxt = "n",
          main = "",
          ylab = "Number of banks",
          xlab = ""))
par(new=T)
with(scf_agg[scf_agg$fs_groups==2 & scf_agg$year>=1995,],
     lines(year, cd_num_noshopgrdl, type = "b", lwd=2, pch=1, col = "red"))





# ----------------------------------------------------------------------------->
# Cross-sectional regressions----
# ----------------------------------------------------------------------------->
scf2004$majorexp5yr_saving[is.na(scf2004$majorexp5yr_saving)]=0

scf2007$majorexp5yr_saving[is.na(scf2007$majorexp5yr_saving)]=0


## ---- Determinants of whether a household holds CDs
model1 = as.formula("hcds ~ (age>=65)+(edcl==4)+log(asset)+
                     savres6+savres7+savres8+
(spendmor <3)+majorexp5yr_saving+
risk_high+budget_horizon_long+ishopgrdl+
(mmmf>0)+
pca_f1_above_q4+pca_f1")


for(y in years[years>=1998]){

  eval(parse(text = paste0('data = scf' , y )))

  prob_hcds = glm(formula = model1,
                      na.action("na.remove"),
                      # weights = wgt,
                      family=binomial(link="probit"),
                      data=data[data$asset>0,])
  eval(parse(text = paste0('prob_hcds' , y,"=prob_hcds" )))
}

#




stargazer(prob_hcds1998,prob_hcds2001,prob_hcds2004,
          prob_hcds2007, prob_hcds2010,prob_hcds2013,
          out = paste0(tables_dir,"CD_demand_reg.tex"),
          column.labels = c("1998","2001","2004","2007","2010","2013"),
          covariate.labels = c("Age 65 plus","Have college education","log(Assets)",
                               "Saving for retirement","Saving for liquidity",
                               "Saving for investment",
                               "Saving for major expense",
                               "Spend more if assets appreciated",
                               "Take above average financial risks",
                               "Budgeting horizon over 5 years",
                               "Great deal shopping for investment",
                               "Own money market mutual fund",
                               "Financial Sophistication (high)",
                               "Financial Sophistication"),
          type = "latex",
          title = "")


## Determinant of number of CD contracts and number of institutions


model2 = as.formula("(cd_instnum>1 | chkg_cd_inst1_diff==1 ) ~
                   (age>=65)+(edcl==4)+log(asset)+
                    deposits_aboveFDIC+ cd_joint+
                    risk_high+ishopgrdl+iinternet+
                    pca_f1_above_q4+pca_f1")

model2.1 = as.formula("(cd_instnum>1 | chkg_cd_inst1_diff==1 ) ~ (age>=65)+(edcl==4)+log(asset)+
                      deposits_aboveFDIC+cd_joint+
                      risk_high+ishopgrdl+
                      pca_f1_above_q4+pca_f1")

for(y in years[years>=1998]){

  eval(parse(text = paste0('data = scf' , y )))


  if(y <2001){
    cd_instnum = glm(formula = model2.1,
                       na.action("na.remove"),
                       # weights = wgt,
                       family=binomial(link="probit"),
                       data=data[data$hcds==1,])
    summary(cd_instnum)}

  if(y >=2001){
    cd_instnum = glm(formula = model2,
                  na.action("na.remove"),
                  # weights = wgt,
                  family=binomial(link="probit"),
                  data=data[data$hcds==1,])
}

  eval(parse(text = paste0('cd_instnum' , y,"=cd_instnum" )))
}
summary(cd_instnum1998)
summary(cd_instnum2007)


stargazer(cd_instnum1998,cd_instnum2001,cd_instnum2004,
          cd_instnum2007,cd_instnum2010,cd_instnum2013,
          out = paste0(tables_dir,"CD_multiple_reg.tex"),
          column.labels = c("1998","2001","2004","2007","2010","2013"),
          covariate.labels = c("Age 65 plus",
                               "College degree",
                               "log(Assets)",
                               "Deposits exceed FDIC limit",
                               "CD held jointly",
                               "Take above average financial risks",
                               "Great deal shopping for investment",
                               "Use Internet for investment decisions",
                               "Financial Sophistication (high)",
                               "Financial Sophistication"),

          type = "latex",
          title = "")
# ----------------------------------------------------------------------------->
# ---------------------> PLOTs
## scatter plot of the number of institutions and number of contracts
# ----------------------------------------------------------------------------->
#scf2007$cd_instnum[scf2007$cd_instnum==1 & scf2007$chkg_cd_inst1_diff==1]=2


# calculate the average share of households who invest in multiple banks
with(scf2007[scf2007$hcds==1,],
     weighted.mean((cd_instnum>1), w = wgt))

with(scf2007[scf2007$hcds==1,],
     mean((cd_instnum>1)))

## plot the scatter plot
pdf(width = 16, height = 10, file = paste0(figures_dir,"scatter_CDs_Institutions.pdf") )
with(scf2007[scf2007$hcds==1,], symbols(cd_num,cd_instnum, circles=wgt*cd_amt, xaxt="n",
                                        yaxt="n",
                                        ylim =c(0,10), xlim =c(0,20),
                                        ylab = "Number of banks",
                                        xlab = "Number of CDs", cex.lab = 1.5))
axis(1, at = seq(1, 20, by = 1), las=2, cex.axis = 1.5)
axis(2, at = seq(1, 10, by = 1), las=2, cex.axis = 1.5)
grid(NULL, NULL, col = "lightblue", equilogs = T)
dev.off()


cd_bank_allocations = foreach(nn = 1:10, .combine = "rbind")%do%{
df_cc=  foreach(cc = 1:20, .combine = "rbind")%do%{


data = scf2007[scf2007$hcds==1 &
                  scf2007$cd_instnum==nn &
                  scf2007$cd_num==cc,list(wgt, cd_amt)]

    data = data[,list(wgt_amt = sum(wgt*cd_amt,na.rm = T)/5)]


    df = data.frame(nbanks=nn,n_cds = cc, wgt = data)

    return(df)
  }

return(df_cc)
}

pdf(width = 16, height = 10, file = paste0(figures_dir,"scatter_CDs_Institutions.pdf") )
cd_bank_allocations$wgt_amt[cd_bank_allocations$wgt_amt==0]=NA
with(cd_bank_allocations, symbols(n_cds,nbanks, circles=wgt_amt,
                                        xaxt="n",
                                        yaxt="n",
                                        lwd = 3,
                                        col = "red",
                                        ylim =c(0,10), xlim =c(0,20),
                                        ylab = "Number of banks",
                                        xlab = "Number of CDs", cex.lab = 1.5))
par(new = T)
abline(h=1:10, col = "gray", lwd = 0.3, lty=2)
abline(v=1:20, col = "gray", lwd = 0.3, lty = 2)

axis(1, at = seq(1, 20, by = 1), las=2, cex.axis = 1.5)
axis(2, at = seq(1, 10, by = 1), las=2, cex.axis = 1.5)
dev.off()
# ----------------------------------------------------------------------------->
