# ----------------------------------------------------------------------------->
# Name: Construct_SCF_Extract_DepositAccounts_FinSoph.R
# Author: Vladimir Yankov
# Code based on ( https://github.com/ajdamico/usgsd/tree/master/Survey%20of%20Consumer%20Finances)
# Description: The code constructs a dataset of select HH variables
# ----------------------------------------------------------------------------->
# Clear workspace
rm(list = ls())
root_dir <- "./"
work_dir <-"./Data/SCF/"
# Load required libaries
require(foreign)   # load foreign package (converts data files into R)
require(geosphere)
require(foreach)
require(iterators)
require(foreach)
require(snow)
require(doSNOW)
require(data.table)
require(PCAmixdata)
# ----------------------------------------------------------------------------->
# Survey years
year = c( 1989 , 1992 , 1995 , 1998 , 2001 , 2004 , 2007 , 2010, 2013 )

# Open each survey and transform the data
setwd(work_dir)
inst_code = read.csv("./inst_codes.csv", stringsAsFactors = F)
# CPI index
cpi = read.dta("./CPIXFE_a.dta") # From FRED at FRB St. Louis.
# base year
base_y = 2008
cpi2008 =cpi
cpi2008$cpi2008 = cpi2008$CPIXFE/cpi2008$CPIXFE[cpi2008$year ==2008]
cpi2008 = cpi2008[cpi2008$year %in% year,c("year","cpi2008")]
# ----------------------------------------------------------------------------->
# Loop over different year SCF-----
# ----------------------------------------------------------------------------->>

for( y in year){
 print(y)
 survey = paste0("SCF",y)
 print(survey)
 setwd(paste0(work_dir,survey))
 #upload the data
 scf.m <-read.dta(paste0(survey,"_main.dta"))
 scf.e <-read.dta(paste0(survey,"_extract.dta"))
 rw <-read.dta(paste0(survey,"_rw.dta"))
 # convert variable names to lower case
 names( scf.m ) <- tolower( names( scf.m ) )
 names( scf.e ) <- tolower( names( scf.e ) )
 names( rw ) <- tolower( names( rw ) )
 # error record in 2007
 if ( y == 2007 ) rw <- rw[ !( rw$yy1 == 1817 ) , ]
 # check if the number of records == 5 times number of replicate weights rw
 if( nrow( scf.m ) != ( nrow( rw ) * 5 )){
   print( "the number of records in the main file doesn't equal five times the number in the rw file" )
   print( paste( 'scf.m rows:' , nrow( scf.m ) , " / rw rows:" , nrow( rw ) ) )
   stop( "this must be fixed before continuing." )
 }

 if (y == 1989 ){
   names( scf.m )[ names( scf.m ) == 'x1' ] <- 'y1' ; names( scf.m )[ names( scf.m ) == 'xx1' ] <- 'yy1' ;
   names( scf.e )[ names( scf.e ) == 'x1' ] <- 'y1' ; names( scf.e )[ names( scf.e ) == 'xx1' ] <- 'yy1'
   names( rw )[ names( rw ) == 'x1' ] <- 'y1' ; names( rw )[ names( rw ) == 'xx1' ] <- 'yy1'
 }

 # ----------------------------------------------------------------------------->
 # Construct bank account information----
 # ----------------------------------------------------------------------------->
 # y1 = yy1*10+(1..5), yy1 id of the case
 # wgt sampling weight

 var.e.keep =c("y1","yy1",
               "wgt", # sampling weight

               #----> demographic characteristics
               "hhsex","age","educ","edcl","married", "race",


               #----> employment
               "lf", # labor force participation: 1=working in some way, 0=not working at all;
               "OCCAT2",
               #----> imcome and its components
               "norminc","income",
               "wageinc", # wage income
               "intdivinc", # income from dividends and interest
               "ssretinc", # social security and retirement income
               "KGINC", # income from capital gains on financial assets
                        # In total, what was your (family's) net annual income from
                        #  gains or losses from mutual funds or from the sale of stocks,
                        #  bonds, or real estate in 2009, before deductions
                        #  for taxes and anything else?
               "RENTINC", # rent income

               ## saving behavior
               "saved",
               "wsaved", # 1=spending exceeded income, 2=spending equaled income, 3=spending less than income;

               ## expenditures
               # Households overall expenses over last 12 months: 1=unusually high, 2=unusually low, 3=normal;
               "expenshilo", # 2010 onwards

               ## late payments
               # household had any late payments in last year; LATE=(X3004=5);
               "late","late60",

               #- reasons for saving
               "savres1", #reasons for saving: 1=cant save, 2=education, 3=family, 4=home,
               "savres2", # 5=purchases, 6=retirement, 7=liquidity/the future, 8=investment,
               "savres3", # 9=no particular reason;
               "savres4", "savres5",  "savres6",  "savres7",  "savres8",  "savres9",

               # spend more or less if assets appreciated/depreciated
               # R would spend less if assets depreciated in value
               # 1=agree strongly, 2=agree somewhat, 3=neither agree nor disagree,
               # 4=disagree somewhat, 5=disagree strongly;
               "spendmor","spendless", # =X7492


               # financial wealth
               ## FIN=LIQ+CDS+NMMF+STOCKS+BOND+RETQLIQ+SAVBND+CASHLI+OTHMA+OTHFIN;
               #   all types of transactions accounts (liquid assets);
               # LIQ=CHECKING+SAVING+MMA+CALL;
               "fin","liq",
               # deposit accounts with commercial banks
               "checking","saving","mmda", "hcds", "cds",


               # other financial institutions/assets
               #-- MMMF
               "hmma", "mmmf",
               #MMA=MMDA+MMMF;
               "mma",
               # other mutual funds
               # total directly-held mutual funds, excluding MMMFs;
               # NMMF=STMUTF+TFBMUTF+GBMUTF+OBMUTF+COMUTF;
               "NMMF","STMUTF","TFBMUTF","GBMUTF","OBMUTF","COMUTF",


               #   total bonds, not including bond funds or savings bonds;
               # BOND=NOTXBND+MORTBND+GOVTBND+OBND;
               "BOND","SAVBND","GOVTBND","NOTXBND","OBND","MORTBND",
               # brokerage accounts
               "hbrok","CALL",
               # savings bonds/treasury bonds
               "savbnd",
               # equity
               # *   financial assets invested in stock:
               #   1. directly-held stock
               # 2. stock mutual funds: full value if described as stock mutual fund,
               # 1/2 value of combination mutual funds
               # 3. IRAs/Keoghs invested in stock:
               #   full value if mostly invested in stock,
               # 1/2 value if split between stocks/bonds or stocks/money market,
               # 1/3 value if split between stocks/bonds/money market
               # 4. other managed assets w/equity interest (annuities, trusts, MIAs):
               #   full value if mostly invested in stock,
               # 1/2 value if split between stocks/MFs & bonds/CDs, or
               # "mixed/diversified,"
               # 1/3 value if "other"
               # 5. thrift-type retirement accounts invested in stock
               # full value if mostly invested in stock
               # 1/2 value if split between stocks and interest earning assets
               # 6. savings accounts classified as 529 or other accounts that may
               # be invested in stocks;
               # *   equity in directly held stocks, stock mutual funds,
               # combination mutual funds, and other mutuals funds ;
               # %IF &YEAR LE 2001 %THEN %DO;
               # DEQ=STOCKS+STMUTF+.5*COMUTF;
               "hequity","equity","deq","equitinc","htrad","nstocks", "stocks", "bond",
               "HOUSES","MRTHEL",
               # non-financial assets
               #HOMEEQ=HOUSES-MRTHEL;
               "homeeq",

               # capital gains on non-financial assets
               #   total gains/losses;
               # KGSTMF: stocks and mutual funds: current value less gains/losses;
               "KGTOTAL","KGHOUSE", "KGORE", "KGBUS","KGSTMF",

               # Turn down for credit or feared to be turned down

               "TURNDOWN","FEARDENIAL","TURNFEAR",

               #   total financial assets;
               # FIN=LIQ+CDS+NMMF+STOCKS+BOND+RETQLIQ+SAVBND+CASHLI+OTHMA+OTHFIN
               "FIN","LIQ","CDS","NMMF","STOCKS","BOND","RETQLIQ","SAVBND","CASHLI","OTHMA","OTHFIN",

               #   total assets;
               #ASSET=FIN+NFIN;
               "asset","nfin",

               #   total debt;
               "DEBT","MRTHEL","RESDBT","OTHLOC","CCBAL","INSTALL","ODEBT",

               # NETWORTH=ASSET-DEBT;
               "networth","asset","debt",

               ## --> shopping for financial services;
                  "ishopnone","ishopmodr", "ishopgrdl",  # borrowing on X7100 (1--5)
                  "bshopnone","bshopmodr", "bshopgrdl", # X7111 investment (1--5)
               ## shopping method
               "binternet", "iinternet",
               "BCALL","BMAGZNEWS", "BMAILADTV",  "BFRIENDWORK", "BFINPRO", "BSELF",
               "BDONT",
               "ICALL", "IMAGZNEWS", "IMAILADTV","IINTERNET", "IFRIENDWORK","IFINPRO",
               "ISELF", "IDONT", "IOTHER", "BFINPLAN", "IFINPLAN"

               )
 var.e.keep = unique(tolower(var.e.keep))
 var.e.keep = var.e.keep[which(var.e.keep %in% names(scf.e))]
 scf_select = scf.e[, var.e.keep]
 scf_select = scf_select[order(scf_select$y1),]

 # merge with main survey data
 scf.m$yy1=NULL
 scf_select = merge(scf_select, scf.m, by = "y1")

 # Characterization of responded

 ## -- ability to understand the survey
 # X6525           THE RESPONDENT'S UNDERSTANDING OF THE
 #                QUESTIONS WAS:
 #
 # 1.     EXCELLENT
 # 2.     GOOD
 # 3.     FAIR
 # 4.     POOR
 if(length(scf_select$x6525)!=0){
 scf_select$scf_understand = scf_select$x6525
 scf_select$scf_under = 0
 scf_select$scf_under[scf_select$scf_understand==1]=1
 }
 # bankruptcy
 if(length(scf_select$x6772)!=0){
 scf_select$bankruptcy = scf_select$x6772
 scf_select$bankruptcy[scf_select$bankruptcy==5]=0
 }
 # business owner

 ##-----> allocations among institutions
 # number of institutions
 # With how many financial institutions do you currently have
 # accounts or loans, or regularly do personal financial
 # business?  Include banks, savings and loans, credit unions,
 # brokerages, loan companies, and so forth, but not
 # institutions where you have only credit cards or business
 # accounts.
 #
 # With how many financial institutions do you and your family
 # living here currently have accounts or loans, or regularly
 # do personal financial business?  Include banks, savings and
 # loans, credit unions, brokerages, loan companies, and so
 # forth, but not institutions where you have only credit
 # cards or business accounts.

 scf_select$num_inst = scf_select$x305
 scf_select$num_inst[which(scf_select$num_inst ==-1)] <- NA
 if(length(scf_select$x8300)==0){scf_select$num_inst_inferred =NA}
 if(length(scf_select$x8300)!=0){
 scf_select$num_inst_inferred = scf_select$x8300
 scf_select$num_inst_inferred[scf_select$num_inst_inferred<0]=NA}

 # number of states of operation

 # #X7036(#1)       Does this institution have offices in more than one state?
 # #   X7038(#2)
 #     X7040(#3)       TREAT FEDERAL GOVERNMENT AGENCY AS HAVING
 #       X7042(#4)       OFFICES IN MORE THAN ONE STATE.
 #         X7044(#5)
 #           X7046(#6)       IF R KNOWS ONLY THAT THE INSTITUTION IS AVAILABLE
 #             X7048(#7)       ON-LINE, CODE YES, BUT MAKE AN [F2] COMMENT

 if(length(scf_select$x7036)>0){
 num_states = tolower(c("X7036","X7038","X7040","X7042","X7044","X7046","X7048"))
 for(vv in 1:7){
   eval(parse(text = paste0("scf_select$num_states",vv,"=scf_select$",num_states[vv] )))
   eval(parse(text = paste0("scf_select$num_states",vv,"[scf_select$num_states",vv,"==0]=NA" )))
   eval(parse(text = paste0("scf_select$num_states",vv,"[scf_select$num_states",vv,"==5]=0" )))
 }
 }

 # Main way of interacting with institution
 # X6600 X6601 X6602 X6603 X6604 X6605 X6606 X6607
 # X6870 X6871 X6872 X6873 (Institution 1)
 # X6608 X6609 X6610 X6611 X6612 X6613 X6614 X6615
 # X6874 X6875 X6876 X6877 (Institution 2)
 # X6616 X6617 X6618 X6619 X6620 X6621 X6622 X6623
 # X6878 X6879 X6880 X6881 (Institution 3)
 # X6624 X6625 X6626 X6627 X6628 X6629 X6630 X6631
 # X6882 X6883 X6884 X6885 (Institution 4)
 # X6632 X6633 X6634 X6635 X6636 X6637 X6638 X6639
 # X6886 X6887 X6888 X6889 (Institution 5)
 # X6640 X6641 X6642 X6643 X6644 X6645 X6646 X6647
 # X6890 X6891 X6892 X6893 (Institution 6)
 # X6656 X6657 X6658 X6659 X6660 X6661 X6662 X6663
 # X6894 X6895 X6896 X6897 (Institution 7)
 # 1.    *ATM/CASH MACHINE/DEBIT CARD
 # 2.    *IN PERSON
 # 3.    *MAIL
 # 4.    *PHONE - TALKING
 # 5.    *AUTOMATIC DEPOSIT OR WITHDRAWAL
 # 6.    *DON'T DO REGULAR BUSINESS
 #                     7.    *PHONE - USING TOUCHTONE SERVICE
 #                     9.    *OTHER ELECTRONIC TRANSFER
 # 10.    *CHECK WRITTEN ON INSTITUTION, n.f.s.
 #                    11.     R's agent or manager; personal banker; go-between
 # (this is a broad category that encompasses both
 #   formal and informal relationships)
 # 12.    *COMPUTER/INTERNET/ONLINE SERVICE/email
 # 30.     Fax Machine
 # 33.     Credit card
 # -7.    *OTHER
 # 0.     Inap. (/no further responses; no institutions:
 #                 X8300=-1/fewer than 2 institutions:
 #                 X8300<2/fewer than 3 institutions:
 #                 X8300<3/ fewer than 4 institutions:
 #                 X8300<4/fewer than 5 institutions:
 #                 X8300<5/fewer than 6 institutions:
 #                 X8300<6/fewer than 7 institutions:
 #                 X8300<7)
 #
 # X6600  (Institution 1)
 # X6608  (Institution 2)
 # X6616  (Institution 3)
 # X6624  (Institution 4)
 # X6632  (Institution 5)
 # X6640  (Institution 6)
 # X6656  (Institution 7)
 if(length(scf_select$x6600)>0){
 main_contact = tolower(c("X6600","X6608","X6616","X6624","X6632","X6640","X6656"))
 for(vv in 1:7){
   eval(parse(text = paste0("scf_select$main_contact",vv,"=scf_select$",main_contact[vv] )))
 }
 }

 # # distance to financial institution
 # X310(#1)        Roughly, how many miles is the office or ATM (cash machine)
 #   X314(#2)        of this institution from (your home or workplace/the home or
 #     X318(#3)        workplace of the person who uses it most often)?
 #       X322(#4)
 #         X326(#5)        IF R ASKS:  WE WANT THE CLOSER OF THE DISTANCE FROM HOME OR
 #           X330(#6)        WORKPLACE.
 #             X334(#7)
 #               (Is it more than 50 miles?)
 #
 #               USE "OVER 50 MILES," "FOREIGN LOCATION", "TOLL-FREE PHONE",
 #               "LOCAL POST BOX" OR "INTERNET" ONLY IF R CANNOT GIVE AN
 #               ESTIMATE OF THE NUMBER OF MILES.
 #
 #               MILES:
 #                 0.     Inap. (no institutions: X8300=-1/fewer than 2
 #                               institutions: X8300<2/fewer than 3 institutions:
 #                                 X8300<3/fewer than 4 institutions: X8300<4/
 #                                 fewer than 5 institutions: X8300<5/fewer than
 #                               6 institutions: X8300<6/fewer than 7 institutions:
 #                                 X8300<7)
 if(length(scf_select$x310)>0){
 inst_dist = tolower(c("X310","X314","X318","X322","X326","X330","X334" ))
 inst_dist = inst_dist[inst_dist %in% names(scf_select)]

 for(vv in 1:length(inst_dist)){
   eval(parse(text = paste0("scf_select$inst_dist",vv,"=scf_select$",inst_dist[vv] )))
   eval(parse(text = paste0("scf_select$inst_dist",vv,"[scf_select$inst_dist",vv,"<=0]=NA")))
 }
}



 # institution type
 main_type = c("x308", "x312", "x316", "x320", "x324", "x328","X332")
 main_type= main_type[which(main_type %in% names(scf_select))]

 for(vv in 1:length(main_type)){
   eval(parse(text = paste0("scf_select$inst_type",vv,"=scf_select$",main_type[vv] )))
 }


 ## count the number of banks
 scf_select$nbanks = 1*(scf_select$inst_type1 %in% c(11,12))+
   1*(scf_select$inst_type2 %in% c(11,12))+1*(scf_select$inst_type3 %in% c(11,12))+
   1*(scf_select$inst_type4 %in% c(11,12))+1*(scf_select$inst_type5 %in% c(11,12))+
   1*(scf_select$inst_type6 %in% c(11,12))
 if(length(main_type)==7){
   scf_select$nbanks = 1*(scf_select$inst_type1 %in% c(11,12))+
     1*(scf_select$inst_type2 %in% c(11,12))+1*(scf_select$inst_type3 %in% c(11,12))+
     1*(scf_select$inst_type4 %in% c(11,12))+1*(scf_select$inst_type5 %in% c(11,12))+
     1*(scf_select$inst_type6 %in% c(11,12))+1*(scf_select$inst_type7 %in% c(11,12))
 }


 # Deposit Accounts----
 # checking accounts
  scf_select$chkg = scf_select$x3501 #  First, do you (or anyone in your family living here)
                                     #  have any checking accounts at any type of institution?
  scf_select$chkg[scf_select$chkg == 5] = 0
 #table(scf_select$chkg)

 scf_select$chkg_num = scf_select$x3504 # X3504  How many checking accounts do you (and your family living
                                        # here) have?
 chkg_inst = c("x3505", "x3509", "x3513", "x3517", "x3521", "x3525")
 chkg_type = c("x9113", "x9114", "x9115", "x9116", "x9117", "x9118")
 chkg_amt = c("x3506", "x3510", "x3514", "x3518", "x3522", "x3526", "x3529")
 chkg_fdic = c("x7065", "x7066", "x7067", "x7068", "x7069", "x7070")

 for(kk in 1:length(chkg_inst)){
   eval(parse(text = paste0("scf_select$chkg_inst",kk,"=scf_select$",chkg_inst[kk] )))
   eval(parse(text = paste0("scf_select$chkg_type",kk,"=scf_select$",chkg_type[kk] )))

   eval(parse(text = paste0("scf_select$chkg_amt",kk,"=scf_select$",chkg_amt[kk] )))
   eval(parse(text = paste0("scf_select$chkg_amt",kk,"[scf_select$chkg_amt",kk," ==-1 ]=NA " )))

   # fdic insurance on checking account
   if(length(scf_select$x7065)>0){
   eval(parse(text = paste0("scf_select$chkg_fdic",kk,"=scf_select$",chkg_fdic[kk] )))
   eval(parse(text = paste0("scf_select$chkg_fdic",kk,"[scf_select$chkg_fdic",kk," ==0 ]=NA " )))
   eval(parse(text = paste0("scf_select$chkg_fdic",kk,"[scf_select$chkg_fdic",kk," ==5 ]=0 " )))}
 }



 # savings accounts
 if(y <2001){scf_select$sav_num = scf_select$x3802}
 sav_inst = tolower(c("x3803", "x3806", "x3809", "x3812"," x3815"))
 sav_type = tolower(c("x9138", "x9139"," x9140"," x9141"," x9142"))
 sav_amt = tolower(c("x3804", "x3807", "x3810", "x3813", "x3816"))
 if(y >2001){
 scf_select$sav_num <- scf_select$x3728
 sav_inst = c("x3729", "x3735", "x3741", "x3747", "x3753", "x3759")
 sav_type = tolower(c("x9259", "x9260", "x9261", "x9262", "x9263", "x9264"))
 sav_amt = tolower(c("x3730", "x3736", "x3742", "x3748", "x3754", "x3760"))
 }
 sav_fdic = tolower(c("x7076", "x7079", "x7082", "x7085", "x7088", "x7091"))

 for(kk in 1:length(sav_inst)){
   eval(parse(text = paste0("scf_select$sav_inst",kk,"=scf_select$",sav_inst[kk] )))
   eval(parse(text = paste0("scf_select$sav_type",kk,"=scf_select$",sav_type[kk] )))

   eval(parse(text = paste0("scf_select$sav_amt",kk,"=scf_select$",sav_amt[kk] )))
   eval(parse(text = paste0("scf_select$sav_amt",kk,"[scf_select$sav_amt",kk," ==-1 ]=NA " )))

   # fdic insurance on checking account
   if(length(scf_select$x7076)>0){
   eval(parse(text = paste0("scf_select$sav_fdic",kk,"=scf_select$",sav_fdic[kk] )))
   eval(parse(text = paste0("scf_select$sav_fdic",kk,"[scf_select$sav_fdic",kk," ==0 ]=NA " )))
   eval(parse(text = paste0("scf_select$sav_fdic",kk,"[scf_select$sav_fdic",kk," ==5 ]=0 " )))
   }

 }



 # certificates of deposit (CDs)
 scf_select$cd <- scf_select$x3719
 scf_select$cd[scf_select$cd == 5] = 0
 scf_select$cd_num = scf_select$x3720
 scf_select$cd_amt <- scf_select$x3721
 scf_select$cd_instnum <- scf_select$x3726
 scf_select$cd_joint1 <- scf_select$x7620
 ##-- recode to yes(1) or no (0)
 scf_select$cd_joint = 0
 scf_select$cd_joint[!scf_select$cd_joint1 %in% c(2,51,-7,0)]=1
 # 1.    *Joint account; with spouse/partner
 # 2.    *R's account
 #                     3.    *Spouse's/partner's account
 #                     4.    *Other family member's account
 # 5.    *CHILD ONLY; grandchild only
 # 6.     Child/grandchild and R/spouse/partner
 # 8.     Other relative with R/spouse/partner
 # 9.     Unrelated person, n.f.s.
 # 10.     Unrelated person with R/spouse/partner
 # 11.     Equal amounts in R/spouse/partner names
 # (only use for CDs)
 # 50.     Trust account
 # 51.     Personal business account
 # -7.    *Other
 # 0.     Inap. (no certificates of deposit: X3719^=1;
 #               R lives alone: X7001=1)


 cd_inst = tolower(c("x3722", "x3723", "x3724", "x3725", "x7618", "x6654", "x6655"))
 cd_type = tolower(c("x9134", "x9135", "x9136", "x9137", "x9214", "x9217", "x9218"))

 cd_inst =cd_inst[cd_inst %in% names(scf_select)]
 cd_type =cd_type[cd_type %in% names(scf_select)]


 for(kk in 1:length(cd_inst)){
   eval(parse(text = paste0("scf_select$cd_inst",kk,"=scf_select$",cd_inst[kk] )))
   eval(parse(text = paste0("scf_select$cd_type",kk,"=scf_select$",cd_type[kk] )))
 }

 if(length(scf_select$cd_inst5)==0){scf_select$cd_inst5=0}
 if(length(scf_select$cd_inst6)==0){scf_select$cd_inst6=0}

 ## count the number of institutions
 if(length(scf_select$cd_instnum)==0 & length(scf_select$cd_inst1)!=0){
 scf_select$cd_instnum = (scf_select$cd_inst1!=0)*1+(scf_select$cd_inst2!=0)*1+
           (scf_select$cd_inst3!=0)*1+(scf_select$cd_inst4!=0)*1+
           (scf_select$cd_inst5!=0)*1+(scf_select$cd_inst6!=0)*1
 }


 ## check if CD account is different from the main checking account
 if(length(scf_select$chkg_inst1)!=0 & length(scf_select$cd_inst1)!=0){
   scf_select$chkg_cd_inst1_diff = 0
   scf_select$chkg_cd_inst1_diff[scf_select$cd_inst1!=1 & scf_select$cd_inst1>0]=1
 }




 # ----------------------------------------------------------------------------->
 # MUTUAL FUNDS----
 # ----------------------------------------------------------------------------->
# x3819           Do you (or anyone in your family livin
scf_select$mf = scf_select$x3819
scf_select$mf[scf_select$mf==5]=0

# x3820 In how many different mutual funds (or hedge funds) do you own shares?
scf_select$num_mf = scf_select$x3820

# institution mutual fund X6799 X6800 X6801 X6802 X6803 X6850 X6851 X6852
mf_inst = tolower(c("X6799", "X6800", "X6801", "X6802", "X6803", "X6850", "X6851", "X6852"))
mf_type = tolower(c("X9225", "X9226", "X9227", "X9228", "X9229", "X9230", "X9231", "X9232"))

for(kk in 1:length(mf_inst)){
  eval(parse(text = paste0("scf_select$mf_inst",kk,"=scf_select$",mf_inst[kk] )))
  eval(parse(text = paste0("scf_select$mf_type",kk,"=scf_select$",mf_type[kk] )))
}


# Planning horizon
# X3006 X3007 X7513 X7514 X7515 X6848

# X3010           In the next five to ten years, are there any foreseeable
# major expenses that you (and your family) expect to have to
# pay for (yourself/yourselves), such as educational expenses,
# purchase of a new home, health care costs, support for other
# family members, or anything else?

scf_select$majorexp5yr = scf_select$x3010
scf_select$majorexp5yr[scf_select$majorexp5yr==5]=0

# X7186           Are you saving for these expenses now?
if(is.null(scf_select$x7186)){scf_select$majorexp5yr_saving =NA}
if(!is.null(scf_select$x7186)){scf_select$majorexp5yr_saving = scf_select$x7186}

scf_select$majorexp5yr_saving[scf_select$majorexp5yr_saving==0]=NA
scf_select$majorexp5yr_saving[scf_select$majorexp5yr_saving==5]=0
scf_select$majorexp5yr_saving[scf_select$majorexp5yr_saving==6]=2

# X3008           IN PERSON VERSION:
#   (SHOW CARD 7)
# In planning or budgeting your (family's) saving and spending,
#                 which of the time periods listed on this page is most
#                 important to you (and your family living here)?
#
#                 TELEPHONE VERSION:
#                 In planning or budgeting your (family's) saving and spending,
# which of the following time periods is most important
# to you (and your family living here): the next few months,
# the next year, the next few years, the next 5 to 10 years, or
# longer than 10 years?
#
# 1.    *NEXT FEW MONTHS
# 2.    *NEXT YEAR
# 3.    *NEXT FEW YEARS
# 4.    *NEXT 5-10 YEARS
# 5.    *LONGER THAN 10 YEARS
if(length(scf_select$x3008)>0){
  scf_select$budget_horizon =scf_select$x3008
  scf_select$budget_horizon_long = 0
  scf_select$budget_horizon_long[scf_select$budget_horizon %in% c(4,5)]=1
}


# X3014           IN PERSON VERSION:
#   (SHOW CARD 8)
# Which of the statements on this page comes
# closest to the amount of financial risk that you
# (and your {husband/wife/partner}) are willing
# to take when you save or make investments?
#
# READ RESPONSES IF NECESSARY.
#
# TELEPHONE VERSION:
#   Which of the following statements comes closest
# to describing the amount of financial risk that you
# (and your {husband/wife/partner}) are willing to
# take when you save or make investments?
#
# INTERVIEWER:  IF MORE THAN ONE RESPONSE IS GIVEN
# USE THE FIRST CATEGORY THAT APPLIES.
#
# 1.    *Take substantial financial risks expecting to
# earn substantial returns
# 2.    *Take above average financial risks expecting
# to earn above average returns
# 3.    *Take average financial risks expecting to earn
# average returns
# 4.    *Not willing to take any financial risks
if(is.null(scf_select$x3014)){scf_select$risk_attitude =NA}
if(!is.null(scf_select$x3014)){
  scf_select$risk_attitude =scf_select$x3014
  scf_select$risk_high = 0
  scf_select$risk_high[scf_select$risk_attitude %in% c(1,2)]=1
  }

# Construct additional variables
# total deposits
scf_select$deposits <- scf_select$cds+scf_select$checking+scf_select$mmda+scf_select$saving

# Amount of deposits per bank account
scf_select$liqdep_inst1 = 0
scf_select$liqdep_inst2 = 0
scf_select$liqdep_inst3 = 0
scf_select$liqdep_inst4 = 0
scf_select$liqdep_inst5 = 0
scf_select$liqdep_inst6 = 0


for(b in 1:6){
  if(length(scf_select$sav_inst6)>0){
    print(b)
  # account with bank 1
  scf_select$liqdep_inst1 = scf_select$liqdep_inst1+
    (eval(parse(text = paste0("scf_select$chkg_inst", b)))==1)*
    (eval(parse(text = paste0("scf_select$chkg_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$chkg_amt", b))) +
    (eval(parse(text = paste0("scf_select$sav_inst", b)))==1)*
    (eval(parse(text = paste0("scf_select$sav_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$sav_amt", b)))

  # account with bank 2
  scf_select$liqdep_inst2 = scf_select$liqdep_inst2+
    (eval(parse(text = paste0("scf_select$chkg_inst", b)))==2)*
    (eval(parse(text = paste0("scf_select$chkg_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$chkg_amt", b))) +
    (eval(parse(text = paste0("scf_select$sav_inst", b)))==2)*
    (eval(parse(text = paste0("scf_select$sav_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$sav_amt", b)))

  # account with bank 3
  scf_select$liqdep_inst3 = scf_select$liqdep_inst3+
    (eval(parse(text = paste0("scf_select$chkg_inst", b)))==3)*
    (eval(parse(text = paste0("scf_select$chkg_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$chkg_amt", b))) +
    (eval(parse(text = paste0("scf_select$sav_inst", b)))==3)*
    (eval(parse(text = paste0("scf_select$sav_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$sav_amt", b)))

  # account with bank 4
  scf_select$liqdep_inst4 = scf_select$liqdep_inst4+
    (eval(parse(text = paste0("scf_select$chkg_inst", b)))==4)*
    (eval(parse(text = paste0("scf_select$chkg_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$chkg_amt", b))) +
    (eval(parse(text = paste0("scf_select$sav_inst", b)))==4)*
    (eval(parse(text = paste0("scf_select$sav_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$sav_amt", b)))

  # account with bank 5
  scf_select$liqdep_inst5 = scf_select$liqdep_inst5+
    (eval(parse(text = paste0("scf_select$chkg_inst", b)))==5)*
    (eval(parse(text = paste0("scf_select$chkg_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$chkg_amt", b))) +
    (eval(parse(text = paste0("scf_select$sav_inst", b)))==5)*
    (eval(parse(text = paste0("scf_select$sav_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$sav_amt", b)))

  # account with bank 6
  scf_select$liqdep_inst6 = scf_select$liqdep_inst6+
    (eval(parse(text = paste0("scf_select$chkg_inst", b)))==6)*
    (eval(parse(text = paste0("scf_select$chkg_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$chkg_amt", b))) +
    (eval(parse(text = paste0("scf_select$sav_inst", b)))==6)*
    (eval(parse(text = paste0("scf_select$sav_type", b)))%in% c(11,12,13))*
    eval(parse(text = paste0("scf_select$sav_amt", b)))
  }
}

scf_select$liqdep_bank = scf_select$liqdep_inst1+scf_select$liqdep_inst2+scf_select$liqdep_inst3+
  scf_select$liqdep_inst4+scf_select$liqdep_inst5+scf_select$liqdep_inst6

scf_select$deposits_bank =scf_select$liqdep_bank+scf_select$cd_amt

scf_select$share_intdivinc = scf_select$intdivinc/scf_select$income
scf_select$share_intdivinc[!is.finite(scf_select$share_intdivinc)]=0
scf_select$share_intdivinc[scf_select$share_intdivinc>1]=1
# ----------------------------------------------------------------------------->
# Construct an index of financial sophistication-----
# ----------------------------------------------------------------------------->

#__ Components-----

#__ 1 Invests in stocks----
scf_select$hdeq = 0
scf_select$hdeq[scf_select$deq>0]=1

#__ 2 Number of stocks and funds----

#__ 3 Use of financial advisers----

#__ 4 Use of brokerage and mmf accounts----

scf_select$hmmf = 0
scf_select$hmmf[scf_select$mmmf>0]=1

# have other mutual funds

scf_select$hnmmf = 0
scf_select$hnmmf[scf_select$nmmf>0]=1


# diversified portfolio of financial assets

#-- share of risky-assets
scf_select$share_risky = (scf_select$deq+scf_select$tfbmutf+scf_select$obmutf+
                            scf_select$bond-scf_select$govtbnd)/scf_select$fin
scf_select$share_risky[scf_select$share_risky>1]=1
scf_select$share_risky[is.na(scf_select$share_risky)]=0

##-- Share of Gov. MMMF + Government bonds + Savings bonds
scf_select$gov_sec = scf_select$govtbnd+scf_select$savbnd
scf_select$share_gov_sec = scf_select$gov_sec/scf_select$fin
scf_select$share_gov_sec[is.na(scf_select$share_gov_sec)]=0

#-- concentration/diversification of asset holdings
scf_select$share_deposits = scf_select$deposits/scf_select$fin
scf_select$share_deposits[is.na(scf_select$share_deposits)]=0

scf_select$share_mmf = scf_select$mmmf/scf_select$fin
scf_select$share_mmf[is.na(scf_select$share_mmf)]=0

scf_select$share_nmmf = scf_select$nmmf/scf_select$fin
scf_select$share_nmmf[is.na(scf_select$share_nmmf)]=0

scf_select$share_equity = scf_select$stocks/scf_select$fin
scf_select$share_equity[is.na(scf_select$share_equity )]=0

scf_select$share_bonds = scf_select$bond/scf_select$fin
scf_select$share_bonds[is.na(scf_select$share_bonds )]=0

scf_select = data.table(scf_select)
scf_select[,fin_hhi := share_deposits^2 + share_mmf^2 + share_nmmf^2+share_equity^2+share_bonds^2,]
scf_select = data.frame(scf_select)
summary(scf_select$fin_hhi)

#__ 4 Risk aversion-----
# "risk_high"


#__ 5 Planning horizon----

#__ 6 Capital gains----
scf_select$kgstmf_stmf = scf_select$kgstmf/scf_select$stmutf
scf_select$kgstmf_stmf[is.infinite(scf_select$kgstmf_stmf)]=0
scf_select$kgstmf_stmf[is.na(scf_select$kgstmf_stmf)]=0
scf_select$kgstmf_stmf[which(scf_select$kgstmf_stmf< -100)]= -100
scf_select$kgstmf_stmf[which(scf_select$kgstmf_stmf>= 100)] = 100
summary(scf_select$kgstmf_stmf[scf_select$kgstmf_stmf!=0])

#-------------------> financial sophistication index----
vars_quan =c("share_intdivinc",
             "share_risky","fin_hhi")
vars_cat = c("bfinpro","ifinpro","bfinplan","ifinplan",
             "hdeq","scf_under",
             "hbrok","hmmf","hnmmf",
             "risk_high","budget_horizon_long")
vars_quan = vars_quan[which(vars_quan %in% names(scf_select))]
vars_cat = vars_cat[which(vars_cat %in% names(scf_select))]

data = scf_select
for(vv in vars_cat){
  data[[vv]]=as.character(data[[vv]])
}

#__mixed components PCA analysis----

pca_mix= PCAmix(X.quanti = data[,vars_quan],
       X.quali = data[,vars_cat],
       ndim = 2, rename.level=T,graph = F)

pca_mix$scores[,2] = -pca_mix$scores[,2]

vars = c(vars_quan,vars_cat)
pca_mix_results = foreach(vv = vars, .combine ="rbind")%do%{

res1 = summary(lm(data = data,formula = data[[vv]] ~pca_mix$scores[,1], weights=wgt ))
res2 = summary(lm(data = data,formula = data[[vv]] ~pca_mix$scores[,2], weights=wgt ))

print(vv)
print(summary(as.numeric(data[[vv]])))
print(res1)


res_data = data.frame(var = vv,
                      year = y,
                      rsq1 = res1$r.squared,
                      loading1 = res1$coefficients[2,1],
                      se_loadings1 = res1$coefficients[2,2],
                      rsq2 = res2$r.squared,
                      loading2 = res2$coefficients[2,1],
                      se_loadings2 = res2$coefficients[2,2])
 return(res_data)
}

stargazer(pca_mix_results[c(1,3,4,5)], summary = F, rownames = F,
          digits = 3)

stargazer(summary(pca_mix$scores[,1]), summary=F)


## Add principal components to data
scf_select$pca_f1 = pca_mix$scores[,1]
scf_select$pca_f2 = pca_mix$scores[,2]
cor(scf_select[,c("pca_f1","pca_f2")])

## assign quantiles of the sophistication factor
scf_select = data.table(scf_select)
scf_select[,pca_f1_above_q4:=(pca_f1>=quantile(pca_f1, probs = 0.75)),]
scf_select[,pca_f1_below_q1:=(pca_f1<=quantile(pca_f1, probs = 0.25)),]
scf_select[,pca_f1_q23:=(pca_f1<quantile(pca_f1, probs = 0.75)&
                         pca_f1>quantile(pca_f1, probs = 0.25))]

scf_select[,pca_f2_abovemedian:=(pca_f2>median(pca_f2)),]

# deposit insurance limit

# deposits exceed the deposit insurance limit
if(y <=2007){
scf_select[, deposits_aboveFDIC:= (deposits>100000)]
scf_select[, cd_aboveFDIC:= (cd_amt>100000)]
}

if(y >=2008){
  scf_select[, deposits_aboveFDIC:= (deposits>250000)]
  scf_select[, cd_aboveFDIC:= (cd_amt>250000)]
}


# ----------------------------------------------------------------------------->
# drop x coded variables
# ----------------------------------------------------------------------------->
scf_select = data.frame(scf_select)
drop_jxvars = names(scf.m)[c(grep("j",names(scf.m)),
                grep("x",names(scf.m)))]

scf_select=scf_select[,-which(names(scf_select) %in% drop_jxvars)]
scf_select$rep = scf_select$y1-scf_select$yy1*10

# ----------------------------------------------------------------------------->
# assign year
# ----------------------------------------------------------------------------->
eval(parse(text = paste0('scf' , y,"=scf_select" )))
eval(parse(text = paste0('rw' , y,"=rw" )))
eval(parse(text = paste0('pca_mix_results' , y,"=pca_mix_results" )))

rm(scf.e,scf.m,rw,pca_mix_results,scf_select)
}

setwd(work_dir)
objtosave = ls(pattern="scf")
objtosave = c(objtosave, ls(pattern="rw"))
objtosave = c(objtosave, ls(pattern="pca_mix_results"))
objtosave = c(objtosave, "inst_code")
objtosave = c(objtosave,"cpi")
objtosave = c(objtosave, "cpi2008")
objtosave = c(objtosave, "year")

save(list = objtosave, file = "SCF_select.rda")
# ----------------------------------------------------------------------------->





