#-------------------------------------------------------------------------------->>
# Author: Vladimir Yankov (vladimir.yankov@gmail.com)

#-------------------------------------------------------------------------------->
# Clear workspace
rm(list = ls())
# Load required libaries
data_dir <-"./Data/"
final_data_dir= "./Data_InSearchOfRiskFreeAsset/MSA_Level_Data/"
str_est_dir   = "./Data_InSearchOfRiskFreeAsset/str_est/"
figures_dir   = "./InSearchRisklessAsset/Results/Figures/"
tables_dir    = "./InSearchRisklessAsset/Results/Tables/"
#-------------------------------------------------------------------------------->
libraries  = c("foreign", "iterators", "foreach", "snow", "doSNOW",
               "xlsx","haven","sandwich","stargazer",
               "lubridate", "data.table", "plm", "zoo","dplyr")
for(l in libraries){
  if(require(l, character.only = T)){
    print(l)
  }
  if(!require(l, character.only = T)){
    install.packages(l)
  }
}
#-------------------------------------------------------------------------------->
# Set path to where the files will be stored under Windows server (RDC)
setwd(data_dir)
#-------------------------------------------------------------------------------->
# user-defined functions
as.numeric.factor <- function(x){as.numeric(levels(x))[x]}
as.character.factor <- function(x){as.character(levels(x))[x]}
source(paste0(code_dir,"PlottingFunctions.R"))
source(paste0(code_dir,"CollapseFunctions.R"))
Pctiles = function(x, type = 1, name = ""){
  t = data.frame(
    mean = mean(x, na.rm = T),
    sd =sd(x,na.rm = T),
    p1 = quantile(x, probs = 0.01, na.rm = TRUE, names = FALSE, type = type),
    p5 =  quantile(x, probs = 0.05, na.rm = TRUE, names = FALSE, type = type),
    p10 =  quantile(x, probs = 0.10, na.rm = TRUE, names = FALSE, type = type),
    p25 =  quantile(x, probs = 0.25, na.rm = TRUE, names = FALSE, type = type),
    p50 =  quantile(x, probs = 0.50, na.rm = TRUE, names = FALSE, type = type),
    p75 =  quantile(x, probs = 0.75, na.rm = TRUE, names = FALSE, type = type),
    p90 =  quantile(x, probs = 0.9, na.rm = TRUE, names = FALSE, type = type),
    p95 =  quantile(x, probs = 0.95, na.rm = TRUE, names = FALSE, type = type),
    p99 =  quantile(x, probs = 0.99, na.rm = TRUE, names = FALSE, type = type),
    min = min(x,na.rm = T),
    max = max(x,na.rm = T))
  names(t) = paste0(name, names(t))
  return(t)
}

decimalplaces <- function(x){
  n = length(x)
  y = rep(NA,n)
  for(i in 1:n){
    z= strsplit(as.character(abs(x[i])), split = "[.]", fixed = F)[[1]]
    y[i] = nchar(z[2])
  }
  return(y)
}
#-------------------------------------------------------------------------------->
# Load data----
#-------------------------------------------------------------------------------->
setwd(data_dir)
load(file = "./RW_data/RW_final/deposits/deposits_fill.rda")
deposits = deposits_fill
rm(deposits_fill)

## Summary of deposits
load(file = "./SOD/sumd_msa_market.rda")
load(file = "./SOD/msa_code.rda")


load(file = "./InterestRates/Treas_FFT_FFF.rda")
load(file = "./InterestRates/Treas_FF_surprise.rda")
load(file = "./CallReports/Call_vars.rda")

##---> BHCs level data
load(file = "./Y_9C/Y9C_vars.rda")

bhc_data=Y9C_vars[,c("id_rssd","date","assets","total_loans_leases",
                     "dep_time_more100K","dep_time_less100K",
                     "share_st_funding","core_dep","deposits","ROE")]

## ---> collapse bank-level data
call_deposit_vars =c("id_rssd", "id_rssd_hh1", "date",
                     "td_mat","dep_mat",
                     "total_deposits","deposits_uninsured",
                     "deposits_listing","deposits_cdar",
                     "time_above100K_q","time_below100K_q",

                     "int_timeabove100K","int_timebelow100K",
                     "int_sav_mmda","int_ff_repo",
                     "brokered_deposits",
                     "brokered_below100K",

                     "deposits_above100K",
                     "deposits_above250K",
                     "num_accounts_above100K",
                     "num_accounts_above250K",
                     "avg_balance_above100K",
                     "avg_balance_above250K",

                     "deposits_below100K",
                     "deposits_below250K",
                     "num_accounts_below100K",
                     "num_accounts_below250K",
                     "avg_balance_below100K",
                     "avg_balance_below250K")

call_deposits = Call_vars[,call_deposit_vars]
call_deposits = data.table(call_deposits)
call_deposits[,year:= year(date)]

call_deposits[,avg_balance_below100K_v2 := max(avg_balance_below100K, na.rm = T),
              by = c("id_rssd","year")]
call_deposits$avg_balance_below100K_v2[is.infinite(call_deposits$avg_balance_below100K_v2)]=NA
call_deposits$avg_balance_below100K[is.na(call_deposits$avg_balance_below100K)]=
  call_deposits$avg_balance_below100K_v2[is.na(call_deposits$avg_balance_below100K)]
call_deposits$num_accounts_below100K[is.na(call_deposits$num_accounts_below100K)]=
  round(call_deposits$deposits_below100K[is.na(call_deposits$num_accounts_below100K)]/call_deposits$avg_balance_below100K_v2[is.na(call_deposits$num_accounts_below100K)],0)
call_deposits[,avg_balance_below100K_v2:=NULL]


call_deposits[,avg_balance_below250K_v2 := max(avg_balance_below250K, na.rm = T),
              by = c("id_rssd","year")]
call_deposits$avg_balance_below250K_v2[is.infinite(call_deposits$avg_balance_below250K_v2)]=NA
call_deposits$avg_balance_below250K[is.na(call_deposits$avg_balance_below250K)]=
  call_deposits$avg_balance_below250K_v2[is.na(call_deposits$avg_balance_below250K)]
call_deposits$num_accounts_below250K[is.na(call_deposits$num_accounts_below250K)]=
  round(call_deposits$deposits_below250K[is.na(call_deposits$num_accounts_below250K)]/call_deposits$avg_balance_below250K_v2[is.na(call_deposits$num_accounts_below250K)],0)
call_deposits[,avg_balance_below250K_v2:=NULL]


call_deposits$id_rssd_top =call_deposits$id_rssd_hh1
call_deposits$id_rssd_top[is.na(call_deposits$id_rssd_top)]=0
call_deposits$id_rssd_top[call_deposits$id_rssd_top==0]=call_deposits$id_rssd[call_deposits$id_rssd_top==0]
#-------------------------------------------------------------------------------->
# collapse at the top holder
#-------------------------------------------------------------------------------->

call_deposits_th = data.table(call_deposits)
call_deposits_th = call_deposits_th[,
                  list(nbanks = length(unique(id_rssd)),
                       td_mat = weighted.mean(td_mat,w = total_deposits, na.rm = T),
                       dep_mat = weighted.mean(dep_mat,w = total_deposits, na.rm = T),

                       int_timebelow100K = mean(int_timebelow100K, na.rm = T),
                       int_timeabove100K = mean(int_timebelow100K, na.rm = T),
                       int_sav_mmda= mean(int_sav_mmda, na.rm = T),
                       int_ff_repo= mean(int_ff_repo, na.rm = T),

                       deposits_cb = sum(total_deposits,na.rm =T),

                       deposits_above100K= sum(deposits_above100K,na.rm =T),
                       deposits_below100K= sum(deposits_below100K,na.rm =T),
                       num_accounts_above100K= sum(num_accounts_above100K,na.rm =T),
                       num_accounts_below100K= sum(num_accounts_below100K,na.rm =T),

                       deposits_above250K= sum(deposits_above250K,na.rm =T),
                       deposits_below250K= sum(deposits_below250K,na.rm =T),
                       num_accounts_above250K= sum(num_accounts_above250K,na.rm =T),
                       num_accounts_below250K= sum(num_accounts_below250K,na.rm =T),

                       avg_balance_below100K = mean(avg_balance_below100K, na.rm=T),
                       avg_balance_above100K = mean(avg_balance_above100K, na.rm=T),
                       avg_balance_below250K = mean(avg_balance_below250K, na.rm=T),
                       avg_balance_above250K = mean(avg_balance_above250K, na.rm=T)),
                  by = c("id_rssd_top","date")]
call_deposits_th = data.frame(call_deposits_th)




#-------------------------------------------------------------------------------->
# SOD -- MSA market level data
#-------------------------------------------------------------------------------->
sumd_msa_market = data.table(sumd_msa_market)
sumd_msa_market[,dep_capita:= deposits_msa/population]


# add set of MSA markets that have structural estimates
setwd(str_est_dir)
load(file = "F_xi_msa_m.rda")
load(file = "qvec_est_msa.rda")

msa_str_est = unique(F_xi_msa_m$msa_code)

#-------------------------------------------------------------------------------->
# Summary table of MSA market level information for 2007----
market_msa_2007 = sumd_msa_market[which(sumd_msa_market$year==2007) &
                                    sumd_msa_market$msa_code %in% msa_str_est,]
market_msa_2007 = data.table(market_msa_2007)
table_msa = rbind(Pctiles(market_msa_2007$pop_total)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(100*market_msa_2007$pop65plus/market_msa_2007$pop_total)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(market_msa_2007$income_capita)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(market_msa_2007$dep_capita)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(market_msa_2007$n_banks_msa)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(market_msa_2007$n_branches_msa)[c("min", "p25", "p50", "p75", "max")],
                  #Pctiles(market_msa_2007$c1)[c("min", "p25", "p50", "p75", "max")],
                  round(100*Pctiles(market_msa_2007$HHI)[c("min", "p25", "p50", "p75", "max")],2))
table_msa = data.frame(table_msa)
table_msa$var = c("Population","Share 65+",
                  "Income per capita",
                  "Deposits per capita",
                  "Number of banks",
                  "Number of branches",
                  "HHI index")
table_msa = data.table(table_msa)
table_msa = table_msa[, list(var,min, p25, p50, p75, max)]
stargazer(table_msa, type = "latex",
          out = paste0(tables_dir,"MSA_summary_stats.tex"),
          #column.labels = c("","min","25th", "median",
          #                  "75th","max"),
          covariate.labels =  c("Population","Share 65+",
                                "Income per capita",
                                "Deposits per capita",
                                "Number of banks",
                                "Number of branches",
                                "HHI index"),
          summary = F, digits= 2, rownames=F)
# deposit pricing at MSA level
length(msa_str_est) # 234

# #-------------------------------------------------------------------------------->
 cluster = makeCluster(12,type="SOCK")
 registerDoSNOW(cluster)
# #------------------------------------------------------------------------------>
select_msa_code =msa_str_est
deposits_cd_msa = foreach(msa =select_msa_code , .combine = "rbind",
                          .packages = c("foreach"),
                          .export= c("Pctiles"))%dopar%{
print(msa)
  deposits_m = deposits[deposits$msa_code==msa & deposits$msa_ind==1,]
  dates_msa = unique(deposits_m$date)

deposits_msa_m = foreach(t = dates_msa, .combine = "rbind")%do%{
    print(t)

    deposits_msa_tt= deposits_m[deposits_m$date==t,]
    deposits_msa_t =cbind(Pctiles(deposits_msa_tt$apy_cd3, name = "apy_cd3_"),
                          Pctiles(deposits_msa_tt$apy_cd6, name = "apy_cd6_"),
                          Pctiles(deposits_msa_tt$apy_cd12, name = "apy_cd12_"),
                          Pctiles(deposits_msa_tt$apy_cd24, name = "apy_cd24_"),
                          Pctiles(deposits_msa_tt$apy_cd60, name = "apy_cd60_"))
    deposits_msa_t$nbanks = length(unique(deposits_msa_tt$id_rssd_hd_off))
    deposits_msa_t$nbranches = length(unique(deposits_msa_tt$acct_nbr))
    deposits_msa_t$date = t
    return(deposits_msa_t)
  }
  deposits_msa_m$msa_code = msa

return(deposits_msa_m)
}
rm(deposits_m,dates_msa)

save(deposits_cd_msa,
     file = paste0(final_data_dir,"deposits_cd_msa.rda"))
#------------------------------------------------------------------------------>
# stop cluster
stopCluster(cluster)
#------------------------------------------------------------------------------>
deposits_cd_msa$year= year(deposits_cd_msa$date)
deposits_cd_msa$week = week(deposits_cd_msa$date)
which(!select_msa_code %in% unique(sumd_msa_market$msa_code))
deposits_cd_msa=deposits_cd_msa[deposits_cd_msa$year<=2011,]
sumd_msa_market = data.table(sumd_msa_market)
sumd_msa_market[, n:=.N, by =  c("msa_code","year")]
summary(sumd_msa_market$n)
sumd_msa_market=sumd_msa_market[sumd_msa_market$n==1,]
sumd_msa_market = data.frame(sumd_msa_market)
sumd_msa_market$n = NULL
deposits_cd_msa = merge(deposits_cd_msa,sumd_msa_market,all.x = T,
                        by = c("msa_code","year"))
load(file = "./InterestRates/Treas_FFT_FFF.rda")
load(file = "./InterestRates/Treas_FF_surprise.rda")
unique(weekdays(unique(deposits_cd_msa$date)))
unique(weekdays(unique(Treasw$date)))
Treasw$date = Treasw$date-4
unique(weekdays(unique(Treasw$date)))
deposits_cd_msa = merge(deposits_cd_msa, Treasw,all.x = T,
                        by = c("date"))
deposits_cd_msa = data.table(deposits_cd_msa)
deposits_cd_msa[,disp_3m_95_5 := apy_cd3_p95-apy_cd3_p5]
deposits_cd_msa[,disp_6m_95_5 := apy_cd6_p95-apy_cd6_p5]
deposits_cd_msa[,disp_12m_95_5 := apy_cd12_p95-apy_cd12_p5]
deposits_cd_msa[,disp_24m_95_5 := apy_cd24_p95-apy_cd24_p5]
deposits_cd_msa[,disp_60m_95_5 := apy_cd60_p95-apy_cd60_p5]
deposits_cd_msa[,spread_3m := treas3mo-apy_cd3_p50]
deposits_cd_msa[,spread_6m := treas6mo-apy_cd6_p50]
deposits_cd_msa[,spread_12m := treas12mo-apy_cd12_p50]




#-------------------------------------------------------------------------------->
# Construct market level regression data----
#-------------------------------------------------------------------------------->
deposits_cd_msa = data.table(deposits_cd_msa)
deposits_cd_msa = deposits_cd_msa[order(deposits_cd_msa$msa_code,deposits_cd_msa$date),]
deposits_cd_msa[,deffr := c(NA,diff(effr))]
deposits_cd_msa[, nbanks_pop:= n_banks_msa/population]
deposits_cd_msa[,n_branches_pop:= n_branches_msa/population]
deposits_cd_msa[, n:= .N, by = c("msa_code","date")]
summary(deposits_cd_msa$n)
deposits_cd_msa = data.frame(deposits_cd_msa)

# standardize variables in the regression
deposits_cd_msa = data.table(deposits_cd_msa)
deposits_cd_msa[, HHI_std:= (HHI - mean(HHI, na.rm =T))/sd(HHI, na.rm =T), by = c("date")]
deposits_cd_msa[, pop_age_65_plus_std:= (pop_age_65_plus - mean(pop_age_65_plus, na.rm =T))/sd(pop_age_65_plus, na.rm =T), by = c("date")]
deposits_cd_msa[, nbanks_pop_std:= (nbanks_pop - mean(nbanks_pop, na.rm =T))/sd(nbanks_pop, na.rm =T), by = c("date")]
deposits_cd_msa[, n_branches_pop_std:= (n_branches_pop - mean(n_branches_pop, na.rm =T))/sd(n_branches_pop, na.rm =T), by = c("date")]
deposits_cd_msa[, dep_capita_std:= (dep_capita - mean(dep_capita, na.rm =T))/sd(dep_capita, na.rm =T), by = c("date")]
deposits_cd_msa[, population_std:= (population - mean(population, na.rm =T))/sd(population, na.rm =T), by = c("date")]
deposits_cd_msa[, income_capita_std:= (income_capita - mean(income_capita, na.rm =T))/sd(income_capita, na.rm =T), by = c("date")]
deposits_cd_msa[,n:=.N, by=c("msa_code","date")]
summary(deposits_cd_msa$n)
deposits_cd_msa$n = NULL
deposits_cd_msa =deposits_cd_msa[order(deposits_cd_msa$msa_code,
                                       deposits_cd_msa$date),]

save(deposits_cd_msa,
     file = paste0(final_data_dir,"deposits_cd_msa.rda"))

#-------------------------------------------------------------------------------->
# Regression analysis on search cost-----
#-------------------------------------------------------------------------------->
setwd(final_data_dir)
load(file=paste0(sod_dir,"sumd_msa_market.rda"))
sumd_msa_market = data.table(sumd_msa_market)
sumd_msa_market[, n:=.N, by = c("msa_code","year")]
summary(sumd_msa_market$n) # check for duplicates
sumd_msa_market[,share65plus:=100*pop_age_65_plus/pop_total]
sumd_msa_market[,share65plus :=100*pop65plus/population]
sumd_msa_market[,dln_pop65plus :=c(NA,diff(log(pop65plus),1)), by = "msa_code"]
sumd_msa_market[,dep_capita :=100*deposits_msa/population]
sumd_msa_market[,pop_banks := 100*population/n_banks_msa]
sumd_msa_market[,pop_br := 100*population/n_branches_msa]
sumd_msa_market[,post2001:=as.numeric(year>2001)] # indicator for post 2001 (pre/post Internet)
sumd_msa_market[,dep_income :=100*deposits_msa/pers_income]
# Merge market information to search-cost estimates
market_Fxi_sumd = merge(market_Fxi, sumd_msa_market, by = c("msa_code","year"))
market_Fxi_sumd[,range := (R_max-R_min)]
unique(market_Fxi_sumd$msa_code)
unique(market_Fxi_sumd$year)
market_Fxi_sumd = market_Fxi_sumd[order(market_Fxi_sumd$msa_code,
                                        market_Fxi_sumd$year),]
market_Fxi_sumd[,t:= seq(.N), by = "msa_code"]
market_Fxi_sumd[,T:= .N, by = "msa_code"]
summary(market_Fxi_sumd[,list(range,share65plus)])
length(unique(market_Fxi_sumd$msa_code[market_Fxi_sumd$T==15]))
market_Fxi_sumd[,log_income_capita:=log(income_capita)]
market_Fxi_sumd[,log_population:=log(population)]
market_Fxi_sumd[,log_pop_banks:=log(pop_banks)]
save(market_Fxi_sumd,file = "market_Fxi_sumd.rda")

#___ Regression search cost median----
xs_xi_median = plm(formula = xi_median ~ share65plus+log(population)+log(pop_banks)+
                     log(income_capita)+dep_income+t,
                   data =market_Fxi_sumd[market_Fxi_sumd$ks_p>0.05,],
                   effect = "time",
                   index = c("msa_code","year"),
                   model = "within",
                   na.action = "na.omit" )
xs_xi_median_se <- sqrt(diag(vcovHC(xs_xi_median,type="HC0",cluster="group")))
summary(xs_xi_median)
r.squared(xs_xi_median)

#___ Regression Search cost mean-----
xs_xi_mean = plm(formula = xi_mean ~ share65plus+log(population)+log(pop_banks)+
                   log(income_capita)+dep_income+t,
                 data =market_Fxi_sumd[market_Fxi_sumd$ks_p>0.05,],
                 effect = "time",
                 index = c("msa_code","year"),
                 model = "within",
                 na.action = "na.omit" )
xs_xi_mean_se <- sqrt(diag(vcovHC(xs_xi_mean,type="HC0",cluster="group")))
summary(xs_xi_mean)
r.squared(xs_xi_mean)


#___ Regression Search cost standard deviation-----
xs_xi_sd = plm(formula = xi_sd ~ share65plus+log(population)+log(pop_banks)+
                 log(income_capita)+dep_income+t,
               data =market_Fxi_sumd[market_Fxi_sumd$ks_p>0.05,],
               effect = "twoway",
               index = c("msa_code","year"),
               model = "within",
               na.action = "na.omit" )
xs_xi_sd_se <- sqrt(diag(vcovHC(xs_xi_sd,type="HC0",cluster="group")))
summary(xs_xi_sd)
#___ Regression Search cost 25th percentile-----
xs_xi_p25 = plm(formula = xi_p25 ~ share65plus+log(population)+log(pop_banks)+
                  log(income_capita)+dep_income+t,
                data =market_Fxi_sumd[market_Fxi_sumd$ks_p>0.05,],
                effect = "twoway",
                index = c("msa_code","year"),
                model = "within",
                na.action = "na.omit" )
xs_xi_p25_se <- sqrt(diag(vcovHC(xs_xi_p25,type="HC0",cluster="group")))
summary(xs_xi_p25)
#___ Regression Search cost 75th percentile-----
xs_xi_p75 = plm(formula = xi_p75 ~ share65plus+log(population)+log(pop_banks)+
                  log(income_capita)+dep_income+t,
                data =market_Fxi_sumd[market_Fxi_sumd$ks_p>0.05,],
                effect = "twoway",
                index = c("msa_code","year"),
                model = "within",
                na.action = "na.omit" )
xs_xi_p75_se <- sqrt(diag(vcovHC(xs_xi_p75,type="HC0",cluster="group")))
summary(xs_xi_p75)
sum(xs_xi_mean$residuals^2)/xs_xi_mean$df.residual
sum(xs_xi_sd$residuals^2)/xs_xi_mean$df.residual
#___ Export regression results-----
# This is Table 11 "Search cost distribution and market characteristics
stargazer(xs_xi_mean,xs_xi_sd,
          covariate.labels = c("Share population age 65+",
                               "log(Population)",
                               #"HHI",
                               "log(Population per bank)",
                               #"log(Population per branch)",
                               "log(Income per capita)",
                               "Deposits/Income",
                               "Time trend"),
          out = paste0(tables_dir, "SearchCost_MSA.tex"),
          se = list(NULL,
                    xs_xi_mean_se,
                    xs_xi_sd_se))
coeftest(xs_xi_median, vcov.=vcovHC(xs_xi_median, method="arellano", type="HC1"))
coeftest(xs_xi_sd, vcov.=vcovHC(xs_xi_sd, method="arellano", type="HC1"))
coeftest(xs_xi_p25, vcov.=vcovHC(xs_xi_p25, method="arellano", type="HC1"))

#-------------------------------------------------------------------------------->
# Fit parametric distribution-----
#-------------------------------------------------------------------------------->
# model y ~ share65plus+log(population)+log(pop_banks)+log(income_capita)+dep_income+t
summary(qvec_msa_data)
qvec_msa_data[,share65plus:=100*pop_age_65_plus/pop_total]
qvec_msa_data[,share65plus :=100*pop65plus/population]
qvec_msa_data[,dln_pop65plus :=c(NA,diff(log(pop65plus),1)), by = "msa_code"]
qvec_msa_data[,dep_capita :=100*deposits_msa/population]
qvec_msa_data[,pop_banks := 100*population/n_banks_msa]
qvec_msa_data[,pop_br := 100*population/n_branches_msa]
qvec_msa_data[,post2001:=as.numeric(year>2001)]
qvec_msa_data[,dep_income :=100*deposits_msa/pers_income]
F_xi_msa_y = F_xi_msa_m[month(F_xi_msa_m$date)==6,]
F_xi_msa_y[,year:=year(date)]
table(F_xi_msa_y$year)
F_xi_msa_y = merge(qvec_msa_data,F_xi_msa_y, by = c("msa_code","year"))
F_xi_msa_y[,log_pop_banks:=log(pop_banks)]
F_xi_msa_y[,log_population:=log(population)]
F_xi_msa_y[,log_income_capita:=log(income_capita)]

summary(F_xi_msa_y[F_xi_msa_y$ks_p.x>0.05,list(share65plus,log_population,log_pop_banks,dep_income)])
summary(market_Fxi_sumd[market_Fxi_sumd$ks_p>0.05,list(share65plus,log_population,log_pop_banks,dep_income)])
#___ Log-normal----
model_nls <-function(
  beta_share65plus,beta_log_pop,
  beta_log_pop_banks,beta_log_income_capita,
  beta_dep_income,beta_t,
  gamma_share65plus,
  gamma_log_pop,
  gamma_log_pop_banks,gamma_log_income_capita,
  gamma_dep_income,gamma_t,
  xi, share65plus,log_population, log_pop_banks, log_income_capita,dep_income,t){

  mu_x =  beta_share65plus*share65plus+beta_log_pop*log_population+
    beta_log_pop_banks*log_pop_banks+
    beta_log_income_capita*log_income_capita+beta_dep_income*dep_income+beta_t*t

  sd_x = gamma_share65plus*share65plus+gamma_log_pop*log_population+
    gamma_log_pop_banks*log_pop_banks+
    gamma_log_income_capita*log_income_capita+gamma_dep_income*dep_income+gamma_t*t

  y = plnorm(xi,
             meanlog= log( mu_x^2/(sqrt(mu_x^2 + sd_x^2))),
             sdlog= log(1+sd_x^2/mu_x^2))

  y[which(is.na(y)|is.infinite(y))] = -10^6
  return(y)
}
names(xs_xi_mean$coefficients)
# [1] "share65plus"        "log(population)"    "log(pop_banks)"
# [4] "log(income_capita)" "dep_income"         "t"
# Initialize the optimization with the coefficients from the panel regression
start =model_nls(
  #beta_const =-0.2,
  beta_share65plus=xs_xi_mean$coefficients[1],
  beta_log_pop =xs_xi_mean$coefficients[2],
  beta_log_pop_banks=xs_xi_mean$coefficients[3],
  beta_log_income_capita=xs_xi_mean$coefficients[4],
  beta_dep_income = xs_xi_mean$coefficients[5],
  beta_t=xs_xi_mean$coefficients[6],

  #gamma_const = 0.1,
  gamma_share65plus=xs_xi_mean_se[1],
  gamma_log_pop =xs_xi_mean_se[2],
  gamma_log_pop_banks=xs_xi_mean_se[3],
  gamma_log_income_capita=xs_xi_mean_se[4],
  gamma_dep_income = xs_xi_mean_se[5],
  gamma_t=xs_xi_mean_se[6],
  xi =mean(F_xi_msa_y$xi),
  share65plus=mean(F_xi_msa_y$share65plus),
  log_population = mean(F_xi_msa_y$log_population),
  log_pop_banks = mean(F_xi_msa_y$log_pop_banks),
  log_income_capita = mean(F_xi_msa_y$log_income_capita),
  dep_income = mean(F_xi_msa_y$dep_income),
  t = mean(F_xi_msa_y$t))
summary(start)


start =model_nls(
  #beta_const =0,
  beta_share65plus=0,
  beta_log_pop =0,
  beta_log_pop_banks=0,
  beta_log_income_capita=0,
  beta_dep_income = 0,
  beta_t=0,

  #gamma_const = 0.1,
  gamma_share65plus=xs_xi_mean_se[1],
  gamma_log_pop =xs_xi_mean_se[2],
  gamma_log_pop_banks=xs_xi_mean_se[3],
  gamma_log_income_capita=xs_xi_mean_se[4],
  gamma_dep_income = xs_xi_mean_se[5],
  gamma_t=xs_xi_mean_se[6],
  xi =F_xi_msa_y$xi,
  share65plus=F_xi_msa_y$share65plus,
  log_population = F_xi_msa_y$log_population,
  log_pop_banks = F_xi_msa_y$log_pop_banks,
  log_income_capita = F_xi_msa_y$log_income_capita,
  dep_income = F_xi_msa_y$dep_income,
  t = F_xi_msa_y$t)
summary(start)




result.nls <- nls(formula = Fxi ~ model_nls(beta_share65plus,beta_log_pop,
                                            beta_log_pop_banks,beta_log_income_capita,
                                            beta_dep_income,beta_t,
                                            gamma_share65plus,
                                            gamma_log_pop,
                                            gamma_log_pop_banks,gamma_log_income_capita,
                                            gamma_dep_income,gamma_t,
                                            xi, share65plus,log_population, log_pop_banks, log_income_capita,dep_income,t),
                  na.action = na.omit,
                  control = list(printEval= T,
                                 gr = "BFGS",
                                 maxiter = 10000,
                                 warnOnly=T,minFactor=1e-16,
                                 tol=1e-6, trace = T),
                  data = F_xi_msa_y[F_xi_msa_y$ks_p.x>0.05,],
                  model = T,
                  start = list(
                    #beta_const = 1,
                    beta_share65plus=xs_xi_mean$coefficients[1]*runif(1),
                    beta_log_pop =xs_xi_mean$coefficients[2]*runif(1),
                    beta_log_pop_banks=xs_xi_mean$coefficients[3]*runif(1),
                    beta_log_income_capita=xs_xi_mean$coefficients[4]*runif(1),
                    beta_dep_income = xs_xi_mean$coefficients[5]*runif(1),
                    beta_t=xs_xi_mean$coefficients[6]*runif(1),

                    #gamma_const = 1,
                    gamma_share65plus=xs_xi_sd$coefficients[1]*runif(1),
                    gamma_log_pop =xs_xi_sd$coefficients[2]*runif(1),
                    gamma_log_pop_banks=xs_xi_sd$coefficients[3]*runif(1),
                    gamma_log_income_capita=xs_xi_sd$coefficients[4]*runif(1),
                    gamma_dep_income = xs_xi_sd$coefficients[5]*runif(1),
                    gamma_t=xs_xi_sd$coefficients[6]*runif(1))

)
warnings()
summary(result.nls)
#___ Export retuls from log-normal fit in latex-----
texreg(list(result.nls),
       stars = c(0.01, 0.05,0.1),
       digits=4)
setwd(tables_dir)
save(result.nls,summary.results.nls, file = paste0("result.nls.",
                                                   month(today()),"_",day(today()),".rda"))
summary.results.nls<-summary(result.nls)
summary.results.nls$coefficients
summary.results.nls
sum(summary.results.nls$residuals^2)/summary.results.nls$df[2]
#-------------------------------------------------------------------------------->
