#-------------------------------------------------------------------------------->
# Description: Summary statistics, stylized facts in Section 2 of
# ``In Search of a Risk-free Asset''
# Author: Vladimir Yankov
# Date: August 2016
#-------------------------------------------------------------------------------->
# Clear workspace
rm(list = ls())
# Load required libaries
final_data_dir<- "./Data_InSearchOfRiskFreeAsset/"
figures_dir = "./InSearchRisklessAsset/Results/Figures/"
tables_dir = "./InSearchRisklessAsset/Results/Tables/"
dev.off()
#-------------------------------------------------------------------------------->
libraries  = c("foreign", "iterators", "foreach", "snow", "doSNOW","doParallel",
               "sandwich","stargazer","neldermead",
               "nleqslv","DEoptim","R.matlab","latex2exp",
               "lubridate", "data.table", "plm", "zoo","dplyr")
for(l in libraries){
  if(require(l, character.only = T)){
    print(l)
  }
  if(!require(l, character.only = T)){
    install.packages(l)
  }
}
# Set multi-threading
setMKLthreads(18)
#-------------------------------------------------------------------------------->
# Set path to where the files will be stored under Windows server (RDC)
data_dir =paste0(root_dir,"Data/")
work_dir <-paste0(root_dir,"Data/SOD/")
NIC_Attr <- paste0(root_dir, "Data/NIC/")
NIC_dir <- paste0(root_dir, "Data/NIC/")
code_dir = paste0(root_dir,"Code/R/Tools/")
#-------------------------------------------------------------------------------->
# user-defined functions
as.numeric.factor <- function(x){as.numeric(levels(x))[x]}
as.character.factor <- function(x){as.character(levels(x))[x]}
source(paste0(code_dir,"PlottingFunctions.R"))
source(paste0(code_dir,"CollapseFunctions.R"))
Pctiles = function(x, type = 1, name = ""){
  t = data.frame(
    p1 = quantile(x, probs = 0.01, na.rm = TRUE, names = FALSE, type = type),
    p5 =  quantile(x, probs = 0.05, na.rm = TRUE, names = FALSE, type = type),
    p10 =  quantile(x, probs = 0.10, na.rm = TRUE, names = FALSE, type = type),
    p25 =  quantile(x, probs = 0.25, na.rm = TRUE, names = FALSE, type = type),
    p50 =  quantile(x, probs = 0.50, na.rm = TRUE, names = FALSE, type = type),
    p75 =  quantile(x, probs = 0.75, na.rm = TRUE, names = FALSE, type = type),
    p90 =  quantile(x, probs = 0.9, na.rm = TRUE, names = FALSE, type = type),
    p95 =  quantile(x, probs = 0.95, na.rm = TRUE, names = FALSE, type = type),
    p99 =  quantile(x, probs = 0.99, na.rm = TRUE, names = FALSE, type = type),
    mean = mean(x, na.rm = T),
    sd = sd(x, na.rm = T),
    n = length(x[!is.na(x)])
  )
  names(t) = paste0(name, names(t))
  return(t)
}

decimalplaces <- function(x){
  n = length(x)
  y = rep(NA,n)
  for(i in 1:n){
    z= strsplit(as.character(abs(x[i])), split = "[.]", fixed = F)[[1]]
    y[i] = nchar(z[2])
  }
  return(y)
}
#-------------------------------------------------------------------------------->>>
# Load data----
#-------------------------------------------------------------------------------->>
setwd(data_dir)
setwd("./RW_data/RW_final/deposits/")
load(file = "deposits_fill.rda")
setwd(data_dir)
# SOD data
load(file = "./SOD/sumd_1975_2014.rda")
load(file = "./SOD/sumd_market.rda")
load(file = "./SOD/sumd_msa_market.rda")
## =====> add top holder information
load("./NIC/toph.rdata")
names(toph) = tolower(names(toph))
toph = unique(toph[toph$holder_type =="BHC" &
            toph$ctrl_flg == "Y" &
            toph$unique_tier_ind=="Y",
            c("d_dt_start", "d_dt_end", "id_rssd", "id_rssd_top")])
toph = data.table(toph)
toph = toph[,list(dend = max(d_dt_end),
                  dstart = max(d_dt_start)),
            by = c("id_rssd", "id_rssd_top")]


## load FF data
setwd(data_dir)
load(file = "./InterestRates/FFT_Treas.rda")
#load(file = "./InterestRates/Treas_FFT_FFF.rda")
#load(file = "./InterestRates/Treas_FF_surprise.rda")
load(file = "./CallReports/Call_funding.rda")
load(file = "./CallReports/Call_assets.rda")
load(file = "./Y_9C/Y9C_vars.rda")
dates = unique(deposits_fill$date)
dates=dates[order(dates)]

#------------------------------------------------------------------------------>
# Construct percentiles
cluster = makeCluster(20, type = "SOCK")
registerDoSNOW(cluster)
registerDoParallel(cl = cluster, cores =20)
#------------------------------------------------------------------------------>
cd_small_agg = foreach(d = dates, .combine = "rbind",
                       .packages = c("foreach"),
                       .export=c("Pctiles"))%do%{
                         print(d)
  data = deposits_fill[deposits_fill$date==d,]
  apy_vars = names(data)[grep("apy_", names(data))]

  pctiles = foreach(v = apy_vars, .combine = "cbind")%do%{
    pct = Pctiles(data[[v]], type=1,name = paste0(v,"_"))
    return(pct)
  }
  pctiles$date = d
  return(pctiles)
}
cd_small_agg = cd_small_agg[order(cd_small_agg$date),]
save(cd_small_agg, file ="cd_small_agg.rda")
setwd(paste0(data_dir,"RW_data/RW_final/deposits_fill/"))
#----------------------------------------------------------------------------->
load("./RW_data/RW_final/deposits/cd_small_agg.rda")
#-----> drop outliers
#dates_drop = c("2002-11-22")
cd_small_agg = cd_small_agg[-1,]
#cd_small_agg = cd_small_agg[-which(cd_small_agg$date %in% dates_drop)]
## ---> merge with treasury data
table(weekdays(cd_small_agg$date))
table(weekdays(Treasw$date))
cd_small_agg = merge(cd_small_agg, Treasw, by = "date", all = F)
max(cd_small_agg$date)
#-------------------------------------------------------------------------------->
#                                 Figure 1----
#-------------------------------------------------------------------------------->
# Plot the distribution of yields
w = 10
h = 1.61803
date_axis = unique(cd_small_agg$date)
date_axis = date_axis[which(month(date_axis)==6)]
date_axis2 = seq(min(date_axis), max(date_axis), by = 365)
date_axis2 = c(date_axis2,max(cd_small_agg$date))
setwd(figures_dir)

#---------------> 3 month----
dev.off()
pdf(file = paste0(figures_dir,"APY3.pdf"), width = w , height = w/h )
with(cd_small_agg, plot(date, apy_cd3_p99, type = "l",
                        xaxt = "n",
                        xlab = "",
                        ylab = "",
                        main = "",
                        lwd = 2,
                        ylim = c(0,6.5),
                        col = "lightgray"))
axis.Date(side = 1, cd_small_agg$date, at =date_axis2 , format = "%m-%Y", las = 2)

par(new = T)
grid(NULL,NULL, col = "black", lty = "dotted", lwd = 1, equilogs = F)

with(cd_small_agg, lines(date, apy_cd3_p1, col = "grey"))

with(cd_small_agg, polygon(c(date, rev(date)), c(apy_cd3_p1,rev(apy_cd3_p99)),
                           col = "grey", border = "grey70"))

with(cd_small_agg, polygon(c(date, rev(date)), c(apy_cd3_p25,rev(apy_cd3_p75)),
                           col = "grey37", border = "grey37"))

with(cd_small_agg, lines(date, apy_cd3_p50, col = "black", lwd = 2))
with(cd_small_agg, lines(date, apy_cd3_p95, col = "red",lty =2, lwd = 1))
with(cd_small_agg, lines(date, apy_cd3_p5, col = "red",lty =2, lwd = 1))

with(cd_small_agg, lines(date, tffr, col = "blue",lty =1, lwd = 2))
#with(cd_small_agg, lines(date, libor3mo, col = "purple",lty =1, lwd = 2))


# legend("topright", c("Target Federal Funds Rate","Median", "25/75th pctile",
#                      "5/95th pctile","1/99th pctile"),
#        cex = 1.2,
#        col = c("blue",
#                "black",
#                "grey37",
#                "red",
#                "grey70"),
#        #bty = "n",
#        bg = "white",
#        box.lwd = 0,
#        lty = c(1,1,2,1,1))

legend("topright", c("Target Federal Funds Rate","Median", "25/75th pctile",
                     "5/95th pctile","1/99th pctile"),
       cex = 1.2,
       col = c("blue",
               "black",
               "grey37",
               "red",
               "grey70"),
       lwd = c(2,2,10,1,10),
       #bty = "n",
       bg = "white",
       box.lwd = 0,
       lty = c(1,1,2,1,1))


abline(h = 0, col = "black")

dev.off()
#-------------> 60-month----
pdf(file = paste0(figures_dir,"APY60.pdf"), width = w , height = w/h )
with(cd_small_agg, plot(date, apy_cd60_p99, type = "l",
                        xaxt = "n",
                        xlab = "",
                        ylab = "",
                        main = "",
                        lwd = 2,
                        ylim = c(0,8),
                        col = "lightgray"))
axis.Date(side = 1, cd_small_agg$date, at =date_axis2 , format = "%m-%Y", las = 2)

par(new = T)
grid(NULL,NULL, col = "black", lty = "dotted", lwd = 1, equilogs = F)

with(cd_small_agg, lines(date, apy_cd60_p1, col = "grey"))

with(cd_small_agg, polygon(c(date, rev(date)), c(apy_cd60_p1,rev(apy_cd60_p99)),
                           col = "grey", border = "grey70"))

with(cd_small_agg, polygon(c(date, rev(date)), c(apy_cd60_p25,rev(apy_cd60_p75)),
                           col = "grey37", border = "grey37"))

with(cd_small_agg, lines(date, apy_cd60_p50, col = "black", lwd = 2))
with(cd_small_agg, lines(date, apy_cd60_p95, col = "red",lty =2, lwd = 1))
with(cd_small_agg, lines(date, apy_cd60_p5, col = "red",lty =2, lwd = 1))

with(cd_small_agg, lines(date, treas60mo, col = "blue",lty =1, lwd = 2))


legend("topright", c("5-year Treasury","Median", "25/75th pctile",
                     "5/95th pctile","1/99th pctile"),
       cex = 0.7,
       col = c("blue",
               "black",
               "grey37",
               "red",
               "grey70"),
       lwd = c(2,1,10,1,10),bg = "white",box.lwd = 0.5,
       lty = c(1,1,2,1,1))
abline(h = 0, col = "black")

dev.off()

#>12 month-----

pdf(file = paste0(figures_dir,"APY12.pdf"), width = w , height = w/h )
with(cd_small_agg, plot(date, apy_cd12_p99, type = "l",
                        xaxt = "n",
                        xlab = "",
                        ylab = "",
                        main = "",
                        lwd = 2,
                        ylim = c(0,8),
                        col = "lightgray"))
axis.Date(side = 1, cd_small_agg$date, at =date_axis2 , format = "%m-%Y", las = 2)

par(new = T)
grid(NULL,NULL, col = "black", lty = "dotted", lwd = 1, equilogs = F)

with(cd_small_agg, lines(date, apy_cd12_p1, col = "grey"))

with(cd_small_agg, polygon(c(date, rev(date)), c(apy_cd12_p1,rev(apy_cd12_p99)),
                           col = "grey", border = "grey70"))

with(cd_small_agg, polygon(c(date, rev(date)), c(apy_cd12_p25,rev(apy_cd12_p75)),
                           col = "grey37", border = "grey37"))

with(cd_small_agg, lines(date, apy_cd12_p50, col = "black", lwd = 2))
with(cd_small_agg, lines(date, apy_cd12_p95, col = "red",lty =2, lwd = 1))
with(cd_small_agg, lines(date, apy_cd12_p5, col = "red",lty =2, lwd = 1))

with(cd_small_agg, lines(date, libor12mo, col = "blue",lty =1, lwd = 2))


legend("topright", c("1-year LIBOR","Median", "25/75th pctile",
                     "5/95th pctile","1/99th pctile"),
       cex = 1.2,
       col = c("blue",
               "black",
               "grey37",
               "red",
               "grey70"),
       lwd = c(2,1,10,1,10),
       #bty = "n",
       bg = "white",
       box.lwd = 0,
       lty = c(1,1,2,1,1))
abline(h = 0, col = "black")

dev.off()



#-------------------------------------------------------------------------------->
#-----------------------> Summary statistics on banks' deposit funding----
deposits_2006_top20 = Y9C_vars[Y9C_vars$date == "2006-06-30" &
                                 Y9C_vars$hh_id_rssd1==0,
                               c("name","id_rssd", "assets","deposits_fill",
                                 "dep_time_less100K","dep_time_more100K")]


deposits_2006_top20 = data.table(deposits_2006_top20)
deposits_2006_top20$dep_rank = rank(-deposits_2006_top20$deposits_fill)
deposits_2006_top20[,dep_agg := sum(deposits_fill, na.rm = T)]
deposits_2006_top20[,dep_top20:= sum(deposits_fill[dep_rank <=20], na.rm = T)]
deposits_2006_top20[,dep_top20:=100*dep_top20/dep_agg ]
deposits_2006_top20[,dep_time:=dep_time_less100K+dep_time_more100K]
deposits_2006_top20[,share_time:=round(100*dep_time/deposits_fill,1)]

deposits_2006_top20 = deposits_2006_top20[order(-deposits_2006_top20$deposits_fill),]
ids_top10 = unique(deposits_2006_top20$id_rssd)
## add count for the number of states
sumd_2006 = sumd_bank[sumd_bank$year==2005,]
sumd_id_states = data.table(sumd)
sumd_id_states = sumd_id_states[sumd_id_states$year ==2005, list(n_states = length(unique(state_code))), by = "id_rssd"]
sumd_2006 = merge(sumd_2006,sumd_id_states, by = "id_rssd")
sumd_2006 = data.table(sumd_2006)
sumd_2006 = sumd_2006[,list(nbanks = length(unique(id_rssd)),
                           n_branches = sum(n_branches, na.rm = T),
                           n_msa_markets = sum(n_msa_markets, na.rm = T),
                           n_states = sum(n_states, na.rm = T)),
                      by = c("id_rssd_top")]
sumd_2006 = data.frame(sumd_2006)
deposits_2006_top20 = merge(deposits_2006_top20, sumd_2006,
                            by.x = "id_rssd",by.y ="id_rssd_top" )
deposits_2006_top20 = merge(deposits_2006_top20, call_th,
                            by.x = "id_rssd",by.y ="id_rssd_top" )
deposits_2006_top20 = data.frame(deposits_2006_top20)
deposits_2006_top20 = deposits_2006_top20[,c("dep_rank","name","assets", "deposits_fill","share_time",
                                             "n_branches","n_msa_markets",
                                            "deposits_above100K", "deposits_below100K",
                                             "num_accounts_above100K", "num_accounts_below100K",
                                            "n_states")]
deposits_2006_top20$avg_balance_below100k =round(deposits_2006_top20$deposits_below100K/deposits_2006_top20$num_accounts_below100K,1)
deposits_2006_top20$avg_balance_above100k = round(deposits_2006_top20$deposits_above100K/deposits_2006_top20$num_accounts_above100K,1)
deposits_2006_top20$num_accounts_above100K = round(deposits_2006_top20$num_accounts_above100K/1000,0)
deposits_2006_top20$num_accounts_below100K = round(deposits_2006_top20$num_accounts_below100K/1000,0)
deposits_2006_top20$share_insured = round(100*deposits_2006_top20$deposits_below100K/(deposits_2006_top20$deposits_above100K+deposits_2006_top20$deposits_below100K),1)
deposits_2006_top20 = deposits_2006_top20[,c("dep_rank","name","assets", "deposits_fill","share_time",
                                             "share_insured",
                                             "num_accounts_below100K",
                                             "avg_balance_below100k",
                                             "num_accounts_above100K",
                                             "avg_balance_above100k",
                                             "n_branches","n_msa_markets", "n_states")]
deposits_2006_top20$deposits_fill = round(deposits_2006_top20$deposits_fill/10^6,0)
deposits_2006_top20$assets = round(deposits_2006_top20$assets/10^6,0)
deposits_2006_all = data.table(deposits_2006_top20)
deposits_2006_all = deposits_2006_all[deposits_2006_all$dep_rank>20, list(assets = weighted.mean(assets,w =deposits_fill, na.rm = T),
                                             deposits_fill = weighted.mean(deposits_fill,w =deposits_fill, na.rm =T),
                                             share_time = weighted.mean(share_time, w=deposits_fill, na.rm = T),
                                             share_insured = weighted.mean(share_insured, w= deposits_fill, na.rm =T),
                                             num_accounts_below100K = round(weighted.mean(num_accounts_below100K, w=deposits_fill, na.rm = T),0),
                                             avg_balance_below100k = weighted.mean(avg_balance_below100k, w=deposits_fill),
                                             num_accounts_above100K = round(weighted.mean(num_accounts_above100K, w=deposits_fill, na.rm = T)),
                                             avg_balance_above100k = weighted.mean(avg_balance_above100k, w= deposits_fill, na.rm = T),
                                             n_branches = round(weighted.mean(n_branches, w = deposits_fill, na.rm = T),0),
                                             n_msa_markets = round(weighted.mean(n_msa_markets, w = deposits_fill, na.rm = T),0),
                                             n_states = round(weighted.mean(n_states, w= deposits_fill, na.rm = T),0)),]

## keep only top 20 banks
deposits_2006_top20 = data.frame(deposits_2006_top20)
deposits_2006_top20 = deposits_2006_top20[deposits_2006_top20$dep_rank<=20,]
deposits_2006_top20 = deposits_2006_top20[order(deposits_2006_top20$dep_rank),]
deposits_2006_top20$dep_rank=NULL


##----> export the summary largest banks table to LaTeX----
deposits_2006_top20 = deposits_2006_top20[order(-deposits_2006_top20$deposits_fill),]
## replace names
deposits_2006_top20$name[deposits_2006_top20$name=="BANK OF AMERICA CORPORATION"]=
  "Bank of America"
deposits_2006_top20$name[deposits_2006_top20$name=="JPMORGAN CHASE & CO."]=
  "JPMorgan Chase"
deposits_2006_top20$name[deposits_2006_top20$name=="WACHOVIA CORPORATION"]=
  "Wachovia"
deposits_2006_top20$name[deposits_2006_top20$name=="WELLS FARGO & COMPANY"]=
  "Wells Fargo"
deposits_2006_top20$name[deposits_2006_top20$name=="CITIGROUP INC."]=
  "Citigroup"
deposits_2006_top20$name[deposits_2006_top20$name=="U.S. BANCORP"]=
  "U.S.B."
deposits_2006_top20$name[deposits_2006_top20$name=="SUNTRUST BANKS, INC."]=
  "Suntrust"
deposits_2006_top20$name[deposits_2006_top20$name=="NATIONAL CITY CORPORATION"]=
  "National City"
deposits_2006_top20$name[deposits_2006_top20$name=="BB&T CORPORATION"]=
  "BB\\&T"
deposits_2006_top20$name[deposits_2006_top20$name=="BANK OF NEW YORK COMPANY, INC., THE"]=
  "BNY"
deposits_2006_top20$name[deposits_2006_top20$name=="FIFTH THIRD BANCORP"]=
  "Fifth Third"
deposits_2006_top20$name[deposits_2006_top20$name=="PNC FINANCIAL SERVICES GROUP, INC., THE"]=
  "PNC"
deposits_2006_top20$name[deposits_2006_top20$name=="REGIONS FINANCIAL CORPORATION"]=
  "Regions"
deposits_2006_top20$name[deposits_2006_top20$name=="KEYCORP"]=
  "Keycorp"
deposits_2006_top20$name[deposits_2006_top20$name=="COMERICA INCORPORATED"]=
  "Comerica"
deposits_2006_top20$name[deposits_2006_top20$name=="NORTH FORK BANCORPORATION, INC."]=
  "North Fork"
deposits_2006_top20$name[deposits_2006_top20$name=="AMSOUTH BANCORPORATION"]=
  "Amsouth"
deposits_2006_top20$name[deposits_2006_top20$name=="COMMERCE BANCORP, INC."]=
  "Commerce"
deposits_2006_top20$name[deposits_2006_top20$name=="COUNTRYWIDE FINANCIAL CORPORATION"]=
  "Countrywide"
deposits_2006_top20$name[deposits_2006_top20$name=="MBNA CORPORATION"]=
  "MBNA"
stargazer(deposits_2006_top20, type = "latex", summary = F, rownames =F, digits = 1)
stargazer(deposits_2006_all, type = "latex", summary = F, rownames =F, digits = 1)
write.csv(deposits_2006_top20, file = paste0(tables_dir, "deposits_2006_top20.csv"))

#-------------------------------------------------------------------------------->
# Within bank rate dispersion across markets/rate setting branches----
#-------------------------------------------------------------------------------->
deposits_fill = data.table(deposits_fill)
deposits_id_count = deposits_fill[year(deposits_fill$date)==2006,
                             list(n_br = length(unique(acct_nbr)),
                                  n_banks = length(unique(id_rssd_hd_off))),
                             by = c("id_rssd_top")]

deposits_rt_2006 = unique(deposits_fill[year(deposits_fill$date)==2006,
                                        list(id_rssd_top,id_rssd_hd_off,
                                                            acct_nbr)])
deposits_rt_2006 = data.frame(deposits_rt_2006)
deposits_rt_2006 = merge(deposits_rt_2006,
                         acct_join_info[,c("acct_nbr_rt","acct_nbr_loc","id_rssd")],
                         by.x = "acct_nbr", by.y = "acct_nbr_rt")

length(unique(deposits_rt_2006$acct_nbr[deposits_rt_2006$id_rssd_top==1120754]))
length(unique(deposits_rt_2006$acct_nbr_loc[deposits_rt_2006$id_rssd_top==1120754]))

#-------------------------------------------------------------------------------->
## Within MSA market rate dispersion-----
#-------------------------------------------------------------------------------->
## --> rate setting branch network for the top 5 banks
d = "2006-06-30"
deposits_2006_top5 = Y9C_vars[Y9C_vars$date == "2006-06-30" &
                                 Y9C_vars$hh_id_rssd1==0,
                               c("name","id_rssd", "deposits_fill")]
deposits_2006_top5 = deposits_2006_top5[order(-deposits_2006_top5$deposits_fill),]
deposits_2006_top5 = deposits_2006_top5[1:10,]


acct_info_2006 = acct_info_all_BK_msa[acct_info_all_BK_msa$d_end >= d &
                                      acct_info_all_BK_msa$d_start <=d,
                                      ]
acct_join_info_2006 = acct_join_info[acct_join_info$d_end >= d &
                                       acct_join_info$d_start <=d &
                                       acct_join_info$state_cd <= 56,]

th2005 = toph[toph$dend >= d & toph$dstart < d,]
acct_info_2006 = merge(acct_info_2006, th2005,by.x = "id_rssd_hd_off",
                       by.y = "id_rssd", all.x = T)
acct_info_2006$id_rssd_top[is.na(acct_info_2006$id_rssd_top)] =
  acct_info_2006$id_rssd_hd_off[is.na(acct_info_2006$id_rssd_top)]
acct_info_2006$dend=NULL
acct_info_2006$dstart=NULL

acct_join_info_2006 = merge(acct_join_info_2006,
                            unique(acct_info_2006[,c("id_rssd_hd_off","id_rssd_top")]),
                            by = "id_rssd_hd_off")


acct_join_info_2006 = data.table(acct_join_info_2006)

msa_rt_br = acct_join_info_2006[,list(n_rt_msa=length(unique(acct_nbr_rt))),
                                by = c("id_rssd_top","msa_code")]
msa_rt_br = msa_rt_br[,list(m_n_rt_msa = mean(n_rt_msa, na.rm = T)),
                      by = "id_rssd_top"]

acct_join_info_2006 = acct_join_info_2006[ ,list(nbranches =length(unique(id_rssd[id_rssd!=id_rssd_hd_off])),
                                                 nbranches_rt =length(unique(acct_nbr_rt)),
                                                 nbanks = length(unique(id_rssd_hd_off)),
                                                 dist_rt_loc = median(dist_rt_loc[acct_nbr_rt!=acct_nbr_loc],na.rm =T),
                                                 dist_rt_loc_max = max(dist_rt_loc[acct_nbr_rt!=acct_nbr_loc], na.rm = T)),
                                           by = "id_rssd_top"]
acct_join_info_2006 = data.frame(acct_join_info_2006)
acct_join_info_2006 = merge(acct_join_info_2006,deposits_2006_top5,by.x = "id_rssd_top", by.y = "id_rssd")
acct_join_info_2006 = data.frame(acct_join_info_2006)
deposits_id_count = data.frame(deposits_id_count)

acct_join_info_2006 = acct_join_info_2006[,c("id_rssd_top","name", "nbranches","nbranches_rt",
                                             "dist_rt_loc","dist_rt_loc_max")]

acct_join_info_2006 = merge(acct_join_info_2006, deposits_id_count, by = "id_rssd_top")
acct_join_info_2006 =acct_join_info_2006[,c("name","nbranches","n_br", "dist_rt_loc", "dist_rt_loc_max")]

acct_join_info_2006 = acct_join_info_2006[order(-acct_join_info_2006$nbranches),]
stargazer(acct_join_info_2006, type = "latex", summary = F, rownames =F, digits = 1)
#-------------------------------------------------------------------------------->






#-------------------------------------------------------------------------------->
# Plot spreads----
#-------------------------------------------------------------------------------->
cd_small_agg$year = year(cd_small_agg$date)

cd_small_agg = data.table(cd_small_agg)
vars = names(cd_small_agg)
vars = vars[-which(vars %in% c("date","n_banks",  "n_branches", "n_msa" ))]

cd_small_agg_year = cd_small_agg[,lapply(.SD,"mean",na.rm = T),
                  .SDcols=vars,
                  by = c("year")]



cd_small_agg_year$Disp_3mo_95_5 = cd_small_agg_year$APY3_p95-cd_small_agg_year$APY3_p5
cd_small_agg_year$Disp_6mo_95_5 = cd_small_agg_year$APY6_p95-cd_small_agg_year$APY6_p5
cd_small_agg_year$Disp_12mo_95_5 = cd_small_agg_year$APY12_p95-cd_small_agg_year$APY12_p5
cd_small_agg_year$Disp_24mo_95_5 = cd_small_agg_year$APY24_p95-cd_small_agg_year$APY24_p5
cd_small_agg_year$Disp_36mo_95_5 = cd_small_agg_year$APY36_p95-cd_small_agg_year$APY36_p5
cd_small_agg_year$Disp_60mo_95_5 = cd_small_agg_year$APY60_p95-cd_small_agg_year$APY60_p5



pdf(file = paste0(figures_dir,"Spreads.pdf"), width = w , height = w/h )
with(cd_small_agg_year, plot(year, APY3_m - treas3mo, type = "l",
                        #xaxt = "n",
                        xlab = "",
                        ylab = "",
                        main = "",
                        lwd = 2,
                        lty = 1,
                        ylim = c(-2.2,1),
                        col = "black"))
par(new = T)
grid(NULL,NULL, col = "lightgray", lty = 1, lwd = 1, equilogs = F)
abline(h = 0, col = "black", lwd = 0.5)

with(cd_small_agg_year, lines(year, APY6_m - treas6mo,
                              lty = 2,
                              col = "cyan", lwd = 2))

with(cd_small_agg_year, lines(year, APY12_m - treas12mo,
                         lty = 3,
                         col = "green", lwd = 2))

with(cd_small_agg_year, lines(year, APY36_m - treas36mo,
                         lty = 4,
                         col = "blue", lwd = 2))

with(cd_small_agg_year, lines(year, APY60_m - treas60mo,
                         lty = 5,
                         col = "grey", lwd = 2))

legend("topleft", c("3-mo","6-mo","12-mo",
                     "36-mo", "60-mo"),
       cex = 0.7,
       col = c("black",
               "cyan",
               "green",
               "blue",
               "grey"),
       lwd = c(2),
       lty = c(1,2,3,4,5))

dev.off()


# scatter plots

pdf(file = paste0(figures_dir,"tffr_disp_spreads_3mo.pdf"), width = w , height = w/h )
with(cd_small_agg, plot(tffr, APY3_m - treas3mo, type = "p",
                             #xaxt = "n",
                             xlab = "Target federal funds rate",
                             ylab = "",
                             main = "",
                             lwd = 1,
                             ylim = c(-2.7,3.1),
                             col = "black"))

par(new = T)
grid(NULL,NULL, col = "lightgray", lty = 1, lwd = 1, equilogs = F)
abline(h = 0, col = "black", lwd = 0.5)
with(cd_small_agg, points(tffr, APY3_p95 - APY3_p5, type = "p",
                             #xaxt = "n",
                             xlab = "",
                             ylab = "",
                             main = "",
                             pch = 2,
                             lwd = 1,
                             col = "red"))
abline(v = 0.25, col = "green")

legend("topleft", c("Spread 3-mo treasury", "Disp P(0.95) - P(0.05)"),
       cex = 0.7,
       pch = c(1,2),
       col = c("black",
               "red"),
       lwd = c(1))
dev.off()


pdf(file = paste0(figures_dir,"tffr_disp_spreads_60mo.pdf"), width = w , height = w/h )
with(cd_small_agg, plot(tffr, APY60_m - treas60mo, type = "p",
                        #xaxt = "n",
                        xlab = "Target federal funds rate",
                        ylab = "",
                        main = "",
                        lwd = 1,
                        ylim = c(-2.7,3.1),
                        col = "black"))

par(new = T)
grid(NULL,NULL, col = "lightgray", lty = 1, lwd = 1, equilogs = F)
abline(h = 0, col = "black", lwd = 0.5)
with(cd_small_agg, points(tffr, APY60_p95 - APY60_p5, type = "p",
                          #xaxt = "n",
                          xlab = "",
                          ylab = "",
                          main = "",
                          pch = 2,
                          lwd = 1,
                          col = "red"))
abline(v = 0.25, col = "green")

legend("topleft", c("Spread 60-mo treasury", "Disp P(0.95) - P(0.05)"),
       cex = 0.7,
       pch = c(1,2),
       col = c("black",
               "red"),
       lwd = c(1))
dev.off()



## Annual frequency
years =unique(cd_small_agg_year$year)
pos = rep(1,15)
pos[which(years %in% c(2009,2010))] = 4
pos[which(years %in% c(1998))] = 4

cd_small_agg_year$spread_3mo = cd_small_agg_year$apy_cd3_p95 - cd_small_agg_year$treas3mo
cd_small_agg_year$spread_6mo = cd_small_agg_year$APY6_m - cd_small_agg_year$treas6mo
cd_small_agg_year$spread_12mo = cd_small_agg_year$APY12_m - cd_small_agg_year$treas12mo
cd_small_agg_year$spread_24mo = cd_small_agg_year$APY24_m - cd_small_agg_year$treas24mo
cd_small_agg_year$spread_36mo = cd_small_agg_year$APY36_m - cd_small_agg_year$treas36mo
cd_small_agg_year$spread_60mo = cd_small_agg_year$APY60_m - cd_small_agg_year$treas60mo


cd_small_agg_year$disp_3mo = cd_small_agg_year$APY3_p95 - cd_small_agg_year$APY3_p5


cd_small_agg_year$spread_60mo = cd_small_agg_year$APY60_m - cd_small_agg_year$treas60mo
cd_small_agg_year$disp_60mo = cd_small_agg_year$APY60_p95 - cd_small_agg_year$APY60_p5



pdf(file = paste0(figures_dir,"tffr_disp_spreads_3mo_a.pdf"), width = w , height = w/h )
with(cd_small_agg_year, plot(tffr, spread_3mo, type = "p",
                        xaxt = "n",
                        xlab = "",
                        ylab = "",
                        main = "",
                        lwd = 2,
                        cex = 1.5,
                        ylim = c(-2.7,3.1),
                        col = "blue"))
axis(1, at=c(0.25,1,2,3,4,5,6), labels=c(0.25,1,2,3,4,5,6))
text(cd_small_agg_year$tffr, cd_small_agg_year$APY3_m - cd_small_agg_year$treas3mo-0.02,
     labels=cd_small_agg_year$year, cex= 0.6, pos=pos, col = "blue")
abline(lm(spread_3mo ~ tffr, data = cd_small_agg_year), col = "blue", lwd =1)


par(new = T)
grid(NULL,NULL, col = "lightgray", lty = 1, lwd = 1, equilogs = F)
abline(h = 0, col = "black", lwd = 0.5)

pos = rep(1,15)
pos[which(years %in% c(2009,2010))] = 4
pos[which(years %in% c(2006, 1998))] = 3

with(cd_small_agg_year, points(tffr, APY3_p95 - APY3_p5, type = "p",
                          #xaxt = "n",
                          xlab = "",
                          ylab = "",
                          main = "",
                          pch = 2,
                          lwd = 2,
                          cex = 1.5,
                          col = "red"))
abline(v = 0.25, col = "green")
text(cd_small_agg_year$tffr, cd_small_agg_year$APY3_p95 - cd_small_agg_year$APY3_p5,
     labels=cd_small_agg_year$year, cex= 0.6, pos=pos, col = "red")
abline(lm(disp_3mo ~ tffr, data = cd_small_agg_year), col = "red", lwd =1)

legend("topleft", c("Spread 3-mo treasury", "Disp P(0.95) - P(0.05)"),
       cex = 1,
       pch = c(1,2),
       col = c("blue",
               "red"),
       lwd = c(2))
dev.off()


pdf(file = paste0(figures_dir,"tffr_disp_spreads_60mo_a.pdf"), width = w , height = w/h )
with(cd_small_agg_year, plot(tffr, APY60_m - treas60mo, type = "p",
                             xaxt = "n",
                             xlab = "Target Federal Funds Rate",
                             ylab = "",
                             main = "",
                             lwd = 2,
                             cex = 1.5,
                             ylim = c(-2.7,3.1),
                             col = "blue"))
axis(1, at=c(0.25,1,2,3,4,5,6), labels=c(0.25,1,2,3,4,5,6))
text(cd_small_agg_year$tffr, cd_small_agg_year$APY60_m - cd_small_agg_year$treas60mo-0.02,
     labels=cd_small_agg_year$year, cex= 0.6, pos=pos, col = "blue")
abline(lm(spread_60mo ~ tffr, data = cd_small_agg_year), col = "blue", lwd =1)


par(new = T)
grid(NULL,NULL, col = "lightgray", lty = 1, lwd = 1, equilogs = F)
abline(h = 0, col = "black", lwd = 0.5)

pos = rep(1,15)
pos[which(years %in% c(2009,2010))] = 4
pos[which(years %in% c(2006))] = 2
pos[which(years %in% c(2007))] = 2

with(cd_small_agg_year, points(tffr, APY60_p95 - APY60_p5, type = "p",
                               #xaxt = "n",
                               xlab = "Target Federal Funds Rate",
                               ylab = "",
                               main = "",
                               pch = 2,
                               lwd = 2,
                               cex = 1.5,
                               col = "red"))
abline(v = 0.25, col = "green")
text(cd_small_agg_year$tffr, cd_small_agg_year$APY60_p95 - cd_small_agg_year$APY60_p5,
     labels=cd_small_agg_year$year, cex= 0.6, pos=pos, col = "red")
abline(lm(disp_60mo ~ tffr, data = cd_small_agg_year), col = "red", lwd =1)


legend("topleft", c("Spread 5-year treasury", "Disp P(0.95) - P(0.05)"),
       cex = 1,
       pch = c(1,2),
       col = c("blue",
               "red"),
       lwd = c(2))
dev.off()

#-------------------------------------------------------------------------------->
## Table: Summary of the relationship between the level of interest rates
# price dispersion and spreads----
#-------------------------------------------------------------------------------->
table_spread_disp = data.table( mat = c("3-mo", "6-mo", "12-mo",
                                        "24-mo", "36-mo", "60-mo"),
     spread = c(mean(cd_small_agg_year$spread_3mo),
                mean(cd_small_agg_year$spread_6mo),
                mean(cd_small_agg_year$spread_12mo),
                mean(cd_small_agg_year$spread_24mo),
                mean(cd_small_agg_year$spread_36mo),
                mean(cd_small_agg_year$spread_60mo)),
     spread_beta = c(lm(spread_3mo ~ tffr, data = cd_small_agg_year)$coefficients[2],
                      lm(spread_6mo ~ tffr, data = cd_small_agg_year)$coefficients[2],
                      lm(spread_12mo ~ tffr, data = cd_small_agg_year)$coefficients[2],
                      lm(spread_24mo ~ tffr, data = cd_small_agg_year)$coefficients[2],
                      lm(spread_36mo ~ tffr, data = cd_small_agg_year)$coefficients[2],
                      lm(spread_60mo ~ tffr, data = cd_small_agg_year)$coefficients[2]),
     spread_rsq = c(summary(lm(spread_3mo ~ tffr, data = cd_small_agg_year))$r.squared,
                    summary(lm(spread_6mo ~ tffr, data = cd_small_agg_year))$r.squared,
                    summary(lm(spread_12mo ~ tffr, data = cd_small_agg_year))$r.squared,
                    summary(lm(spread_24mo ~ tffr, data = cd_small_agg_year))$r.squared,
                    summary(lm(spread_36mo ~ tffr, data = cd_small_agg_year))$r.squared,
                    summary(lm(spread_60mo ~ tffr, data = cd_small_agg_year))$r.squared),
    disp = c(mean(cd_small_agg_year$Disp_3mo_95_5),
            mean(cd_small_agg_year$Disp_6mo_95_5),
            mean(cd_small_agg_year$Disp_12mo_95_5),
            mean(cd_small_agg_year$Disp_24mo_95_5),
            mean(cd_small_agg_year$Disp_36mo_95_5),
            mean(cd_small_agg_year$Disp_60mo_95_5)),
  disp_beta = c(lm(Disp_3mo_95_5 ~ tffr, data = cd_small_agg_year)$coefficients[2],
                    lm(Disp_6mo_95_5 ~ tffr, data = cd_small_agg_year)$coefficients[2],
                    lm(Disp_12mo_95_5 ~ tffr, data = cd_small_agg_year)$coefficients[2],
                    lm(Disp_24mo_95_5 ~ tffr, data = cd_small_agg_year)$coefficients[2],
                    lm(Disp_36mo_95_5 ~ tffr, data = cd_small_agg_year)$coefficients[2],
                    lm(Disp_60mo_95_5 ~ tffr, data = cd_small_agg_year)$coefficients[2]),
  disp_rsq = c(summary(lm(Disp_3mo_95_5 ~ tffr, data = cd_small_agg_year))$r.squared,
                 summary(lm(Disp_6mo_95_5 ~ tffr, data = cd_small_agg_year))$r.squared,
                 summary(lm(Disp_12mo_95_5 ~ tffr, data = cd_small_agg_year))$r.squared,
                 summary(lm(Disp_24mo_95_5 ~ tffr, data = cd_small_agg_year))$r.squared,
                 summary(lm(Disp_36mo_95_5 ~ tffr, data = cd_small_agg_year))$r.squared,
                 summary(lm(Disp_60mo_95_5 ~ tffr, data = cd_small_agg_year))$r.squared))

summary(lm(spread_3mo ~ tffr, data = cd_small_agg_year))
summary(lm(spread_6mo ~ tffr, data = cd_small_agg_year))
summary(lm(spread_12mo ~ tffr, data = cd_small_agg_year))
summary(lm(spread_24mo ~ tffr, data = cd_small_agg_year))
summary(lm(spread_36mo ~ tffr, data = cd_small_agg_year))
summary(lm(spread_60mo ~ tffr, data = cd_small_agg_year))



summary(lm(Disp_3mo_95_5 ~ tffr, data = cd_small_agg_year))
summary(lm(Disp_6mo_95_5 ~ tffr, data = cd_small_agg_year))
summary(lm(Disp_12mo_95_5 ~ tffr, data = cd_small_agg_year))
summary(lm(Disp_24mo_95_5 ~ tffr, data = cd_small_agg_year))
summary(lm(Disp_36mo_95_5 ~ tffr, data = cd_small_agg_year))
summary(lm(Disp_60mo_95_5 ~ tffr, data = cd_small_agg_year))


stargazer(table_spread_disp, type = "latex", summary = F, digits= 2)

#-------------------------------------------------------------------------------->
# Table 1: Total variation decomposition----
#-------------------------------------------------------------------------------->
# Decompose the cross-sectional variation
cd_small_bank_msa = data.table(cd_small_bank_msa)

# Bank level total variance decomposition
vars = colnames(cd_small_bank_msa)[grep("APY", colnames(cd_small_bank_msa))]

## overall means
cd_means_t = cd_small_bank_msa[,lapply(.SD,"mean",na.rm = T),
                               .SDcols=vars,
                               by = c("date")]
cd_means_t = as.data.frame(cd_means_t)
colnames(cd_means_t)[which(colnames(cd_means_t) %in% vars)] = paste0(vars,"_mt")


## bank variation
cd_means_bank = cd_small_bank_msa[,lapply(.SD,"mean",na.rm = T),
                                  .SDcols=vars,
                                  by = c("id_rssd_hd_off","date")]
cd_means_bank = as.data.frame(cd_means_bank)
colnames(cd_means_bank)[which(colnames(cd_means_bank) %in% vars)] = paste0(vars,"_mb")


## market variation
cd_means_market = cd_small_bank_msa[,lapply(.SD,"mean",na.rm = T),
                                    .SDcols=vars,
                                    by = c("msa_code","date")]
cd_means_market = as.data.frame(cd_means_market)
colnames(cd_means_market)[which(colnames(cd_means_market) %in% vars)] = paste0(vars,"_mm")

cd_small_bank_msa = as.data.frame(cd_small_bank_msa)
cd_variance_decomp = merge(cd_small_bank_msa, cd_means_t, by ="date")
cd_variance_decomp = merge(cd_variance_decomp, cd_means_bank, by =c("id_rssd_hd_off","date"))
cd_variance_decomp = merge(cd_variance_decomp, cd_means_market, by =c("msa_code","date"))


cd_variance_decomp_agg = data.table(cd_variance_decomp)
cd_variance_decomp_agg = cd_variance_decomp_agg[,list(
  WAPY3_w_bank = sum((APY3-APY3_mb)^2,na.rm=T)/sum((APY3-APY3_mt)^2,na.rm=T),
  WAPY3_a_bank = sum((APY3_mb-APY3_mt)^2,na.rm=T)/sum((APY3-APY3_mt)^2,na.rm=T),
  WAPY3_w_market = sum((APY3-APY3_mm)^2,na.rm=T)/sum((APY3-APY3_mt)^2,na.rm=T),
  WAPY3_a_market = sum((APY3_mm-APY3_mt)^2,na.rm=T)/sum((APY3-APY3_mt)^2,na.rm=T),
  # 6
  WAPY6_w_bank = sum((APY6-APY6_mb)^2,na.rm=T)/sum((APY6-APY6_mt)^2,na.rm=T),
  WAPY6_a_bank = sum((APY6_mb-APY6_mt)^2,na.rm=T)/sum((APY6-APY6_mt)^2,na.rm=T),
  WAPY6_w_market = sum((APY6-APY6_mm)^2,na.rm=T)/sum((APY6-APY6_mt)^2,na.rm=T),
  WAPY6_a_market = sum((APY6_mm-APY6_mt)^2,na.rm=T)/sum((APY6-APY6_mt)^2,na.rm=T),

  WAPY12_w_bank = sum((APY12-APY12_mb)^2,na.rm=T)/sum((APY12-APY12_mt)^2,na.rm=T),
  WAPY12_a_bank = sum((APY12_mb-APY12_mt)^2,na.rm=T)/sum((APY12-APY12_mt)^2,na.rm=T),
  WAPY12_w_market = sum((APY12-APY12_mm)^2,na.rm=T)/sum((APY12-APY12_mt)^2,na.rm=T),
  WAPY12_a_market = sum((APY12_mm-APY12_mt)^2,na.rm=T)/sum((APY12-APY12_mt)^2,na.rm=T),

  WAPY24_w_bank = sum((APY24-APY24_mb)^2,na.rm=T)/sum((APY24-APY24_mt)^2,na.rm=T),
  WAPY24_a_bank = sum((APY24_mb-APY24_mt)^2,na.rm=T)/sum((APY24-APY24_mt)^2,na.rm=T),
  WAPY24_w_market = sum((APY24-APY24_mm)^2,na.rm=T)/sum((APY24-APY24_mt)^2,na.rm=T),
  WAPY24_a_market = sum((APY24_mm-APY24_mt)^2,na.rm=T)/sum((APY24-APY24_mt)^2,na.rm=T),

  WAPY36_w_bank = sum((APY36-APY36_mb)^2,na.rm=T)/sum((APY36-APY36_mt)^2,na.rm=T),
  WAPY36_a_bank = sum((APY36_mb-APY36_mt)^2,na.rm=T)/sum((APY36-APY36_mt)^2,na.rm=T),
  WAPY36_w_market = sum((APY36-APY36_mm)^2,na.rm=T)/sum((APY36-APY36_mt)^2,na.rm=T),
  WAPY36_a_market = sum((APY36_mm-APY36_mt)^2,na.rm=T)/sum((APY36-APY36_mt)^2,na.rm=T),

  WAPY60_w_bank = sum((APY60-APY60_mb)^2,na.rm=T)/sum((APY60-APY60_mt)^2,na.rm=T),
  WAPY60_a_bank = sum((APY60_mb-APY60_mt)^2,na.rm=T)/sum((APY60-APY60_mt)^2,na.rm=T),
  WAPY60_w_market = sum((APY60-APY60_mm)^2,na.rm=T)/sum((APY60-APY60_mt)^2,na.rm=T),
  WAPY60_a_market = sum((APY60_mm-APY60_mt)^2,na.rm=T)/sum((APY60-APY60_mt)^2,na.rm=T),

  n_banks = length(unique(id_rssd_hd_off)),
  n_markets= length(unique(msa_code))), by = "date"]



var_decomp_2006 = cd_variance_decomp_agg[which(cd_variance_decomp_agg$date == "2006-06-15"),]
var_decomp_2006 = as.data.frame(var_decomp_2006)

var_decomp_table =rbind(as.vector(c(var_decomp_2006[,grep("_a_bank", names(var_decomp_2006))]), mode=  "numeric"),
                        as.vector(c(var_decomp_2006[,grep("_w_market", names(var_decomp_2006))]), mode=  "numeric"))



stargazer(var_decomp_table, type = "latex", summary = F, digits= 2)
#-------------------------------------------------------------------------------->


#-------------------------------------------------------------------------------->
# Table 2 Dispersion across and within markets -----
#-------------------------------------------------------------------------------->
deposits_fill$year = year(deposits_fill$date)
years = unique(deposits_fill$year)


# ----------------------------------------------------------------------------->
cluster = makeCluster(20, type = "SOCK")
registerDoSNOW(cluster)
registerDoParallel(cl = cluster, cores =20)
#------------------------------------------------------------------------------>

cd_agg_market = foreach(y = years, .combine = "rbind",
                       .packages = c("foreach"),
                       .verbose =T,
                       .export=c("Pctiles"))%do%{
                         print(y)
  dep_t = deposits_fill[deposits_fill$year==y,]
  apy_vars = names(dep_t)[grep("apy_", names(dep_t))]
  msa_t = unique(dep_t$msa_code)

  cd_msa_t = foreach(m = msa_t, .combine = "rbind")%do%{
    dep_m_t = dep_t[dep_t$msa_code==m,]
    pctiles = foreach(v = apy_vars, .combine = "cbind")%do%{
      pct = Pctiles(dep_m_t[[v]], type=1,name = paste0(v,"_"))
      return(pct)
    }
    pctiles$msa_code = m
  return(pctiles)
  }
  cd_msa_t$year = y
  return(cd_msa_t)
}
cd_agg_market = cd_agg_market[order(cd_agg_market$msa_code,
                                    cd_agg_market$year),]
setwd(final_data_dir)
save(cd_agg_market, file ="cd_agg_market.rda")

#-------------------------------------------------------------------------------->
cd_agg_market = data.table(cd_agg_market)
sumd_msa_market = data.table(sumd_msa_market)
cd_agg_market = merge(cd_agg_market,sumd_msa_market[,list(msa_code,year,deposits_msa)],
                      all.x =T, all.y = F)
cd_agg_market = cd_agg_market[order(-cd_agg_market$deposits_msa,
                                    cd_agg_market$year)]
# measure of dispersion 95th vs 5th percentiles
cd_agg_market[,disp_cd3 :=apy_cd3_p95-apy_cd3_p5 ]
cd_agg_market[,disp_cd6 :=apy_cd6_p95-apy_cd6_p5 ]
cd_agg_market[,disp_cd12 :=apy_cd12_p95-apy_cd12_p5 ]
cd_agg_market[,disp_cd24 :=apy_cd24_p95-apy_cd24_p5 ]
cd_agg_market[,disp_cd60 :=apy_cd60_p95-apy_cd60_p5 ]
cd_agg_market = cd_agg_market[order(cd_agg_market$msa_code,cd_agg_market$year),]
cd_agg_market[,deposits_msa2011 := deposits_msa[year==2011],by = "msa_code"]
cd_agg_market$deposits_msa[cd_agg_market$year>2011]=
  cd_agg_market$deposits_msa2011[cd_agg_market$year>2011]
cd_agg_market$deposits_msa = as.numeric(cd_agg_market$deposits_msa)
summary(cd_agg_market$disp_cd3)
summary(cd_agg_market$disp_cd60)

cd_agg_market[,share_dep:=sum(deposits_msa,na.rm=T), by = "year"]
cd_agg_market[,share_dep:=deposits_msa/share_dep, by = "year"]


cd_market = cd_agg_market[,list(
  ## 3 month
  # median
  cd3_med_p25 =quantile(apy_cd3_p50,probs = 0.25, na.rm = T),
  cd3_med_p50 =quantile(apy_cd3_p50,probs = 0.5, na.rm = T),
  cd3_med_p75 =quantile(apy_cd3_p50,probs = 0.75, na.rm = T),
  cd3_med_mw = sum(apy_cd3_p50*share_dep, na.rm = T),
  #dispersion
  disp_cd3_95_5_p25 =quantile(disp_cd3,probs = 0.25, na.rm = T),
  disp_cd3_95_5_p50 =quantile(disp_cd3,probs = 0.5, na.rm = T),
  disp_cd3_95_5_p75 =quantile(disp_cd3,probs = 0.75, na.rm = T),
  disp_cd3_95_5_mw = sum(disp_cd3*share_dep, na.rm = T),
  ## 6 month
  # median
  cd6_med_p25 =quantile(apy_cd6_p50,probs = 0.25, na.rm = T),
  cd6_med_p50 =quantile(apy_cd6_p50,probs = 0.5, na.rm = T),
  cd6_med_p75 =quantile(apy_cd6_p50,probs = 0.75, na.rm = T),
  cd6_med_mw = sum(apy_cd6_p50*share_dep, na.rm = T),
  #dispersion
  disp_cd6_95_5_p25 =quantile(disp_cd6,probs = 0.25, na.rm = T),
  disp_cd6_95_5_p50 =quantile(disp_cd6,probs = 0.5, na.rm = T),
  disp_cd6_95_5_p75 =quantile(disp_cd6,probs = 0.75, na.rm = T),
  disp_cd6_95_5_mw = sum(disp_cd6*share_dep, na.rm = T),
  ## 12 month
  # median
  cd12_med_p25 =quantile(apy_cd12_p50,probs = 0.25, na.rm = T),
  cd12_med_p50 =quantile(apy_cd12_p50,probs = 0.5, na.rm = T),
  cd12_med_p75 =quantile(apy_cd12_p50,probs = 0.75, na.rm = T),
  cd12_med_mw = sum(apy_cd12_p50*share_dep, na.rm = T),
  #dispersion
  disp_cd12_95_5_p25 =quantile(disp_cd12,probs = 0.25, na.rm = T),
  disp_cd12_95_5_p50 =quantile(disp_cd12,probs = 0.5, na.rm = T),
  disp_cd12_95_5_p75 =quantile(disp_cd12,probs = 0.75, na.rm = T),
  disp_cd12_95_5_mw = sum(disp_cd12*share_dep, na.rm = T),
  ## 24 month
  # median
  cd24_med_p25 =quantile(apy_cd24_p50,probs = 0.25, na.rm = T),
  cd24_med_p50 =quantile(apy_cd24_p50,probs = 0.5, na.rm = T),
  cd24_med_p75 =quantile(apy_cd24_p50,probs = 0.75, na.rm = T),
  cd24_med_mw = sum(apy_cd24_p50*share_dep, na.rm = T),
  #dispersion
  disp_cd24_95_5_p25 =quantile(disp_cd24,probs = 0.25, na.rm = T),
  disp_cd24_95_5_p50 =quantile(disp_cd24,probs = 0.5, na.rm = T),
  disp_cd24_95_5_p75 =quantile(disp_cd24,probs = 0.75, na.rm = T),
  disp_cd24_95_5_mw = sum(disp_cd24*share_dep, na.rm = T),
  ## 60-month
  # median
  cd60_med_p25 =quantile(apy_cd60_p50,probs = 0.25, na.rm = T),
  cd60_med_p50 =quantile(apy_cd60_p50,probs = 0.5, na.rm = T),
  cd60_med_p75 =quantile(apy_cd60_p50,probs = 0.75, na.rm = T),
  cd60_med_mw = sum(apy_cd60_p50*share_dep, na.rm = T),
  #dispersion
  disp_cd60_95_5_p25 =quantile(disp_cd60,probs = 0.25, na.rm = T),
  disp_cd60_95_5_p50 =quantile(disp_cd60,probs = 0.5, na.rm = T),
  disp_cd60_95_5_p75 =quantile(disp_cd60,probs = 0.75, na.rm = T),
  disp_cd60_95_5_mw = sum(disp_cd60*share_dep, na.rm = T)),by = "year"]
cd_market = cd_market[order(cd_market$year),]

cd_market2006 = data.frame(cd_market[cd_market$year==2006,])

table_1_2006 = data.frame( mat = as.vector(c( "3-mo","6-mo", "12-mo","24-mo", "60-mo")),
                           med_p25 = as.vector(cd_market2006[,grep("_med_p25", names(cd_market2006))], mode = "numeric"),
                           med_p50 =as.vector(cd_market2006[,grep("_med_p50", names(cd_market2006))], mode = "numeric"),
                           med_p75 = as.vector(cd_market2006[,grep("_med_p75", names(cd_market2006))], mode = "numeric"),
                           med_dw = as.vector(cd_market2006[,grep("_med_mw", names(cd_market2006))], mode = "numeric"),
                           disp_p25= as.vector(cd_market2006[,grep("_95_5_p25", names(cd_market2006))], mode = "numeric"),
                           disp_p50= as.vector(cd_market2006[,grep("_95_5_p50", names(cd_market2006))], mode = "numeric"),
                           disp_p75= as.vector(cd_market2006[,grep("_95_5_p75", names(cd_market2006))], mode = "numeric"),
                           disp_dw= as.vector(cd_market2006[,grep("_95_5_mw", names(cd_market2006))], mode = "numeric"))

table_1_2006[,2:9] = round(table_1_2006[,2:9],2)



stargazer(table_1_2006, type = "latex", summary = F, digits= 2, rownames = F)
