#--------------------------------------------------------------------------------
# Run regressions at the MSA level to determine factors influencing observed
# pricing behavior
# file: Run_MSA_level_regression.R
# Description: Summary of deposit
# Author: Vladimir Yankov
# Date: August 2015
#--------------------------------------------------------------------------------
# Clear workspace
rm(list = ls())
# Load required libaries
root_dir <- "/fire/home/m1vly00/"
data_dir <-"/ofs/scratch-m1vly00/Data/"
final_data_dir<- "/ofs/scratch-m1vly00/Data/Data_InSearchOfRiskFreeAsset/MSA_Level_Data/"
lib_loc = paste0(root_dir, "Code/R/Packages/")
figures_dir = paste0(root_dir,"Academic_Research_Vladimir_Yankov/InSearchRisklessAsset/Results/Figures/" )
#--------------------------------------------------------------------------------
libraries  = c("foreign", "iterators", "foreach", "snow", "doSNOW",
               "xlsx","haven","sandwich","stargazer","AER","sandwich",
               "lubridate", "data.table", "plm", "zoo","dplyr")
for(l in libraries){
  if(require(l, character.only = T)){
    print(l)
  }
  if(!require(l, character.only = T)){
    install.packages(l)
  }
}

#--------------------------------------------------------------------------------
# Set path to where the files will be stored under Windows server (RDC)
setwd(data_dir)
#--------------------------------------------------------------------------------
# user-defined functions
as.numeric.factor <- function(x){as.numeric(levels(x))[x]}
as.character.factor <- function(x){as.character(levels(x))[x]}
source(paste0(code_dir,"PlottingFunctions.R"))
source(paste0(code_dir,"CollapseFunctions.R"))
Pctiles = function(x, type = 1, name = ""){
  t = data.frame(
    mean = mean(x, na.rm = T),
    sd =sd(x,na.rm = T),
    p1 = quantile(x, probs = 0.01, na.rm = TRUE, names = FALSE, type = type),
    p5 =  quantile(x, probs = 0.05, na.rm = TRUE, names = FALSE, type = type),
    p10 =  quantile(x, probs = 0.10, na.rm = TRUE, names = FALSE, type = type),
    p25 =  quantile(x, probs = 0.25, na.rm = TRUE, names = FALSE, type = type),
    p50 =  quantile(x, probs = 0.50, na.rm = TRUE, names = FALSE, type = type),
    p75 =  quantile(x, probs = 0.75, na.rm = TRUE, names = FALSE, type = type),
    p90 =  quantile(x, probs = 0.9, na.rm = TRUE, names = FALSE, type = type),
    p95 =  quantile(x, probs = 0.95, na.rm = TRUE, names = FALSE, type = type),
    p99 =  quantile(x, probs = 0.99, na.rm = TRUE, names = FALSE, type = type),
    min = min(x,na.rm = T),
    max = max(x,na.rm = T))
  names(t) = paste0(name, names(t))
  return(t)
}

decimalplaces <- function(x){
  n = length(x)
  y = rep(NA,n)
  for(i in 1:n){
    z= strsplit(as.character(abs(x[i])), split = "[.]", fixed = F)[[1]]
    y[i] = nchar(z[2])
  }
  return(y)
}
#--------------------------------------------------------------------------------
# Load data
#--------------------------------------------------------------------------------
setwd(data_dir)
load(file = "./RW_data/RW_final/deposits/deposits_fill.rda")


## SOD
load(file = "./SOD/sumd_msa_market.rda")
load(file = "./SOD/msa_code.rda")

## Interest rates
load(file = "./InterestRates/Treas_FFT_FFF.rda")
load(file = "./InterestRates/Treas_FF_surprise.rda")

##---> BHCs level data
load(file = "./Y_9C/Y9C_vars.rda")

bhc_data=Y9C_vars[,c("id_rssd","date","assets","total_loans_leases",
                     "dep_time_more100K","dep_time_less100K",
                     "share_st_funding","core_dep","deposits","ROE")]

## ---> collapse bank-level data
load(file = "./CallReports/Call_vars.rda")
max(Call_vars$date)
call_deposit_vars =c("id_rssd", "id_rssd_hh1", "date",
                     "td_mat","dep_mat",
                     "total_deposits",
                     "demand_deposits",
                     "transactions_deposits","nontransactions_deposits",
                     "deposits_uninsured",
                     "deposits_listing","deposits_cdar",
                     "time_above100K_q","time_below100K_q",

                     "int_timeabove100K","int_timebelow100K",
                     "int_sav_mmda","int_ff_repo",
                     "brokered_deposits",
                     "brokered_below100K",

                     "deposits_above100K",
                     "deposits_above250K",
                     "num_accounts_above100K",
                     "num_accounts_above250K",
                     "avg_balance_above100K",
                     "avg_balance_above250K",

                     "deposits_below100K",
                     "deposits_below250K",
                     "num_accounts_below100K",
                     "num_accounts_below250K",
                     "avg_balance_below100K",
                     "avg_balance_below250K")

call_deposits = Call_vars[,call_deposit_vars]
call_deposits = data.table(call_deposits)
call_deposits[,year:= year(date)]

call_deposits[,avg_balance_below100K_v2 := max(avg_balance_below100K, na.rm = T),
              by = c("id_rssd","year")]
call_deposits$avg_balance_below100K_v2[is.infinite(call_deposits$avg_balance_below100K_v2)]=NA
call_deposits$avg_balance_below100K[is.na(call_deposits$avg_balance_below100K)]=
  call_deposits$avg_balance_below100K_v2[is.na(call_deposits$avg_balance_below100K)]
call_deposits$num_accounts_below100K[is.na(call_deposits$num_accounts_below100K)]=
  round(call_deposits$deposits_below100K[is.na(call_deposits$num_accounts_below100K)]/call_deposits$avg_balance_below100K_v2[is.na(call_deposits$num_accounts_below100K)],0)
call_deposits[,avg_balance_below100K_v2:=NULL]


call_deposits[,avg_balance_below250K_v2 := max(avg_balance_below250K, na.rm = T),
              by = c("id_rssd","year")]
call_deposits$avg_balance_below250K_v2[is.infinite(call_deposits$avg_balance_below250K_v2)]=NA
call_deposits$avg_balance_below250K[is.na(call_deposits$avg_balance_below250K)]=
  call_deposits$avg_balance_below250K_v2[is.na(call_deposits$avg_balance_below250K)]
call_deposits$num_accounts_below250K[is.na(call_deposits$num_accounts_below250K)]=
  round(call_deposits$deposits_below250K[is.na(call_deposits$num_accounts_below250K)]/call_deposits$avg_balance_below250K_v2[is.na(call_deposits$num_accounts_below250K)],0)
call_deposits[,avg_balance_below250K_v2:=NULL]


call_deposits$id_rssd_top =call_deposits$id_rssd_hh1
call_deposits$id_rssd_top[is.na(call_deposits$id_rssd_top)]=0
call_deposits$id_rssd_top[call_deposits$id_rssd_top==0]=call_deposits$id_rssd[call_deposits$id_rssd_top==0]
#--------------------------------------------------------------------------------
# collapse at the top holder
#--------------------------------------------------------------------------------

call_deposits_th = data.table(call_deposits)
call_deposits_th = call_deposits_th[,
                  list(nbanks = length(unique(id_rssd)),
                       td_mat = weighted.mean(td_mat,w = total_deposits, na.rm = T),
                       dep_mat = weighted.mean(dep_mat,w = total_deposits, na.rm = T),

                       int_timebelow100K = mean(int_timebelow100K, na.rm = T),
                       int_timeabove100K = mean(int_timeabove100K, na.rm = T),
                       int_sav_mmda= mean(int_sav_mmda, na.rm = T),
                       int_ff_repo= mean(int_ff_repo, na.rm = T),

                       total_deposits = sum(total_deposits,na.rm =T),
                       demand_deposits = sum(demand_deposits,na.rm=T),
                       transactions_deposits = sum(transactions_deposits, na.rm =T),
                       nontransactions_deposits = sum(nontransactions_deposits, na.rm =T),
                       time_above100K_q = sum(time_above100K_q, na.rm=T),
                       time_below100K_q = sum(time_below100K_q, na.rm=T),

                       deposits_above100K= sum(deposits_above100K,na.rm =T),
                       deposits_below100K= sum(deposits_below100K,na.rm =T),
                       num_accounts_above100K= sum(num_accounts_above100K,na.rm =T),
                       num_accounts_below100K= sum(num_accounts_below100K,na.rm =T),

                       deposits_above250K= sum(deposits_above250K,na.rm =T),
                       deposits_below250K= sum(deposits_below250K,na.rm =T),
                       num_accounts_above250K= sum(num_accounts_above250K,na.rm =T),
                       num_accounts_below250K= sum(num_accounts_below250K,na.rm =T),

                       avg_balance_below100K = mean(avg_balance_below100K, na.rm=T),
                       avg_balance_above100K = mean(avg_balance_above100K, na.rm=T),
                       avg_balance_below250K = mean(avg_balance_below250K, na.rm=T),
                       avg_balance_above250K = mean(avg_balance_above250K, na.rm=T)),
                  by = c("id_rssd_top","date")]
call_deposits_th = data.table(call_deposits_th)
call_deposits_th[,q := quarter(date)]
call_deposits_th[,y := year(date)]

call_deposits_th = call_deposits_th[call_deposits_th$date>="1997-01-01",]


# deposit pricing
deposits_th = data.table(deposits_fill)
deposits_th[,q:=quarter(date)]
deposits_th[,y:=year(date)]

deposits_th = deposits_th[, list(apy_cd3 = mean(apy_cd3,na.rm=T),
                                   apy_cd6 = mean(apy_cd6,na.rm=T),
                                   apy_cd12 = mean(apy_cd12,na.rm=T),
                                   apy_cd24 = mean(apy_cd24,na.rm=T),
                                   apy_cd60 = mean(apy_cd60,na.rm=T),
                                   apy_sav = mean(apy_sav,na.rm=T),
                                   apy_intck = mean(apy_intck,na.rm=T)),
                            by = c("id_rssd_top","q","y")]
deposits_th = merge(deposits_th,call_deposits_th, by =c("id_rssd_top","q","y") )
#--------------------------------------------------------------------------------
##--- time aggregation

deposits_agg = deposits_th[, list(apy_sav = weighted.mean(apy_sav,w=total_deposits,na.rm=T),
                                  apy_cd3 = weighted.mean(apy_cd3,w=total_deposits,na.rm=T),
                                  apy_cd6 = weighted.mean(apy_cd6,w=total_deposits,na.rm=T),
                                  apy_cd12 = weighted.mean(apy_cd12,w=total_deposits,na.rm=T),
                                  apy_cd60 = weighted.mean(apy_cd60,w=total_deposits,na.rm=T),
                                  total_deposits = sum(total_deposits,na.rm=T),
                                  demand_deposits = sum(demand_deposits,na.rm=T),
                                  td_mat= weighted.mean(td_mat,w=total_deposits,na.rm=T),
                                  int_sav_mmda = weighted.mean(int_sav_mmda,w=total_deposits,na.rm=T),
                                  int_timebelow100K = weighted.mean(int_timebelow100K,w=total_deposits,na.rm=T),
                                  int_timeabove100K = weighted.mean(int_timeabove100K,w=total_deposits,na.rm=T),
                                  transactions_deposits = sum(transactions_deposits,na.rm=T),
                                  nontransactions_deposits = sum(nontransactions_deposits,na.rm=T),
                                  time_above100K_q = sum(time_above100K_q,na.rm=T),
                                  time_below100K_q = sum(time_below100K_q,na.rm=T)),
                           by = "date"]
deposits_agg=deposits_agg[order(deposits_agg$date),]
deposits_agg[, y:=year(date)]
deposits_agg[, q:=quarter(date)]


## interest rates

FFF_surprise_q = data.table(FFF_surprise)
FFF_surprise_q[,q := quarter(date)]
FFF_surprise_q[,y := year(date)]
FFF_surprise_q = FFF_surprise_q[,list(treas3mo=mean(treas3mo,na.rm=T),
                                      treas6mo=mean(treas6mo,na.rm=T),
                                      treas12mo=mean(treas12mo,na.rm=T),
                                      dtffr = mean(dtffr,na.rm=T),
                                      deffr = mean(deffr,na.rm =T),
                                      dtffr_unexp = mean(dtffr_unexp,na.rm=T),
                                      dtffr_exp= mean(dtffr_exp,na.rm=T)),
                                by = c("y","q")]
deposits_agg = merge(deposits_agg,FFF_surprise_q, by = c("y","q"))



deposits_agg[,trend :=seq(.N)]

deposits_agg[,time_deposits_q :=time_above100K_q+time_below100K_q ]
deposits_agg[,nontime_deposits :=total_deposits-time_deposits_q ]

deposits_agg[,nontr_trans :=nontransactions_deposits/transactions_deposits ]
deposits_agg[,td_trans :=time_deposits_q/transactions_deposits ]
deposits_agg[,td_dem :=time_deposits_q/demand_deposits ]
deposits_agg[,td_total :=time_deposits_q/total_deposits ]
deposits_agg[,td_nontd :=time_deposits_q/nontime_deposits ]


deposits_agg = deposits_agg[order(deposits_agg$date),]
deposits_agg[,dln_td := c(NA,diff(log(time_deposits_q),1)), ]

deposits_agg[,dln_td_sm:= c(NA,diff(log(time_below100K_q),1)), ]
deposits_agg[,dln_td_lg := c(NA,diff(log(time_above100K_q),1)), ]



deposits_agg[,dln_cd6 := c(NA,diff(log(apy_cd6),1)), ]
deposits_agg[,dln_cd12 := c(NA,diff(log(apy_cd12),1)), ]

deposits_agg[,dln_treas3mo := c(NA,diff(log(treas3mo),1)), ]
deposits_agg[,dln_treas6mo := c(NA,diff(log(treas6mo),1)), ]
deposits_agg[,dln_treas12mo := c(NA,diff(log(treas12mo),1)), ]

deposits_agg[, spread3mo := apy_cd3-treas3mo]
deposits_agg[, spread6mo := apy_cd6-treas6mo]
deposits_agg[, spread12mo := apy_cd12-treas12mo]
deposits_agg[, spread3mo_sav := apy_cd3-apy_sav]



summary(deposits_agg$nontr_trans)
summary(deposits_agg$td_trans)

deposits_agg[,s_R6 := (1-td_total)*dln_cd6, ]
deposits_agg[,s_R12 := (1-td_total)*dln_cd12, ]

setkey(deposits_agg,date)

#--------------------------------------------------------------------------------
# Save regression data
#--------------------------------------------------------------------------------
save(deposits_agg, file=paste0(root_dir,
"Academic_Research_Vladimir_Yankov/InSearchRisklessAsset/Results/reg_ies_data.rda"))


#--------------------------------------------------------------------------------
# Run specifications
reg_ies6mo = with(deposits_agg,lm(dln_td~lag(s_R6,1) ))
ivreg_ies6mo = with(deposits_agg,ivreg(dln_td~lag(s_R6,1)|
          lag(dtffr_unexp,1)+lag(dln_td,2)))

sandwich(reg_ies6mo)
vcovHC(reg_ies6mo)

reg_ies12mo= with(deposits_agg,lm(dln_td~lag(s_R12,1)))
ivreg_ies12mo = with(deposits_agg,ivreg(dln_td~lag(s_R12,1)|
                                         lag(dtffr_unexp,1)+
                                          lag(dln_td,2)))




# Summary
## OLS estimation
summary(reg_ies6mo, vcov =sandwich,  diagnostics = TRUE)
summary(reg_ies12mo, robust = T)
# IV estimation
summary(ivreg_ies6mo, vcov =sandwich,  diagnostics = TRUE)
summary(ivreg_ies12mo, vcov =sandwich,  diagnostics = TRUE)


stargazer(reg_ies6mo,  # OLS estimates
          reg_ies12mo,
          ivreg_ies6mo, # IV estimates
          ivreg_ies12mo, type = "latex")



# Plots of variables and error terms
setwd(figures_dir)
pdf(file = "IES_estimation.pdf")
par(mar=c(2,2,2,2))
par(mfrow=c(2,2))
with(deposits_agg,plot(date, dln_td, type = "l",
                       main = "Growth in time deposits"))
abline(h=0)
with(deposits_agg,plot(date, dln_cd6, type = "l",
                       main = "Growth in 6-month CD rate"))
abline(h=0)
with(deposits_agg,plot(date, td_total, type = "l",
                       main = "Share of time deposits in total deposits"))
with(deposits_agg,plot(date, dtffr_unexp, type = "l",
                       main = "Unexpected changes in the FFR"))
abline(h=0)
dev.off()
#--------------------------------------------------------------------------------



