#--------------------------------------------------------------------------------
# Run regressions at the MSA level to determine factors influencing observed
# pricing behavior
# file: Run_MSA_level_regression.R
# Description: Summary of deposit
# Author: Vladimir Yankov
# Date: August 2015
#--------------------------------------------------------------------------------
# Clear workspace
rm(list = ls())
# Load required libaries
root_dir <- "/fire/home/m1vly00/"
data_dir <-"/ofs/scratch-m1vly00/Data/"
final_data_dir<- "/ofs/scratch-m1vly00/Data/Data_InSearchOfRiskFreeAsset/MSA_Level_Data/"
lib_loc = paste0(root_dir, "Code/R/Packages/")
figures_dir = paste0(root_dir,"Academic_Research_Vladimir_Yankov/InSearchRisklessAsset/Results/Figures/" )
#--------------------------------------------------------------------------------
libraries  = c("foreign", "iterators", "foreach", "snow", "doSNOW",
               "xlsx","haven","sandwich","stargazer",
               "lubridate", "data.table", "plm", "zoo","dplyr")
for(l in libraries){
  if(require(l, character.only = T)){
    print(l)
  }
  if(!require(l, character.only = T)){
    install.packages(l)
  }
}

#--------------------------------------------------------------------------------
# Set path to where the files will be stored under Windows server (RDC)
setwd(data_dir)
#--------------------------------------------------------------------------------
# user-defined functions
as.numeric.factor <- function(x){as.numeric(levels(x))[x]}
as.character.factor <- function(x){as.character(levels(x))[x]}
source(paste0(code_dir,"PlottingFunctions.R"))
source(paste0(code_dir,"CollapseFunctions.R"))
Pctiles = function(x, type = 1, name = ""){
  t = data.frame(
    mean = mean(x, na.rm = T),
    sd =sd(x,na.rm = T),
    p1 = quantile(x, probs = 0.01, na.rm = TRUE, names = FALSE, type = type),
    p5 =  quantile(x, probs = 0.05, na.rm = TRUE, names = FALSE, type = type),
    p10 =  quantile(x, probs = 0.10, na.rm = TRUE, names = FALSE, type = type),
    p25 =  quantile(x, probs = 0.25, na.rm = TRUE, names = FALSE, type = type),
    p50 =  quantile(x, probs = 0.50, na.rm = TRUE, names = FALSE, type = type),
    p75 =  quantile(x, probs = 0.75, na.rm = TRUE, names = FALSE, type = type),
    p90 =  quantile(x, probs = 0.9, na.rm = TRUE, names = FALSE, type = type),
    p95 =  quantile(x, probs = 0.95, na.rm = TRUE, names = FALSE, type = type),
    p99 =  quantile(x, probs = 0.99, na.rm = TRUE, names = FALSE, type = type),
    min = min(x,na.rm = T),
    max = max(x,na.rm = T))
  names(t) = paste0(name, names(t))
  return(t)
}

decimalplaces <- function(x){
  n = length(x)
  y = rep(NA,n)
  for(i in 1:n){
    z= strsplit(as.character(abs(x[i])), split = "[.]", fixed = F)[[1]]
    y[i] = nchar(z[2])
  }
  return(y)
}
#--------------------------------------------------------------------------------
# Load data
#--------------------------------------------------------------------------------
setwd(data_dir)
load(file = "./RW_data/RW_final/deposits/deposits_1997_2016.rda")
deposits = deposits_1997_2016
rm(deposits_1997_2016)

## assign top holder
load(file = "./SOD/sumd_msa_market.rda")
load(file = "./SOD/msa_code.rda")


load(file = "./InterestRates/Treas_FFT_FFF.rda")
load(file = "./InterestRates/Treas_FF_surprise.rda")
load(file = "./CallReports/Call_vars.rda")

##---> BHCs level data
load(file = "./Y_9C/Y9C_vars.rda")

bhc_data=Y9C_vars[,c("id_rssd","date","assets","total_loans_leases",
                     "dep_time_more100K","dep_time_less100K",
                     "share_st_funding","core_dep","deposits","ROE")]

## ---> collapse bank-level data
call_deposit_vars =c("id_rssd", "id_rssd_hh1", "date",
                     "td_mat","dep_mat",
                     "total_deposits","deposits_uninsured",
                     "deposits_listing","deposits_cdar",
                     "time_above100K_q","time_below100K_q",

                     "int_timeabove100K","int_timebelow100K",
                     "int_sav_mmda","int_ff_repo",
                     "brokered_deposits",
                     "brokered_below100K",

                     "deposits_above100K",
                     "deposits_above250K",
                     "num_accounts_above100K",
                     "num_accounts_above250K",
                     "avg_balance_above100K",
                     "avg_balance_above250K",

                     "deposits_below100K",
                     "deposits_below250K",
                     "num_accounts_below100K",
                     "num_accounts_below250K",
                     "avg_balance_below100K",
                     "avg_balance_below250K")

call_deposits = Call_vars[,call_deposit_vars]
call_deposits = data.table(call_deposits)
call_deposits[,year:= year(date)]

call_deposits[,avg_balance_below100K_v2 := max(avg_balance_below100K, na.rm = T),
              by = c("id_rssd","year")]
call_deposits$avg_balance_below100K_v2[is.infinite(call_deposits$avg_balance_below100K_v2)]=NA
call_deposits$avg_balance_below100K[is.na(call_deposits$avg_balance_below100K)]=
  call_deposits$avg_balance_below100K_v2[is.na(call_deposits$avg_balance_below100K)]
call_deposits$num_accounts_below100K[is.na(call_deposits$num_accounts_below100K)]=
  round(call_deposits$deposits_below100K[is.na(call_deposits$num_accounts_below100K)]/call_deposits$avg_balance_below100K_v2[is.na(call_deposits$num_accounts_below100K)],0)
call_deposits[,avg_balance_below100K_v2:=NULL]


call_deposits[,avg_balance_below250K_v2 := max(avg_balance_below250K, na.rm = T),
              by = c("id_rssd","year")]
call_deposits$avg_balance_below250K_v2[is.infinite(call_deposits$avg_balance_below250K_v2)]=NA
call_deposits$avg_balance_below250K[is.na(call_deposits$avg_balance_below250K)]=
  call_deposits$avg_balance_below250K_v2[is.na(call_deposits$avg_balance_below250K)]
call_deposits$num_accounts_below250K[is.na(call_deposits$num_accounts_below250K)]=
  round(call_deposits$deposits_below250K[is.na(call_deposits$num_accounts_below250K)]/call_deposits$avg_balance_below250K_v2[is.na(call_deposits$num_accounts_below250K)],0)
call_deposits[,avg_balance_below250K_v2:=NULL]


call_deposits$id_rssd_top =call_deposits$id_rssd_hh1
call_deposits$id_rssd_top[is.na(call_deposits$id_rssd_top)]=0
call_deposits$id_rssd_top[call_deposits$id_rssd_top==0]=call_deposits$id_rssd[call_deposits$id_rssd_top==0]
#--------------------------------------------------------------------------------
# collapse at the top holder
#--------------------------------------------------------------------------------

call_deposits_th = data.table(call_deposits)
call_deposits_th = call_deposits_th[,
                  list(nbanks = length(unique(id_rssd)),
                       td_mat = weighted.mean(td_mat,w = total_deposits, na.rm = T),
                       dep_mat = weighted.mean(dep_mat,w = total_deposits, na.rm = T),

                       int_timebelow100K = mean(int_timebelow100K, na.rm = T),
                       int_timeabove100K = mean(int_timebelow100K, na.rm = T),
                       int_sav_mmda= mean(int_sav_mmda, na.rm = T),
                       int_ff_repo= mean(int_ff_repo, na.rm = T),

                       deposits_cb = sum(total_deposits,na.rm =T),

                       deposits_above100K= sum(deposits_above100K,na.rm =T),
                       deposits_below100K= sum(deposits_below100K,na.rm =T),
                       num_accounts_above100K= sum(num_accounts_above100K,na.rm =T),
                       num_accounts_below100K= sum(num_accounts_below100K,na.rm =T),

                       deposits_above250K= sum(deposits_above250K,na.rm =T),
                       deposits_below250K= sum(deposits_below250K,na.rm =T),
                       num_accounts_above250K= sum(num_accounts_above250K,na.rm =T),
                       num_accounts_below250K= sum(num_accounts_below250K,na.rm =T),

                       avg_balance_below100K = mean(avg_balance_below100K, na.rm=T),
                       avg_balance_above100K = mean(avg_balance_above100K, na.rm=T),
                       avg_balance_below250K = mean(avg_balance_below250K, na.rm=T),
                       avg_balance_above250K = mean(avg_balance_above250K, na.rm=T)),
                  by = c("id_rssd_top","date")]
call_deposits_th = data.frame(call_deposits_th)




#--------------------------------------------------------------------------------
# SOD -- MSA market level data
#--------------------------------------------------------------------------------
sumd_msa_market = data.table(sumd_msa_market)
sumd_msa_market[,dep_capita:= deposits_msa/population]




#--------------------------------------------------------------------------------
# Summary table of MSA market level information for 2007

market_msa_2007 = sumd_msa_market[which(sumd_msa_market$year==2007),]
market_msa_2007 = data.table(market_msa_2007)


table_msa = rbind(Pctiles(market_msa_2007$pop_total)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(100*market_msa_2007$pop65plus/market_msa_2007$pop_total)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(market_msa_2007$income_capita)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(market_msa_2007$dep_capita)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(market_msa_2007$n_banks_msa)[c("min", "p25", "p50", "p75", "max")],
                  Pctiles(market_msa_2007$n_branches_msa)[c("min", "p25", "p50", "p75", "max")],
                  #Pctiles(market_msa_2007$c1)[c("min", "p25", "p50", "p75", "max")],
                  round(100*Pctiles(market_msa_2007$HHI)[c("min", "p25", "p50", "p75", "max")],2))


table_msa = data.frame(table_msa)
table_msa$var = c("Population","Share 65+",
                  "Income per capita",
                  "Deposits per capita",
                  "Number of banks",
                  "Number of branches",
                  "HHI index")
table_msa = data.table(table_msa)
table_msa = table_msa[, list(var,min, p25, p50, p75, max)]

stargazer(table_msa, type = "latex", summary = F, digits= 2, rownames=F)

# deposit pricing at MSA level
msa_codes = deposits[, list(nbanks= length(unique(id_rssd_hd_off))), by = c("msa_code","date")]
msa_codes[, nbanks_min:= min(nbanks)]
msa_codes[, nbanks_m:= median(nbanks)]

msa_codes = msa_codes[msa_codes$nbanks > 10,]
msa_codes[,nperiods := length(unique(date)), by = "msa_code"]
length(unique(msa_codes$msa_code[msa_codes$nperiods>1000]))
msa_codes = msa_codes[msa_codes$nperiods > 1000,]
unique(sumd_msa_market$msa[sumd_msa_market$msa_code %in% unique(msa_codes$msa_code)])
summary(msa_codes$nbanks)
length(unique(msa_codes$msa_code)) # 157


# # -----------------------------------------------------------------------------
 cluster = makeCluster(12,type="SOCK")
 registerDoSNOW(cluster)
# #------------------------------------------------------------------------------

select_msa_code = unique(msa_codes$msa_code)

deposits_cd_msa = foreach(msa =select_msa_code , .combine = "rbind",
                          .packages = c("foreach"),
                          .export= c("Pctiles"))%dopar%{
print(msa)
  deposits_m = deposits[deposits$msa_code==msa & deposits$msa_ind==1,]
  dates_msa = unique(deposits_m$date)

deposits_msa_m = foreach(t = dates_msa, .combine = "rbind")%do%{
    print(t)

    deposits_msa_tt= deposits_m[deposits_m$date==t,]
    deposits_msa_t =cbind(Pctiles(deposits_msa_tt$apy_cd3, name = "apy_cd3_"),
                          Pctiles(deposits_msa_tt$apy_cd6, name = "apy_cd6_"),
                          Pctiles(deposits_msa_tt$apy_cd12, name = "apy_cd12_"),
                          Pctiles(deposits_msa_tt$apy_cd24, name = "apy_cd24_"),
                          Pctiles(deposits_msa_tt$apy_cd60, name = "apy_cd60_"))
    deposits_msa_t$nbanks = length(unique(deposits_msa_tt$id_rssd_hd_off))
    deposits_msa_t$nbranches = length(unique(deposits_msa_tt$acct_nbr))
    deposits_msa_t$date = t
    return(deposits_msa_t)
  }
  deposits_msa_m$msa_code = msa

return(deposits_msa_m)
}
rm(deposits_m,dates_msa)
save(deposits_cd_msa,
     file = paste0(final_data_dir,"deposits_cd_msa.rda"))
#------------------------------------------------------------------------------
# stop cluster
stopCluster(cluster)
#------------------------------------------------------------------------------

deposits_cd_msa$year= year(deposits_cd_msa$date)
deposits_cd_msa$week = week(deposits_cd_msa$date)

which(!select_msa_code %in% unique(sumd_msa_market$msa_code))
deposits_cd_msa=deposits_cd_msa[deposits_cd_msa$year<=2011,]

sumd_msa_market = data.table(sumd_msa_market)
sumd_msa_market[, n:=.N, by =  c("msa_code","year")]
summary(sumd_msa_market$n)
sumd_msa_market=sumd_msa_market[sumd_msa_market$n==1,]
sumd_msa_market = data.frame(sumd_msa_market)
sumd_msa_market$n = NULL

deposits_cd_msa = merge(deposits_cd_msa,sumd_msa_market,all.x = T,
                        by = c("msa_code","year"))

load(file = "./InterestRates/Treas_FFT_FFF.rda")
load(file = "./InterestRates/Treas_FF_surprise.rda")

unique(weekdays(unique(deposits_cd_msa$date)))
unique(weekdays(unique(Treasw$date)))
Treasw$date = Treasw$date-4
unique(weekdays(unique(Treasw$date)))


deposits_cd_msa = merge(deposits_cd_msa, Treasw,all.x = T,
                        by = c("date"))


deposits_cd_msa = data.table(deposits_cd_msa)
deposits_cd_msa[,disp_3m_95_5 := apy_cd3_p95-apy_cd3_p5]
deposits_cd_msa[,disp_6m_95_5 := apy_cd6_p95-apy_cd6_p5]
deposits_cd_msa[,disp_12m_95_5 := apy_cd12_p95-apy_cd12_p5]
deposits_cd_msa[,disp_24m_95_5 := apy_cd24_p95-apy_cd24_p5]
deposits_cd_msa[,disp_60m_95_5 := apy_cd60_p95-apy_cd60_p5]

deposits_cd_msa[,spread_3m := treas3mo-apy_cd3_p50]
deposits_cd_msa[,spread_6m := treas6mo-apy_cd6_p50]




#--------------------------------------------------------------------------------
# Market level regression analysis
#--------------------------------------------------------------------------------
deposits_cd_msa = data.table(deposits_cd_msa)
deposits_cd_msa = deposits_cd_msa[order(deposits_cd_msa$msa_code,deposits_cd_msa$date),]
deposits_cd_msa[,deffr := c(NA,diff(effr))]
deposits_cd_msa[, nbanks_pop:= n_banks_msa/population]
deposits_cd_msa[,n_branches_pop:= n_branches_msa/population]
deposits_cd_msa[, n:= .N, by = c("msa_code","date")]
summary(deposits_cd_msa$n)
deposits_cd_msa = data.frame(deposits_cd_msa)

# standardize variables in the regression
deposits_cd_msa = data.table(deposits_cd_msa)
deposits_cd_msa[, HHI_std:= (HHI - mean(HHI, na.rm =T))/sd(HHI, na.rm =T), by = c("date")]
deposits_cd_msa[, pop_age_65_plus_std:= (pop_age_65_plus - mean(pop_age_65_plus, na.rm =T))/sd(pop_age_65_plus, na.rm =T), by = c("date")]
deposits_cd_msa[, nbanks_pop_std:= (nbanks_pop - mean(nbanks_pop, na.rm =T))/sd(nbanks_pop, na.rm =T), by = c("date")]
deposits_cd_msa[, n_branches_pop_std:= (n_branches_pop - mean(n_branches_pop, na.rm =T))/sd(n_branches_pop, na.rm =T), by = c("date")]
deposits_cd_msa[, dep_capita_std:= (dep_capita - mean(dep_capita, na.rm =T))/sd(dep_capita, na.rm =T), by = c("date")]
deposits_cd_msa[, population_std:= (population - mean(population, na.rm =T))/sd(population, na.rm =T), by = c("date")]
deposits_cd_msa[, income_capita_std:= (income_capita - mean(income_capita, na.rm =T))/sd(income_capita, na.rm =T), by = c("date")]

deposits_cd_msa[,n:=.N, by=c("msa_code","date")]
summary(deposits_cd_msa$n)


# estimate pass-through at the msa level

# 3-month CD
model2_ols_apy3_est = lm(formula =apy_cd3_p50 ~ effr+HHI_std:effr+nbanks_pop_std:effr+
                           n_branches_pop_std:effr+population_std:effr+income_capita_std:effr+dep_capita_std:effr+
                           pop_age_65_plus_std:effr,
                          data = deposits_cd_msa,
                          #index = c("msa_code","date"), effect = "individual",
                         na.action = na.omit)

summary(model2_ols_apy3_est)

model2_ols_disp3 = disp_3m_95_5 ~ effr+HHI_std:effr+nbanks_pop_std:effr+
  n_branches_pop_std:effr+population_std:effr+income_capita_std:effr+dep_capita_std:effr+
  pop_age_65_plus_std:effr
model2_ols_disp3_est = lm(formula = model2_ols_disp3, data = deposits_cd_msa,
                         #index = c("msa_code","date"), effect = "time",
                         na.action = na.omit)
summary(model2_ols_disp3_est)

model2_ols_p5 = apy_cd3_p5~ effr+HHI_std:effr+nbanks_pop_std:effr+
  n_branches_pop_std:effr+population_std:effr+income_capita_std:effr+dep_capita_std:effr+
  pop_age_65_plus_std:effr

model2_ols_p5_est = lm(formula = model2_ols_p5, data = deposits_cd_msa,
                          #index = c("msa_code","date"), effect = "time",
                          na.action = na.omit)
summary(model2_ols_p5_est)


model2_ols_p95 = apy_cd3_p95~effr+HHI_std:effr+nbanks_pop_std:effr+
  n_branches_pop_std:effr+population_std:effr+income_capita_std:effr+dep_capita_std:effr+
  pop_age_65_plus_std:effr

model2_ols_p95_est = lm(formula = model2_ols_p95, data = deposits_cd_msa,
                       #index = c("msa_code","date"), effect = "time",
                       na.action = na.omit)
summary(model2_ols_p95_est)
stargazer(model2_ols_apy3_est,
          model2_ols_p5_est,model2_ols_p95_est,model2_ols_disp3_est)

length(unique(deposits_cd_msa$msa_code))

# loop over maturities
model2_ols_apy60 = "apy_cd60_p50 ~ effr+HHI:effr+pop_age_65_plus:effr+log(nbanks_pop):effr+log(n_branches_pop):effr"
model2_ols_apy60_est = lm(formula = model2_ols_apy60, data = deposits_cd_msa,
                         #index = c("msa_code","date"), effect = "time",
                         na.action = na.omit)
summary(model2_ols_apy60_est)





#--------------------------------------------------------------------------------
# Run bank level regression analysis
#--------------------------------------------------------------------------------



















#--------------------------------------------------------------------------------
