#-------------------------------------------------------------------------------->
# file: Test_mixingstrategies.R
# Author: Vladimir Yankov
# Date: May 2018
#-------------------------------------------------------------------------------->
# Clear workspace
rm(list = ls())
# Load required libaries
tables_dir = "/fire/home/m1vly00/Academic_Research_Vladimir_Yankov/InSearchRisklessAsset/Results/Tables/"
rw_data <- "/rsma/ratewatch/sas_datasets/"
root_dir <- "/fire/home/m1vly00/"
data_dir <-"/ofs/scratch-m1vly00/Data/"
final_data_dir<- "/ofs/scratch-m1vly00/Data/Data_InSearchOfRiskFreeAsset/MSA_Level_Data/"
lib_loc = paste0(root_dir, "Code/R/Packages/")
figures_dir = paste0(root_dir,"Academic_Research_Vladimir_Yankov/InSearchRisklessAsset/Results/Figures/" )
#-------------------------------------------------------------------------------->
libraries  = c("foreign", "iterators", "foreach", "snow", "doSNOW",
               "xlsx","haven","sandwich","stargazer","AER",
               "lfe","multiwayvcov","survival",
               "lubridate", "data.table", "plm", "zoo","dplyr")
for(l in libraries){
  if(require(l, character.only = T)){
    print(l)
  }
  if(!require(l, character.only = T)){
    install.packages(l)
  }
}

#-------------------------------------------------------------------------------->
# Set path to where the files will be stored under Windows server (RDC)
setwd(data_dir)
#-------------------------------------------------------------------------------->
# user-defined functions
as.numeric.factor <- function(x){as.numeric(levels(x))[x]}
as.character.factor <- function(x){as.character(levels(x))[x]}
source(paste0(code_dir,"PlottingFunctions.R"))
source(paste0(code_dir,"CollapseFunctions.R"))
Pctiles = function(x, type = 1, name = ""){
  t = data.frame(
    mean = mean(x, na.rm = T),
    sd =sd(x,na.rm = T),
    p1 = quantile(x, probs = 0.01, na.rm = TRUE, names = FALSE, type = type),
    p5 =  quantile(x, probs = 0.05, na.rm = TRUE, names = FALSE, type = type),
    p10 =  quantile(x, probs = 0.10, na.rm = TRUE, names = FALSE, type = type),
    p25 =  quantile(x, probs = 0.25, na.rm = TRUE, names = FALSE, type = type),
    p50 =  quantile(x, probs = 0.50, na.rm = TRUE, names = FALSE, type = type),
    p75 =  quantile(x, probs = 0.75, na.rm = TRUE, names = FALSE, type = type),
    p90 =  quantile(x, probs = 0.9, na.rm = TRUE, names = FALSE, type = type),
    p95 =  quantile(x, probs = 0.95, na.rm = TRUE, names = FALSE, type = type),
    p99 =  quantile(x, probs = 0.99, na.rm = TRUE, names = FALSE, type = type),
    min = min(x,na.rm = T),
    max = max(x,na.rm = T))
  names(t) = paste0(name, names(t))
  return(t)
}

decimalplaces <- function(x){
  n = length(x)
  y = rep(NA,n)
  for(i in 1:n){
    z= strsplit(as.character(abs(x[i])), split = "[.]", fixed = F)[[1]]
    y[i] = nchar(z[2])
  }
  return(y)
}
#-------------------------------------------------------------------------------->
# Load data
#-------------------------------------------------------------------------------->
setwd(data_dir)
fee_rates <- read_sas(data_file = "fee_rates.sas7bdat")



## SOD
load(file = "./SOD/sumd_msa_market.rda")
load(file = "./SOD/msa_code.rda")

## Interest rates
load(file = "./InterestRates/Treas_FFT_FFF.rda")
load(file = "./InterestRates/Treas_FF_surprise.rda")


libor <- read.csv("./InterestRates/libor.csv", stringsAsFactors = F)
libor$date<-as.Date(libor$date)
libor$libor_3mo = as.numeric(libor$libor_3mo)
libor$libor_6mo = as.numeric(libor$libor_6mo)
libor$libor_12mo = as.numeric(libor$libor_12mo)

##---> BHCs level data
load(file = "./Y_9C/Y9C_vars.rda")

bhc_data=Y9C_vars[,c("id_rssd","date","assets","total_loans_leases",
                     "dep_time_more100K","dep_time_less100K",
                     "share_st_funding","core_dep","deposits","ROE")]

## ---> collapse bank-level data
load(file = "./CallReports/Call_vars.rda")
Call_vars = call
rm(call)
max(Call_vars$date)
call_deposit_vars =c("id_rssd", "date",
                     "td_mat","dep_mat",
                     "assets",
                     "EquityRatio",
                     "total_deposits",
                     "demand_deposits",
                     "transactions_deposits",
                     "nontransactions_deposits",
                     "deposits_uninsured",
                     "deposits_listing","deposits_cdar",
                     "time_above100K_q","time_below100K_q",

                     "int_timeabove100K",
                     "int_timebelow100K",
                     "int_sav_mmda","int_ff_repo",
                     "brokered_deposits",
                     "brokered_below100K",

                     "deposits_above100K",
                     "deposits_above250K",
                     "num_accounts_above100K",
                     "num_accounts_above250K",
                     "avg_balance_above100K",
                     "avg_balance_above250K",

                     "deposits_below100K",
                     "deposits_below250K",
                     "num_accounts_below100K",
                     "num_accounts_below250K",
                     "avg_balance_below100K",
                     "avg_balance_below250K")

call_deposits = Call_vars[Call_vars$date>"1997-01-01",
                          call_deposit_vars]
call_deposits = data.table(call_deposits)
call_deposits[,year:= year(date)]

call_deposits[,avg_balance_below100K_v2 := max(avg_balance_below100K, na.rm = T),
              by = c("id_rssd","year")]
call_deposits$avg_balance_below100K_v2[is.infinite(call_deposits$avg_balance_below100K_v2)]=NA
call_deposits$avg_balance_below100K[is.na(call_deposits$avg_balance_below100K)]=
  call_deposits$avg_balance_below100K_v2[is.na(call_deposits$avg_balance_below100K)]
call_deposits$num_accounts_below100K[is.na(call_deposits$num_accounts_below100K)]=
  round(call_deposits$deposits_below100K[is.na(call_deposits$num_accounts_below100K)]/call_deposits$avg_balance_below100K_v2[is.na(call_deposits$num_accounts_below100K)],0)
call_deposits[,avg_balance_below100K_v2:=NULL]


call_deposits[,avg_balance_below250K_v2 := max(avg_balance_below250K, na.rm = T),
              by = c("id_rssd","year")]
call_deposits$avg_balance_below250K_v2[is.infinite(call_deposits$avg_balance_below250K_v2)]=NA
call_deposits$avg_balance_below250K[is.na(call_deposits$avg_balance_below250K)]=
  call_deposits$avg_balance_below250K_v2[is.na(call_deposits$avg_balance_below250K)]
call_deposits$num_accounts_below250K[is.na(call_deposits$num_accounts_below250K)]=
  round(call_deposits$deposits_below250K[is.na(call_deposits$num_accounts_below250K)]/call_deposits$avg_balance_below250K_v2[is.na(call_deposits$num_accounts_below250K)],0)
call_deposits[,avg_balance_below250K_v2:=NULL]




# RW data
load(file = "./RW_data/RW_final/deposits/deposits_fill.rda")
deposits_fill$lon_rt=NULL
deposits_fill$lat_rt=NULL

names(deposits_fill)
length(unique(deposits_fill$id_rssd)) # 83,876 branch locations
length(unique(deposits_fill$acct_nbr_rt)) # 7,791 branch locations
length(unique(deposits_fill$id_rssd_hd_off)) # 11,374 banks


## collapse at the bank level
deposits_fill = data.table(deposits_fill)
deposits_b_m_t = deposits_fill[, list(nbranches = length(unique(id_rssd)),
                                      apy_cd3 = mean(apy_cd3,na.rm=T),
                                      apy_cd6 = mean(apy_cd6,na.rm=T),
                                      apy_cd12 = mean(apy_cd12,na.rm=T),
                                      apy_cd24 = mean(apy_cd24,na.rm=T),
                                      apy_cd60 = mean(apy_cd60,na.rm=T),
                                      apy_sav = mean(apy_sav,na.rm=T),
                                      apy_intck = mean(apy_intck,na.rm=T),
                                      apy_mmda = mean(apy_mmda,na.rm=T),
                                      apy_pmmda = mean(apy_pmmda,na.rm=T),
                                      depsumbr = sum(depsumbr,na.rm=T),
                                      dist_rt_loc = weighted.mean(dist_rt_loc,
                                                                  w=depsumbr,na.rm=T),
                                      dlndep = weighted.mean(dlndep, w=depsumbr,na.rm=T)),

                               by = c("id_rssd_hd_off",
                                      "msa_code","date")]

# total number of competitiors/market concentration


# add ranking
deposits_b_m_t[,nbanks:= length(unique(id_rssd_hd_off)), by = c("msa_code","date")]
deposits_b_m_t[,apy_cd3_rank:= rank(-apy_cd3,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]
deposits_b_m_t[,apy_cd6_rank:= rank(-apy_cd6,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]

deposits_b_m_t[,apy_cd12_rank:= rank(-apy_cd12,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]

deposits_b_m_t[,apy_cd24_rank:= rank(-apy_cd24,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]

deposits_b_m_t[,apy_cd60_rank:= rank(-apy_cd60,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]


deposits_b_m_t[,apy_sav_rank:= rank(-apy_sav,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]

deposits_b_m_t[,apy_intck_rank:= rank(-apy_intck,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]

## asset ranks
deposits_b_m_t[,assets_rank:= rank(-assets,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]

deposits_b_m_t[,deposits_rank:= rank(-total_deposits,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]

deposits_b_m_t[,deposits_br_rank:= rank(-depsumbr,na.last="keep", ties.method = "random"),
               by = c("msa_code","date")]







# merge call report data
deposits_b_m_t[,quarter:=quarter(date)]
deposits_b_m_t[,year:=year(date)]

call_deposits[,quarter:=quarter(date)]
call_deposits[,year:=year(date)]
call_deposits$date=NULL


deposits_b_m_t = merge(deposits_b_m_t, call_deposits,
                       by.x = c("id_rssd_hd_off","quarter","year"),
                       by.y = c("id_rssd","quarter","year"), all =F)


## add libor
deposits_b_m_t = merge(deposits_b_m_t,libor, all.x=T, by = "date")

## add other interest rates
#deposits_b_m_t = merge(deposits_b_m_t,FFT, all.x=T, by = "date")
deposits_b_m_t[,nbranches_m:= sum(nbranches,na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[,nbranches_b_avg:= mean(nbranches,na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[,assets_b_avg:= mean(assets,na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[,nbranches_dev_avg:= (nbranches-nbranches_b_avg)/nbranches_b_avg]
deposits_b_m_t[,assets_dev_avg:= (assets-assets_b_avg)/assets_b_avg]

summary(deposits_b_m_t$nbranches_dev_avg)
summary(deposits_b_m_t$assets_dev_avg)

## drop outliers
summary(deposits_b_m_t$apy_cd3)
summary(deposits_b_m_t$apy_cd6)
summary(deposits_b_m_t$apy_cd12)
summary(deposits_b_m_t$apy_cd24)
summary(deposits_b_m_t$apy_cd60)


deposits_b_m_t$apy_cd3[deposits_b_m_t$apy_cd3>20]=NA
deposits_b_m_t$apy_cd6[deposits_b_m_t$apy_cd6>20]=NA

# assets rank
## median rates by market
deposits_b_m_t[,apy_cd3_m:= median(apy_cd3,na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[,apy_cd6_m:= median(apy_cd6,na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[,apy_cd12_m:= median(apy_cd12,na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[,apy_cd24_m:= median(apy_cd24,na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[,apy_cd60_m:= median(apy_cd60,na.rm=T), by = c("msa_code","date")]

## deviations from the median

deposits_b_m_t[,apy_cd3_dev_m:=apy_cd3-apy_cd3_m]
deposits_b_m_t[,apy_cd6_dev_m:=apy_cd6-apy_cd6_m]
deposits_b_m_t[,apy_cd12_dev_m:=apy_cd12-apy_cd12_m]
deposits_b_m_t[,apy_cd24_dev_m:=apy_cd24-apy_cd24_m]
deposits_b_m_t[,apy_cd60_dev_m:=apy_cd60-apy_cd60_m]

## log-variables
summary(deposits_b_m_t$assets)
deposits_b_m_t[,log_assets:= log(assets)]
deposits_b_m_t$log_assets[is.infinite(deposits_b_m_t$log_assets)]=NA
summary(deposits_b_m_t$log_assets)

deposits_b_m_t = deposits_b_m_t[order(deposits_b_m_t$msa_code,
                                      deposits_b_m_t$date,
                                      -deposits_b_m_t$apy_cd3_rank),]


## number of banks in a market
deposits_b_m_t[, nbanks_m:=length(unique(id_rssd_hd_off)),
               by = c("msa_code","date")]
summary(deposits_b_m_t$nbanks_m)


#-------------------------------------------------------------------------------->
# Run fixed-effects regression----
se <- function(object) return(sqrt(diag(object)))
length(unique(deposits_b_m_t$id_rssd_hd_off))
length(unique(deposits_b_m_t$msa_code))
summary(deposits_b_m_t$int_timeabove100K)

## rates


## 3-month
##--libor only
# Note the regressions allow for clustered standard errors -- time x MSA market
reg_cd3_v1<-felm(data =deposits_b_m_t,
     formula = apy_cd3 ~ libor_3mo|0|0|msa_code+date,
     na.action = na.omit)
summary(reg_cd3_v1)

##--libor only+branches+assets
reg_cd3_v2<-felm(data =deposits_b_m_t,
                 formula = apy_cd3~libor_3mo+nbranches_dev_avg+assets_dev_avg|0|0|msa_code+date,
                 na.action = na.omit)
summary(reg_cd3_v2)

##--libor only+branches+assets + FE(bank FE)
reg_cd3_v3<-felm(data =deposits_b_m_t,
             formula = apy_cd3~ libor_3mo+nbranches_dev_avg|factor(id_rssd_hd_off)|0|msa_code+date)
summary(reg_cd3_v3)

##--libor only+branches+assets + FE(bank FE + market)

reg_cd3_v4<-felm(data =deposits_b_m_t,
             formula = apy_cd3~libor_3mo+nbranches_dev_avg|factor(msa_code)*factor(id_rssd_hd_off)|0|msa_code+date)



stargazer(reg_cd3_v1, reg_cd3_v2, reg_cd3_v3, reg_cd3_v4)



## 12-month

reg_cd12_v1<-felm(data =deposits_b_m_t,
                 formula = apy_cd12 ~ libor_12mo|0|0|msa_code+date,
                 na.action="na.exclude")
summary(reg_cd12_v1)

##--libor only+branches+assets
reg_cd12_v2<-felm(data =deposits_b_m_t,
                 formula = apy_cd12~libor_12mo+nbranches_dev_avg+assets_dev_avg|0|0|msa_code+date,
                 na.action=na.exclude)
summary(reg_cd12_v2)

##--libor only+branches+assets + FE(bank FE)
reg_cd12_v3<-felm(data =deposits_b_m_t,
                 formula = apy_cd12~ libor_12mo+nbranches_dev_avg+assets_dev_avg|factor(id_rssd_hd_off)|0|msa_code+date,
                 na.action=na.exclude)
summary(reg_cd12_v3)

##--libor only+branches+assets + FE(bank FE + market)

reg_cd12_v4<-felm(data =deposits_b_m_t,
                 formula = apy_cd12~libor_12mo+nbranches_dev_avg+assets_dev_avg|factor(msa_code)*factor(id_rssd_hd_off)|0|msa_code+date,
                 na.action=na.exclude)

summary(reg_cd12_v4)

stargazer(reg_cd12_v1, reg_cd12_v2, reg_cd12_v3, reg_cd12_v4,
         out = paste0(tables_dir,"Disp_12mo_bank_fe.tex"),
         covariate.labels = c("LIBOR 12-mo",
                              "$\frac{B_{j,m}-B_{m}}{B_{m}}$ ",
                              "$\frac{A_{j}-A_{m}}{A_{m}}$")
          )

res1 = quantile(reg_cd12_v1$residuals[which(year(as.Date(as.character(reg_cd12_v1$clustervar$date)))==2006)],
                probs = c(0.05, 0.25,0.5, 0.75, 0.95))
disp95_5_v1 = res1[5]-res1[1]

res2 = quantile(reg_cd12_v2$residuals[which(year(as.Date(as.character(reg_cd12_v1$clustervar$date)))==2006)],
                probs = c(0.05, 0.25,0.5, 0.75, 0.95))
disp95_5_v2 = res2[5]-res2[1]

res3 = quantile(reg_cd12_v3$residuals[which(year(as.Date(as.character(reg_cd12_v1$clustervar$date)))==2006)],
                probs = c(0.05, 0.25,0.5, 0.75, 0.95))
disp95_5_v3 = res3[5]-res3[1]

res4 = quantile(reg_cd12_v4$residuals[which(year(as.Date(as.character(reg_cd12_v1$clustervar$date)))==2006)],
                probs = c(0.05, 0.25,0.5, 0.75, 0.95))
disp95_5_v4 = res4[5]-res4[1]

## Dispersion measures P(95)-P(5)
paste(round(c(disp95_5_v1,disp95_5_v2,disp95_5_v3,disp95_5_v4),2), collapse= " & ")

## Standard deviation
paste(round(c(sd(reg_cd12_v1$residuals[which(year(as.Date(as.character(reg_cd12_v1$clustervar$date)))==2006)]),
              sd(reg_cd12_v2$residuals[which(year(as.Date(as.character(reg_cd12_v1$clustervar$date)))==2006)]),
              sd(reg_cd12_v3$residuals[which(year(as.Date(as.character(reg_cd12_v1$clustervar$date)))==2006)]),
              sd(reg_cd12_v4$residuals[which(year(as.Date(as.character(reg_cd12_v1$clustervar$date)))==2006)])),2),
      collapse = " & ")


reg_cd3_v5<-felm(data =deposits_b_m_t,
                 formula = apy_cd12~apy_cd12_m|factor(msa_code)*factor(id_rssd_hd_off)+factor(date)|0|msa_code+date)

summary(reg_cd3_v5)

res_df5 = data.frame(date =as.Date(as.character(reg_cd3_v5$clustervar$date)),
                     msa_code=as.numeric(reg_cd3_v5$clustervar$msa_code),
                     residuals = as.numeric(reg_cd3_v5$residuals))
res_df5 = res_df5[order(res_df4$msa_code,res_df4$date),]
res_df5 = data.table(res_df4)


res_df5 = res_df5[, list(p95 = quantile(residuals,probs = 0.95, na.rm=T),
                         p5 = quantile(residuals,probs = 0.05, na.rm=T)), by = "date"]
res_df5 = res_df5[order(res_df5$date),]

with(res_df5, plot(date, p95, type = "l", lwd = 2, ylim = c(-2,2)))
par(new=T)
with(res_df5, lines(date, p5, type = "l", lwd = 2))
abline(h=0)



#-------------------------------------------------------------------------------->
# Rate rank persistence----


## add quantiles

# 3-months
deposits_b_m_t[, apy_cd3_q1:= quantile(apy_cd3, probs = 0.25, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd3_q2:= quantile(apy_cd3, probs = 0.50, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd3_q3:= quantile(apy_cd3, probs = 0.75, na.rm=T), by = c("msa_code","date")]


deposits_b_m_t[, apy_cd3_q1_ind:= as.numeric(apy_cd3<=apy_cd3_q1)]
deposits_b_m_t[, apy_cd3_q2_ind:= as.numeric(apy_cd3> apy_cd3_q1 & apy_cd3<=apy_cd3_q2)]
deposits_b_m_t[, apy_cd3_q3_ind:= as.numeric(apy_cd3> apy_cd3_q2 & apy_cd3<=apy_cd3_q3)]
deposits_b_m_t[, apy_cd3_q4_ind:= as.numeric(apy_cd3> apy_cd3_q3)]


# 6-months
deposits_b_m_t[, apy_cd6_q1:= quantile(apy_cd6, probs = 0.25, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd6_q2:= quantile(apy_cd6, probs = 0.50, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd6_q3:= quantile(apy_cd6, probs = 0.75, na.rm=T), by = c("msa_code","date")]


deposits_b_m_t[, apy_cd6_q1_ind:= as.numeric(apy_cd6<=apy_cd6_q1)]
deposits_b_m_t[, apy_cd6_q2_ind:= as.numeric(apy_cd6> apy_cd6_q1 & apy_cd6<=apy_cd6_q2)]
deposits_b_m_t[, apy_cd6_q3_ind:= as.numeric(apy_cd6> apy_cd6_q2 & apy_cd6<=apy_cd6_q3)]
deposits_b_m_t[, apy_cd6_q4_ind:= as.numeric(apy_cd6> apy_cd6_q3)]




# 12-months
deposits_b_m_t[, apy_cd12_q1:= quantile(apy_cd12, probs = 0.25, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd12_q2:= quantile(apy_cd12, probs = 0.50, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd12_q3:= quantile(apy_cd12, probs = 0.75, na.rm=T), by = c("msa_code","date")]


deposits_b_m_t[, apy_cd12_q1_ind:= as.numeric(apy_cd12<=apy_cd12_q1)]
deposits_b_m_t[, apy_cd12_q2_ind:= as.numeric(apy_cd12> apy_cd12_q1 & apy_cd12<=apy_cd12_q2)]
deposits_b_m_t[, apy_cd12_q3_ind:= as.numeric(apy_cd12> apy_cd12_q2 & apy_cd12<=apy_cd12_q3)]
deposits_b_m_t[, apy_cd12_q4_ind:= as.numeric(apy_cd12> apy_cd12_q3)]


# 24-months
deposits_b_m_t[, apy_cd24_q1:= quantile(apy_cd24, probs = 0.25, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd24_q2:= quantile(apy_cd24, probs = 0.50, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd24_q3:= quantile(apy_cd24, probs = 0.75, na.rm=T), by = c("msa_code","date")]


deposits_b_m_t[, apy_cd24_q1_ind:= as.numeric(apy_cd24<=apy_cd24_q1)]
deposits_b_m_t[, apy_cd24_q2_ind:= as.numeric(apy_cd24> apy_cd24_q1 & apy_cd24<=apy_cd24_q2)]
deposits_b_m_t[, apy_cd24_q3_ind:= as.numeric(apy_cd24> apy_cd24_q2 & apy_cd24<=apy_cd24_q3)]
deposits_b_m_t[, apy_cd24_q4_ind:= as.numeric(apy_cd24> apy_cd24_q3)]


# 60-months
deposits_b_m_t[, apy_cd60_q1:= quantile(apy_cd60, probs = 0.25, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd60_q2:= quantile(apy_cd60, probs = 0.50, na.rm=T), by = c("msa_code","date")]
deposits_b_m_t[, apy_cd60_q3:= quantile(apy_cd60, probs = 0.75, na.rm=T), by = c("msa_code","date")]


deposits_b_m_t[, apy_cd60_q1_ind:= as.numeric(apy_cd60<=apy_cd60_q1)]
deposits_b_m_t[, apy_cd60_q2_ind:= as.numeric(apy_cd60> apy_cd60_q1 & apy_cd60<=apy_cd60_q2)]
deposits_b_m_t[, apy_cd60_q3_ind:= as.numeric(apy_cd60> apy_cd60_q2 & apy_cd60<=apy_cd60_q3)]
deposits_b_m_t[, apy_cd60_q4_ind:= as.numeric(apy_cd60> apy_cd60_q3)]
#------------------------------------------------------------------------------------------------------>
## duration of rankings
deposits_b_m_t = deposits_b_m_t[order(deposits_b_m_t$id_rssd_hd_off,
                                      deposits_b_m_t$msa_code,
                                      deposits_b_m_t$date),]



# 3-month-------------------------------------------------------------->

cd3 <- deposits_b_m_t[deposits_b_m_t$nbanks_m>20 &
                      deposits_b_m_t$date<="2009-01-01", list(id_rssd_hd_off,msa_code,date,
                             apy_cd3,apy_cd3_rank,assets_rank,
                             apy_cd3_q1_ind,
                             apy_cd3_q2_ind,
                             apy_cd3_q3_ind,
                             apy_cd3_q4_ind)]
cd3 = cd3[order(cd3$id_rssd_hd_off,
                cd3$msa_code,
                cd3$date),]

cd3[, bank_m_t:=.N,by = c("msa_code","date") ]

## q1
cd3[ ,cd3_q1_tran:=diff(apy_cd3_q1_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd3[cd3$apy_cd3_q1_ind==1,cd3_dur_q1:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]
##q2
cd3[ ,cd3_q2_tran:=diff(apy_cd3_q2_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd3[cd3$apy_cd3_q2_ind==1,cd3_dur_q2:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]
##q3
cd3[ ,cd3_q3_tran:=diff(apy_cd3_q3_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd3[cd3$apy_cd3_q3_ind==1,cd3_dur_q3:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]
##q4
cd3[ ,cd3_q4_tran:=diff(apy_cd3_q4_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd3[cd3$apy_cd3_q4_ind==1,cd3_dur_q4:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]


cd3[ cd3$apy_cd3_q1_ind==1,cd3_quartile:=1]
cd3[ cd3$apy_cd3_q2_ind==1,cd3_quartile:=2]
cd3[ cd3$apy_cd3_q3_ind==1,cd3_quartile:=3]
cd3[ cd3$apy_cd3_q4_ind==1,cd3_quartile:=4]

cd3 = cd3[cd3$cd3_q1_tran==-1 |cd3$cd3_q2_tran==-1|cd3$cd3_q3_tran==-1|cd3$cd3_q4_tran==-1,]
cd3[,cd3_duration:= as.numeric(date - lag(date,1))/7, by = c("id_rssd_hd_off","msa_code") ]
cd3 = cd3[!is.na(cd3$cd3_duration),]

cd3[cd3$cd3_duration<=4,duration_category:=1]
cd3[cd3$cd3_duration %in% seq(5,8),duration_category:=2]
cd3[cd3$cd3_duration %in% seq(9,12),duration_category:=3]
cd3[cd3$cd3_duration %in% seq(13,16),duration_category:=4]
cd3[cd3$cd3_duration %in% seq(17,23),duration_category:=5]
cd3[cd3$cd3_duration >24 ,duration_category:=6]
cd3=cd3[!is.na(cd3$duration_category)]
cd3=cd3[!is.na(cd3$cd3_quartile)]

cd3_avg_durations = cd3[!is.na(cd3$cd3_quartile),
                              list(dur_mean = mean(cd3_duration,na.rm=T)/4,
                               dur_median = median(cd3_duration,na.rm=T)/4,
                               dur_p25 = quantile(cd3_duration,prob =0.25,na.rm=T)/4,
                               dur_p75 = quantile(cd3_duration,prob =0.75,na.rm=T)/4,
                               dur_max = max(cd3_duration, na.rm=T)/4),
                        by = "cd3_quartile"]

cd3_avg_durations = cd3_avg_durations[order(cd3_avg_durations$cd3_quartile)]


## all observations
cd3_durations = cd3[,list(Nfirms=length(unique(id_rssd_hd_off))),
                    by = c("duration_category","cd3_quartile")]
cd3_durations[,NN:=sum(Nfirms), by = "cd3_quartile"]

cd3_durations[,freq:=Nfirms/NN, by = "cd3_quartile"]
cd3_durations=cd3_durations[,list(duration_category,cd3_quartile,freq)]


cd3_durations = dcast(cd3_durations, duration_category ~ cd3_quartile,value.var = "freq")
names(cd3_durations) = paste0("q",names(cd3_durations))

stargazer(cd3_durations, summary = F, rownames = F)

# 12-month-------------------------------------------------------------->

cd12 <- deposits_b_m_t[deposits_b_m_t$nbanks_m>20 &
                       deposits_b_m_t$date<="2009-01-01",
                       list(id_rssd_hd_off,msa_code,date,
                              apy_cd12,apy_cd12_rank,assets_rank,
                              apy_cd12_q1_ind,
                              apy_cd12_q2_ind,
                              apy_cd12_q3_ind,
                              apy_cd12_q4_ind)]
cd12 = cd12[order(cd12$id_rssd_hd_off,
                  cd12$msa_code,
                  cd12$date),]

cd12[, bank_m_t:=.N,by = c("msa_code","date") ]

## q1
cd12[ ,cd12_q1_tran:=diff(apy_cd12_q1_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd12[cd12$apy_cd12_q1_ind==1,cd12_dur_q1:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]
##q2
cd12[ ,cd12_q2_tran:=diff(apy_cd12_q2_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd12[cd12$apy_cd12_q2_ind==1,cd12_dur_q2:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]
##q3
cd12[ ,cd12_q3_tran:=diff(apy_cd12_q3_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd12[cd12$apy_cd12_q3_ind==1,cd12_dur_q3:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]
##q4
cd12[ ,cd12_q4_tran:=diff(apy_cd12_q4_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd12[cd12$apy_cd12_q4_ind==1,cd12_dur_q4:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]


cd12[ cd12$apy_cd12_q1_ind==1,cd12_quartile:=1]
cd12[ cd12$apy_cd12_q2_ind==1,cd12_quartile:=2]
cd12[ cd12$apy_cd12_q3_ind==1,cd12_quartile:=3]
cd12[ cd12$apy_cd12_q4_ind==1,cd12_quartile:=4]

cd12 = cd12[cd12$cd12_q1_tran==-1 |cd12$cd12_q2_tran==-1|cd12$cd12_q3_tran==-1|cd12$cd12_q4_tran==-1,]
cd12[,cd12_duration:= as.numeric(date - lag(date,1))/7, by = c("id_rssd_hd_off","msa_code") ]
summary(cd12$cd12_duration)

cd12 = cd12[!is.na(cd12$cd12_duration),]
cd12[cd12$cd12_duration<=4,duration_category:=1]
cd12[cd12$cd12_duration %in% seq(5,8),duration_category:=2]
cd12[cd12$cd12_duration %in% seq(9,12),duration_category:=3]
cd12[cd12$cd12_duration %in% seq(13,16),duration_category:=4]
cd12[cd12$cd12_duration %in% seq(17,23),duration_category:=5]
cd12[cd12$cd12_duration >24 ,duration_category:=6]
cd12=cd12[!is.na(cd12$duration_category)]
cd12=cd12[!is.na(cd12$cd12_quartile)]


## all observations
cd12_durations = cd12[,list(Nfirms=length(unique(id_rssd_hd_off))),
                      by = c("duration_category","cd12_quartile")]
cd12_durations[,NN:=sum(Nfirms), by = "cd12_quartile"]

cd12_durations[,freq:=Nfirms/NN, by = "cd12_quartile"]
cd12_durations=cd12_durations[,list(duration_category,cd12_quartile,freq)]


cd12_durations = dcast(cd12_durations, duration_category ~ cd12_quartile,value.var = "freq")
names(cd12_durations) = paste0("q",names(cd12_durations))



cd12_avg_durations = cd12[!is.na(cd12$cd12_quartile),
                        list(dur_mean = mean(cd12_duration,na.rm=T)/4,
                             dur_median = median(cd12_duration,na.rm=T)/4,
                             dur_p25 = quantile(cd12_duration,prob =0.25,na.rm=T)/4,
                             dur_p75 = quantile(cd12_duration,prob =0.75,na.rm=T)/4,
                             dur_max = max(cd12_duration, na.rm=T)/4),
                        by = "cd12_quartile"]

cd12_avg_durations = cd12_avg_durations[order(cd12_avg_durations$cd12_quartile)]


cd_3_12_durations = cbind(cd3_durations,cd12_durations)
stargazer(cd_3_12_durations, summary = F, rownames = F)


cd_3_12_avg_durations = cbind(t(cd3_avg_durations),t(cd12_avg_durations))

stargazer(cd_3_12_avg_durations, summary = F, rownames = F, digits = 0)


## Transition matrix--1-month horizon

cd12 <- deposits_b_m_t[, list(id_rssd_hd_off,msa_code,date,
                              apy_cd12,apy_cd12_rank,assets_rank,
                              apy_cd12_q1_ind,
                              apy_cd12_q2_ind,
                              apy_cd12_q3_ind,
                              apy_cd12_q4_ind)]
cd12 = cd12[order(cd12$id_rssd_hd_off,
                  cd12$msa_code,
                  cd12$date),]
## q1
cd12[ ,cd12_q1_tran:=diff(apy_cd12_q1_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd12[cd12$apy_cd12_q1_ind==1,cd12_dur_q1:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]
##q2
cd12[ ,cd12_q2_tran:=diff(apy_cd12_q2_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd12[cd12$apy_cd12_q2_ind==1,cd12_dur_q2:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]
##q3
cd12[ ,cd12_q3_tran:=diff(apy_cd12_q3_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd12[cd12$apy_cd12_q3_ind==1,cd12_dur_q3:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]
##q4
cd12[ ,cd12_q4_tran:=diff(apy_cd12_q4_ind,1), by = c("id_rssd_hd_off","msa_code") ]
cd12[cd12$apy_cd12_q4_ind==1,cd12_dur_q4:=sequence(.N), by = c("id_rssd_hd_off","msa_code") ]


cd12[ cd12$apy_cd12_q1_ind==1,cd12_quartile:=1]
cd12[ cd12$apy_cd12_q2_ind==1,cd12_quartile:=2]
cd12[ cd12$apy_cd12_q3_ind==1,cd12_quartile:=3]
cd12[ cd12$apy_cd12_q4_ind==1,cd12_quartile:=4]

## transitions 1 month and 3-months
cd12 = cd12[order(cd12$id_rssd_hd_off,
                  cd12$msa_code,
                  cd12$date),]
cd12[, quartile_next_1m:=lead(cd12_quartile,n=4),
     by = c("id_rssd_hd_off","msa_code")]

cd12[, quartile_next_3m:=lead(cd12_quartile,n=12),
     by = c("id_rssd_hd_off","msa_code")]


cd12[,trans_1m:= paste0("p",cd12_quartile,quartile_next_1m)]
cd12[,trans_3m:= paste0("p",cd12_quartile,quartile_next_3m)]


cd12_trans_1m = cd12[!is.na(cd12$cd12_quartile)&!is.na(cd12$quartile_next_1m),
                  list(NN=.N), by = c("trans_1m","cd12_quartile")]

cd12_trans_1m[, N:=sum(NN), by ="cd12_quartile" ]
cd12_trans_1m[, prob := NN/N]



cd12_trans_3m = cd12[!is.na(cd12$cd12_quartile)&!is.na(cd12$quartile_next_3m),
                     list(NN=.N), by = c("trans_3m","cd12_quartile")]

cd12_trans_3m[, N:=sum(NN), by ="cd12_quartile" ]
cd12_trans_3m[, prob := NN/N]

cd12_trans_1m = cd12_trans_1m[order(cd12_trans_1m$trans_1m)]

trans_1m = rbind(cd12_trans_1m$prob[cd12_trans_1m$trans_1m %in% c("p11","p12","p13","p14")],
                 cd12_trans_1m$prob[cd12_trans_1m$trans_1m %in% c("p21","p22","p23","p24")],
                 cd12_trans_1m$prob[cd12_trans_1m$trans_1m %in% c("p31","p32","p33","p34")],
                 cd12_trans_1m$prob[cd12_trans_1m$trans_1m %in% c("p41","p42","p43","p44")])

cd12_trans_3m = cd12_trans_3m[order(cd12_trans_3m$trans_3m)]
trans_3m = rbind(cd12_trans_3m$prob[cd12_trans_3m$trans_3m %in% c("p11","p12","p13","p14")],
                 cd12_trans_3m$prob[cd12_trans_3m$trans_3m %in% c("p21","p22","p23","p24")],
                 cd12_trans_3m$prob[cd12_trans_3m$trans_3m %in% c("p31","p32","p33","p34")],
                 cd12_trans_3m$prob[cd12_trans_3m$trans_3m %in% c("p41","p42","p43","p44")])



trans = cbind(trans_1m,trans_3m)


stargazer(trans, rownames = F, summary = F, digits = 2)
#-------------------------------------------------------------------------------->