#-------------------------------------------------------------------------------->
# File: Analysis_structural_estimates.R
# Description:
# Author: Vladimir Yankov
# Date: October 2017
#-------------------------------------------------------------------------------->
# Clear workspace
rm(list = ls())
# Folders----
root_dir <- "/fire/home/m1vly00/"
data_dir <-"/ofs/scratch-m1vly00/Data/"
sod_dir <-"/ofs/scratch-m1vly00/Data/SOD/"
final_data_dir<- "/ofs/scratch-m1vly00/Data/Data_InSearchOfRiskFreeAsset/str_est/"
lib_loc = paste0(root_dir, "Code/R/Packages/")
tables_dir = paste0(root_dir,"Academic_Research_Vladimir_Yankov/InSearchRisklessAsset/Results/Tables/" )
figures_dir = paste0(root_dir,"Academic_Research_Vladimir_Yankov/InSearchRisklessAsset/Results/Figures/")
code_proj_dir = paste0(root_dir,"Academic_Research_Vladimir_Yankov/InSearchRisklessAsset/Code/StructuralEstimation/functions/")
#-------------------------------------------------------------------------------->
# Libraries----
source(paste0(code_proj_dir,"functions.R"))
require(texreg)
require(nleqslv)
# Set multi-threading
setMKLthreads(18)
#-------------------------------------------------------------------------------->
# User-defined functions----
source(paste0(code_proj_dir,"structural_est_functions.R"))

Fzero = function(z,R,qvec,R_min,R_tilde,sigma,beta,tau){
  N = length(qvec)
  q1 = qvec[1]
  sum_q_F = 0
  for(i in 1:N){
    sum_q_F = sum_q_F+ i*qvec[i]*z^(i-1)
  }
  y = ( (R_tilde - R)*(1-h(R,sigma,beta,tau)))*sum_q_F - (R_tilde - R_min)*(1-h(R_min,sigma,beta,tau))*q1
  return(y)
}

F_R <-function(R){
  FFzero = function(z){return(Fzero(z,R,qvec,R_min,R_tilde,sigma,beta,tau))}
  FF = nleqslv(1/2,FFzero)$x
  if(R>R_max){FF=1}
  if(R<R_min){FF=0}
  return(FF)
}
F_R = Vectorize(F_R)

f_R = function(R){
  FF = F_R(R)
  q_sum_1 = 0
  q_sum_2 = 0
  N=length(qvec)
  for(i in 1:N){
    q_sum_1 = q_sum_1+i*qvec[i]*FF^(i-1)
    q_sum_2 = q_sum_2+i*(i-1)*qvec[i]*FF^(i-2)
  }
  ff = ((-1)*psi_prime(R,R_tilde, sigma, beta, tau)/psi(R,R_tilde, sigma, beta, tau))*q_sum_1/q_sum_2
  if(R>R_max){ff=0}
  if(R<R_min){ff=0}
  return(ff)
}
f_R = Vectorize(f_R)

V_k = function(k){
  int = function(R){
    return(phi(R,sigma, beta, tau)*k*F_R(R)^(k-1)*f_R(R))
  }
  VV = integrate(int,R_min,R_max)$value
  return(VV)
}
V_k = Vectorize(V_k)

V_all = foreach(k=1:Nbanks, .combine ="sum")%do%{
  VV=V_k(k)*qvec[k]
  return(VV)
}
V_R_t =phi(R_tilde,sigma, beta, tau)
# end----
# Plot margins----
dev.off()
par(mar=c(4,3,0.5,0.5))
w = 10
h = 1.61803
#-------------------------------------------------------------------------------->
# Load data----
#-------------------------------------------------------------------------------->
setwd(paste0(data_dir,"RW_data/RW_final/deposits/"))
load(file="deposits_msa.rda")
load(file="deposits_msa_m.rda")
load(file = "/ofs/scratch-m1vly00/Data/Data_InSearchOfRiskFreeAsset/MSA_Level_Data/deposits_cd_msa.rda")
load("msa_code.rda")
load(file=paste0(sod_dir,"sumd_msa_market.rda"))
sumd_msa$msa_code=sumd_msa$msa_code_close
sumd_msa$msa_code_close=NULL
setwd(final_data_dir)
load(file = "F_xi_msa_m.rda")
load(file = "qvec_est_msa.rda")
load(file = "/ofs/scratch-m1vly00/Data/InterestRates/FFT_Treas.rda")
load(file = paste0(final_data_dir,"F_xi_msa_m.rda"))
load(file =paste0(final_data_dir,"F_xi.rda"))
load(file = paste0(final_data_dir,"qvec_est_msa.rda"))
# Add data on time deposits by bank
setwd(data_dir)
load(file = "./InterestRates/FFT_Treas.rda")
load(file = "./CallReports/Call_funding.rda")
load(file = "./CallReports/Call_assets.rda")
load(file = "./Y_9C/Y9C_vars.rda")
load(file = "./SOD/sumd_1975_2014.rda")
load(file = "./SOD/sumd_msa_market.rda")
msa_educ = read.csv("/ofs/scratch-m1vly00/Data/Data_InSearchOfRiskFreeAsset/MSA_Level_Data/education_by_msa.csv")
msa_codes <- unique(qvec_msa_data$msa_code)
msa_educ = msa_educ[msa_educ$fips_code %in% msa_codes,]
unique(msa_educ$msa_name)
# primary credit/federal funds rate
ffr <- read.csv(file = "/ofs/scratch-m1vly00/Data/Data_InSearchOfRiskFreeAsset/InterestRates/Daily_7_Day.csv", stringsAsFactors = F)
primcredit <- read.csv(file = "/ofs/scratch-m1vly00/Data/Data_InSearchOfRiskFreeAsset/InterestRates/primary_credit.csv", stringsAsFactors = F)
names(primcredit)<- tolower(names(primcredit))
primcredit$date = as.Date(primcredit$date,format = "%m/%d/%Y")
names(ffr) <- tolower(names(ffr))
ffr$date = as.Date(ffr$date)
ffr = data.table(ffr)
primcredit = merge(ffr,primcredit,by="date",all=T)
primcredit = data.table(primcredit)
primcredit[,pcredit8:=nafill(pcredit8,type="locf")]
with(primcredit, plot(date, dff, type = "l"))
par(new=T)
with(primcredit,lines(date,pcredit8, type="l", col = "red"))
primcredit[,y:=year(date)]
primcredit[,m:=month(date)]
primcredit = primcredit[,list(dff=mean(dff),prcredit8=mean(pcredit8)), by = c("y","m")]
with(primcredit[primcredit$y>2004],mean(prcredit8-dff, na.rm=T))

#-------------------------------------------------------------------------------->
# Goodness of fit -- Kolmogorov Smirnov----
#-------------------------------------------------------------------------------->
ks_pvalue_y = qvec_msa_m[, list(ks_p = round(100*mean(ks_p,na.rm =T),1),
                                nbanks = mean(nbanks,na.rm=T)),
                         by = c("msa_code","year")]
ks_pvalue_y = ks_pvalue_y[order(ks_pvalue_y$msa_code,
                                ks_pvalue_y$year)]
sumd_msa_market = data.table(sumd_msa_market)
sumd_msa_market$deposits_msa=as.numeric(sumd_msa_market$deposits_msa)
sumd_msa_market[,deposits_agg:=sum(deposits_msa,na.rm=T), by = "year"]
sumd_msa_market[,share_dep_msa:= 100*deposits_msa/deposits_agg]
summary(sumd_msa_market$share_dep_msa)
ks_pvalue_y = merge(ks_pvalue_y,
                    sumd_msa_market[,list(msa_code,year,deposits_msa,deposits_agg,share_dep_msa)],
                    all.x = T)
ks_pvalue_y[ ,share_dep_msa_last :=share_dep_msa[year==2011][1], by = "msa_code" ]
ks_pvalue_y$share_dep_msa[ks_pvalue_y$year>2011] =
  ks_pvalue_y$share_dep_msa_last[ks_pvalue_y$year>2011]

ks_stat_agg= qvec_msa_m[, list(ks_p5 =round(100*quantile(ks_p,probs = 0.05,na.rm=T),1),
                               ks_p25 =round(100*quantile(ks_p,probs = 0.25,na.rm=T),1),
                               ks_med=round(100*median(ks_p,na.rm=T),1),
                               ks_p75 =round(100*quantile(ks_p,probs = 0.75,na.rm=T),1),
                               ks_p95 =round(100*quantile(ks_p,probs = 0.95,na.rm=T),1),
                               ks_p_w = round(100*weighted.mean(ks_p,w=dep_msa,na.rm=T),1)),
                               by="year"]

ks_pvalue_agg = ks_pvalue_y[, list(nmsa_p10 = length(unique(msa_code[ks_p<10])),
                                   share_p10 = round(sum(share_dep_msa[ks_p<10],na.rm=T),0),
                                   nmsa = length(unique(msa_code))),
                            by = "year"]

ks_pvalue_agg = ks_pvalue_agg[order(ks_pvalue_agg$year)]
ks_stat_agg = merge(ks_stat_agg,ks_pvalue_agg, by = "year")


##>>> Export table KS statistics----
stargazer(ks_stat_agg,summary=F,type="latex",
          rownames = F, digits=1,
          column.labels = c("year","5","25","Median","75","95","Mean","#(<10%)","Share<10%",
                            "Num.MSA"))




#-------------------------------------------------------------------------------->
#  Plot structural estimates----
#-------------------------------------------------------------------------------->
setwd(figures_dir)
date_axis = unique(qvec_agg_m$date)
date_axis = date_axis[which(month(date_axis)==6)]
date_axis2 = seq(min(date_axis), max(date_axis), by = 365)
date_axis2 = c(date_axis2,max(qvec_agg_m$date))


#----------------------------> Plots: Search segments----
par(mar=c(4,3,0.5,0.5))
# plot of q1

pdf(file = "Estimate_q1_w.pdf", height = 10, width = 16)
with(qvec_agg_m,plot(date,q1_p95,type ="l",
                     cex=2,
                     cex.lab=2,
                     xaxt = "n",
                     xlab = "",
                     ylab = "",
                     main = "",
                     cex.axis =2,
                     lwd = 2,
                     ylim = c(0.02,0.99), col = "gray"))

axis.Date(side = 1, qvec_agg_m$date, at =date_axis2 , format = "%Y", las = 2, cex.axis = 2)
par(new =T)
with(qvec_agg_m, polygon(c(date, rev(date)), c(q1_p5,rev(q1_p95)),
                         col = "grey", border = "grey70", ylim = c(0,0,9)))

with(qvec_agg_m,lines(date,q1,type ="l", ylim = c(0.02,0.9), col = "red",lwd=2))

with(qvec_agg_m,lines(date,q1_w,type ="l", ylim = c(0.02,0.7),lwd=2,col="blue"))

legend("topleft", c(TeX("5/95th percentiles of MSA estimates"),
                    TeX("Mean of MSA estimates"),
                    TeX("Deposit-weighted mean of MSA estimates")),
       cex = 2,
       col = c("gray",
               "red",
               "blue"),bg = "white",bty="n",
       lwd = c(20,2,2),
       lty = c(1,1,1))

dev.off()

pdf(file = "Estimate_qN_w.pdf", height = 10, width = 16)
with(qvec_agg_m,plot(date,q2_w,type ="l", ylim = c(0,0.5), col = "red", lwd=2,
                     cex=2,
                     cex.lab=2,
                     cex.axis = 2,
                     xaxt = "n",
                     xlab = "",
                     ylab = "",
                     main = ""))
axis.Date(side = 1, qvec_agg_m$date, at =date_axis2 ,
          format = "%Y", las = 2, cex.axis =2)
par(new =T)
with(qvec_agg_m,lines(date,qN_w,type ="l", col = "blue", lwd =2))
with(qvec_agg_m,lines(date,search_avg_w,type ="l", col = "green", lwd =2))



legend("topleft", c(TeX("$q_2$"),
                    TeX("$q_N$"),
                    TeX("Fraction sampled")),
       cex = 2, ncol = 3,
       col = c("red",
               "blue", "green"),bg = "white",bty="n",
       lwd = c(2,2),
       lty = c(1,1))
dev.off()


pdf(file = "Estimate_q2_w.pdf", height = 10, width = 16)
with(qvec_agg_m,plot(date,q2,type ="l", ylim = c(0.02,0.7), col = "red"))
par(new =T)
with(qvec_agg_m,lines(date,q2_w,type ="l", ylim = c(0.02,0.7)))
dev.off()


pdf(file = "Estimate_q3_w.pdf", height = 10, width = 16)
with(qvec_agg_m,plot(date,q3,type ="l", ylim = c(0.02,0.7), col = "red"))
par(new =T)
with(qvec_agg_m,lines(date,q3_w,type ="l", ylim = c(0.02,0.7)))
dev.off()



pdf(file = "Estimate_q1_qN.pdf", height = 10, width = 16)
with(qvec_agg_m,plot(date,q1_w,type ="l", ylim = c(0.02,0.65), lwd=2,
                     ylab = "",xlab=""))
par(new=T)
with(qvec_agg_m,lines(date,q2_w,type ="l", col = "green", lwd=2))
with(qvec_agg_m,lines(date,qN_w,type ="l", col="purple", lwd=2))
with(qvec_agg_m,lines(date,search_avg_w,type ="l",lty=2, col="blue", lwd=2))

abline(h=0)
grid(nx=NULL,ny=NULL, col ="darkgray", lwd=1)
legend('topleft',bty="n",ncol=3,
       c(TeX('$q_1$'),TeX("$q_2$"),TeX("$q_N$"),
         TeX("$\\frac{1}{N}\\sum_{k=1}^N k q_k$")),
       lty=1,lwd =2,col=c("black","green","purple", "blue"))
dev.off()

pdf(file = "Estimate_q1_qN_annual.pdf", height = 10, width = 16)
with(qvec_agg_y,plot(year,q1_w,type ="l", ylim = c(0.02,0.7), lwd=2,
                     cex.axis= 1.5, xaxt = "n",
                     ylab = "",xlab=""))
axis(side=1, at=1996:2016, cex.axis = 1.5)


par(new=T)
with(qvec_agg_y,lines(year,q2_w,type ="l", col = "green", lwd=2))
with(qvec_agg_y,lines(year,qN_w,type ="l", col="purple", lwd=2))
with(qvec_agg_y,lines(year,search_avg_w,type ="l",lty=2, col="blue", lwd=2))

abline(h=0)
grid(nx=NA,ny=NULL, col ="darkgray", lwd=1)
abline(v=1996:2016,col="darkgray",lwd=1,lty=2)
legend('topleft',bty="n",ncol=3,cex =2,
       c(TeX('$q_1$'),TeX("$q_2$"),TeX("$q_N$"),
         TeX("$\\frac{1}{N}\\sum_{k=1}^N k q_k$")),
       lty=c(1,1,1,2),lwd =2,col=c("black","green","purple", "blue"))
dev.off()
#-------------------------------------------------------------------------------->










# --------------------> Search cost distribution: Delta_k/q_k----
F_xi_msa_m[,delta_R_1:=10^4*delta1/phi_prime(R=R_min,beta=0.99,tau=1, sigma=1.27) ]

# plot of xi
F_xi_msa_m=data.table(F_xi_msa_m)
F_xi_agg = F_xi_msa_m[, list(delta1 =mean(deltak[q_k==1],na.rm=T),
                             delta2 =mean(deltak[q_k==2],na.rm=T),
                             delta3 =mean(deltak[q_k==3],na.rm=T),
                             delta4 =mean(deltak[q_k==4],na.rm=T),
                             delta5 =mean(deltak[q_k==5],na.rm=T),
                             delta6 =mean(deltak[q_k==6],na.rm=T),
                             delta7 =mean(deltak[q_k==7],na.rm=T),
                             delta8 =mean(deltak[q_k==8],na.rm=T),
                             delta9 =mean(deltak[q_k==9],na.rm=T),
                             delta10 =mean(deltak[q_k==10],na.rm=T),
                             delta11 =mean(deltak[q_k==11],na.rm=T),
                             delta12 =mean(deltak[q_k==12],na.rm=T),
                             delta13 =mean(deltak[q_k==13],na.rm=T),
                             delta14 =mean(deltak[q_k==14],na.rm=T),
                             delta15 =mean(deltak[q_k==15],na.rm=T),
                             delta16 =mean(deltak[q_k==16],na.rm=T),
                             delta17 =mean(deltak[q_k==17],na.rm=T),
                             delta18 =mean(deltak[q_k==18],na.rm=T),
                             delta19 =mean(deltak[q_k==19],na.rm=T),
                             delta20 =mean(deltak[q_k==20],na.rm=T),
                             deltaN =mean(deltak[q_k==nbanks],na.rm=T),
                             R_min = weighted.mean(R_min,w=dep_msa,
                                                   na.rm=T),
                             R_range = 100*mean(R_range,w=dep_msa,
                                                na.rm=T)),
                      by = "date"]
F_xi_agg = F_xi_agg[order(F_xi_agg$date),]
F_xi_agg[,delta_R_1:=10^4*delta1/phi_prime(R=R_min,beta=0.99,tau=1, sigma=1.27)]
F_xi_agg[,delta_R_2:=10^4*delta2/phi_prime(R=R_min,beta=0.99,tau=1, sigma=1.27)]
F_xi_agg[,delta_R_3:=10^4*delta3/phi_prime(R=R_min,beta=0.99,tau=1, sigma=1.27)]
F_xi_agg[,delta_R_10:=10^4*delta10/phi_prime(R=R_min,beta=0.99,tau=1, sigma=1.27)]
F_xi_agg[,delta_R_20:=10^4*delta20/phi_prime(R=R_min,beta=0.99,tau=1, sigma=1.27)]


## Figure: Plot of delta / indifference points

par(mar=c(6,6,0.5,0.5))
pdf(file = paste0(figures_dir,"Delta_estimates.pdf"), height =10,width=16)

with(F_xi_agg,plot(date,delta_R_1,
                   lwd=2,
                   xaxt = "n",
                   ylim = c(0,70),
                   xlab="",
                   lty=1,
                   ylab = "basis points",
                   type="l", col = "darkgreen",
                   cex.lab =2.3,
                   cex.axis =2.3,
                   cex =2.3))
abline(h=0,lwd=0.5)
axis.Date(side = 1, qvec_treas_m$date, at =date_axis2,
          cex.axis = 2.3,
          format = "%Y", las = 2.3)
par(new=T)

with(F_xi_agg,lines(date,delta_R_2,col = "purple",lwd=2))

with(F_xi_agg,lines(date,delta_R_3, col = "blue",lwd=2))
with(F_xi_agg,lines(date,delta_R_10, col = "darkgray",lwd=2))
with(F_xi_agg,lines(date,delta_R_20, col = "black", lwd=2))

legend('topright',
       c("Sample size:",
         TeX("k=1"),TeX("k=2$"),
         TeX("$k=3$"),TeX("$k=10$"),
         TeX("$k=20$")),
       bty="n",ncol=1,cex =2.3,
       lty=c(1,1,1,1,1),
       lwd =2,
       #pch=c(1,2,5,0,0),
       col=c("white","darkgreen","purple","blue","darkgray","black"))

dev.off()

# plot the cross-section of deltas
date2006=F_xi_agg$date[which(F_xi_agg$delta1==max(F_xi_agg$delta1))]
F_xi_agg=data.frame(F_xi_agg)
delta_2006 = F_xi_agg[F_xi_agg$date==date2006,grep("delta",names(F_xi_agg))]
delta_2006 = delta_2006[1:21]
R_min2006 =F_xi_agg[F_xi_agg$date==date2006,"R_min"]

datelast=max(F_xi_agg$date)
delta_last = F_xi_agg[F_xi_agg$date==max(F_xi_agg$date),grep("delta",names(F_xi_agg))]
delta_last = delta_last[1:21]
R_minlast =F_xi_agg[F_xi_agg$date==max(F_xi_agg$date),"R_min"]

pdf(file = paste0(figures_dir,"Delta_xs.pdf"), height =10,width=16)
plot(1:21,10^4*delta_2006/phi_prime(R=R_min2006,beta=0.99,tau=1, sigma=1.27),
     xlab ="Number of banks sampled",
     ylab = "basis points",
     xaxt = "n",
     cex=2.3,
     cex.axis = 2.3,
     cex.lab=2.3,
     type="b", lwd=2)
axis(1,at = 1:21,labels = as.character(c(1:20,"N")), cex.axis=2.3)
par(new=T)
lines(1:21,10^4*delta_last/phi_prime(R=R_minlast,beta=0.99,tau=1, sigma=1.27),
      type="b",pch=2,lwd=2,col ="red",cex= 2.3)

legend('topright',as.character(year(c(date2006,datelast))),
       bty="n",
       cex=2.3,
       lty=1,lwd=2,col=c("black","red"),
       pch=c(1,2))

dev.off()

# relationship between deltas and qs
par(mar=c(5,5,0.5,0.5))

pdf(paste0(figures_dir,"Range_R_delta.pdf"), height =10,width=16)

with(F_xi_msa_m[F_xi_msa_m$q_k==1 & F_xi_msa_m$ks_p>0.1,],
     plot(100*R_range,10^4*deltak/phi_prime(R=R_min2006,beta=0.99,tau=1, sigma=1.27),
          xlab = TeX("Range of rates $R_{max}-R_{min}$ (percentage points)"),
          ylab ="basis points",
          cex.lab = 2.3,
          cex.axis =2.3,
          cex=2.3,
          pch = ".",
          type="p", col = "green",
          ylim = c(0,110)))
par(new=T)
with(F_xi_msa_m[F_xi_msa_m$q_k==nbanks & F_xi_msa_m$ks_p>0.1,],
     lines(100*R_range,10^4*deltak/phi_prime(R=R_min2006,beta=0.99,tau=1, sigma=1.27),
          xlab = TeX("Range of rates $R_{max}-R_{min}$ (percentage points)"),
          ylab = TeX("$\\Delta_k$"),
          type="p", col = "blue"))

legend('topleft',c(TeX("$\\Delta_1(R_{min})$")
                    ,TeX("$\\Delta_N(R_{min})$")),
       bty="n",
       cex=2.5,
       lty=0,lwd=2,pch=1,
       col=c("green","blue"))


dev.off()


# relationship between deltas and qs

qvec_delta_msa_m = merge(unique(qvec_msa_m[,list(msa_code,date,R_range,q1,ks_p)]),
                         unique(F_xi_msa_m[, list(msa_code,date,delta1,delta_R_1,dep_msa)]),
                   by = c("msa_code","date"))
summary(qvec_delta_msa_m$ks_p)
qvec_delta_msa_m = qvec_delta_msa_m[qvec_delta_msa_m$ks_p>0.1,]


qvec_delta_msa_m = qvec_delta_msa_m[order(qvec_delta_msa_m$msa_code,
                                          qvec_delta_msa_m$date),]

# full sample
fit_q1_delta <- lm(q1 ~ delta_R_1 + I(delta_R_1^2)+ I(delta_R_1^3),
                   weights =dep_msa,
                   data = qvec_delta_msa_m)
summary(fit_q1_delta)
qvec_delta_msa_m[,fitted_q1_delta := fitted(fit_q1_delta)]
summary(qvec_delta_msa_m$q1)

# fit subsamples

## 1997-2002
fit_q1_delta_s1 <- lm(q1 ~ delta_R_1 + I(delta_R_1^2)+ I(delta_R_1^3),
                   weights =dep_msa,
                   data = qvec_delta_msa_m[year(qvec_delta_msa_m$date)<=2002,])
qvec_delta_msa_m[,fitted_q1_delta_s1 := fitted(fit_q1_delta_s1)]
## 2002-2010
fit_q1_delta_s2 <- lm(q1 ~ delta_R_1 + I(delta_R_1^2)+ I(delta_R_1^3),
                      weights =dep_msa,
                      data = qvec_delta_msa_m[year(qvec_delta_msa_m$date)>=2002&
                                              year(qvec_delta_msa_m$date)<=2010  ,])
qvec_delta_msa_m[,fitted_q1_delta_s2 := fitted(fit_q1_delta_s2)]

summary(fit_q1_delta)
summary(fit_q1_delta_s1)
summary(fit_q1_delta_s2)

par(mar=c(6,8,0.1,0.1))

pdf(paste0(figures_dir,"R_delta_q1.pdf"), height =10,width=16)

with(qvec_delta_msa_m,
     plot(delta_R_1,q1,
          cex.lab = 2.3,
          cex.axis =2.3,
          cex=2.3,
          xlab = TeX("$\\Delta_1(R_{min})$"),
          ylab = TeX("$q_1$"),
          type="p",
          pch = ".",
          col = "green"))
par(new =T)
with(qvec_delta_msa_m,
     lines(delta_R_1,fitted_q1_delta,
          type="p",
          lwd=3,
          col = "black"))

dev.off()




#------------------------------------------------------------------------------>

# Plots of the estimates of the search cost distribution----

xi_median_sd_agg = unique(F_xi[,list(date,xi_median,xi_sd,xi_Rmin_median,xi_Rmin_sd)])

# ______ by year 1997-2016----
for(y in 1997:2016){
par(mar=c(6,6,0.1,0.1))
dd = paste0(y,"-06-30")
if(y==2016){
  dd = paste0(y,"-05-31")}
ff = paste0("Fxi_",y,".pdf")

pdf(paste0(figures_dir,ff), height =10,width=16)

with(F_xi[F_xi$date==dd,],
     plot(xi_Rmin,Fxi_p95,type="l",
          ylab = "Cumulative probability",
          xlab = "Search cost (basis points)",
          col = "gray",
          cex.lab =1.8,
          cex = 1.8,
          main = "",
          xaxt='n',
          yaxt='n',
          lwd=2,
          ylim = c(0,1)))
axis(side=1, at=seq(0,max(F_xi$xi_Rmin[F_xi$date==dd]), by =10), cex.axis = 1.7)
axis(side=2, at=seq(0,1, by =0.1), cex.axis = 1.7)

par(new=T)
with(F_xi[F_xi$date==dd,],
     lines(xi_Rmin,Fxi_p5,type="l",
          col="gray",
          lwd=2,
          ylim = c(0,1)))

with(F_xi[F_xi$date==dd,],
     polygon(c(xi_Rmin, rev(xi_Rmin)), c(Fxi_p5,rev(Fxi_p95)),
                           col = "grey", border = "grey70"))

with(F_xi[F_xi$date==dd,],
     lines(xi_Rmin,Fxi_p50,type="l",
           col="blue",
           lty=2,
           lwd=2,
           ylim = c(0,1)))

with(F_xi[F_xi$date==dd,],
     lines(xi_Rmin,Fxi_wm,type="l",
           col="black",
           lty=1,
           lwd=2,
           ylim = c(0,1)))

legend("topleft",
       c("5th/95th percentile",
         "Median","Deposit-weighted"),
       col = c("gray70","blue","black"),
       cex =2,
       bty = "n",
       lty = c(1,2,1),
       lwd = c(10,2,2))

with(F_xi[F_xi$date==dd,],abline(v=delta1_Rmin_wm,lwd=2,lty=2, col = "red"))
with(F_xi[F_xi$date==dd,],abline(v=delta2_Rmin_wm,lwd=2,lty=2, col = "slateblue4"))



max_xi = unique(F_xi$xi_Rmin_max[F_xi$date==dd])
min_xi = unique(F_xi$delta1_Rmin_min_d[F_xi$date==dd])
wm_xi = unique(F_xi$delta1_Rmin_wm[F_xi$date==dd])

wm_delta2 = unique(F_xi$delta2_Rmin_wm[F_xi$date==dd])
   text(x =wm_xi, y = 0.15,pos = 4, col ="red",
        labels =TeX(paste0("$\\leftarrow$ $\\Delta_1$: ",round(wm_xi)," (bps)")),
        cex = 2)

   text(x =wm_delta2, y = 0.05,pos = 4, col ="slateblue4",
        labels =TeX(paste0("$\\leftarrow$ $\\Delta_2$: ",
                           round(wm_delta2)," (bps)")),
        cex = 2)

dev.off()
}



# ______ Comparisons 2000 - 2006----
pdf(paste0(figures_dir,"Fxi_2000_2006.pdf"), height =10,width=16)


with(F_xi[F_xi$date=="2000-06-30",],
     plot(xi_Rmin,Fxi_wm,type="l",
          ylab = "Cumulative probability",
          xlab = "Search cost (basis points)",
          col = "black",
          cex.lab =2.3,
          cex = 2.3,
          cex.axis=2.3,
          main = "",
          xaxt='n',
          yaxt='n',
          lwd=3,
          xlim = c(0,120),
          ylim = c(0,1)))
axis(side=1, at=seq(0,max(F_xi$xi_Rmin[F_xi$date=="2006-06-30"]), by = 20),cex.axis=2.3)
axis(side=2, at=seq(0,1, by =0.1),cex.axis=2.3)
par(new=T)
with(F_xi[F_xi$date=="2006-06-30",],
     lines(xi_Rmin,Fxi_wm,type="l",
           lty=2,
           lwd =3, col ="red"))


with(F_xi[F_xi$date=="2006-06-30",],abline(v=delta1_Rmin_wm,lwd=2,lty=2,
                                 col = "red"))


with(F_xi[F_xi$date=="2000-06-30",],abline(v=delta1_Rmin_wm,lwd=2,lty=1,
                                           col = "black"))


text(x =unique(F_xi$delta1_Rmin_wm[F_xi$date=="2000-06-30"]),
     y = 0.15,pos =4, col ="black",offset = -0.2,
     labels =TeX(paste0(" $\\leftarrow$ $\\Delta_1(R_{min})$=",
                        round(unique(F_xi$delta1_Rmin_wm[F_xi$date=="2000-06-30"])),
                        " (bps)")),
     cex = 2.3)

text(x =unique(F_xi$delta1_Rmin_wm[F_xi$date=="2006-06-30"]),
     y = 0.3,pos =4, col ="red",offset = -0.2,
     labels =TeX(paste0(" $\\leftarrow$ $\\Delta_1(R_{min})$=",
                        round(unique(F_xi$delta1_Rmin_wm[F_xi$date=="2006-06-30"])),
                        " (bps)")),
     cex = 2.3)

legend("topleft",c("2000","2006"), cex = 2.3,
       col = c("black","red"),
       lwd = c(3,3), lty = c(1,2),bty="n")

dev.off()

# ______ Comparisons 2003 - 2010----
par(mar=c(4.5,4.5,0.1,0.1))
pdf(paste0(figures_dir,"Fxi_2003_2010.pdf"), height =10,width=16)
with(F_xi[F_xi$date=="2003-06-30",],
     plot(xi_Rmin,Fxi_wm,type="l",
          ylab = "Cumulative probability",
          xlab = "Search cost (basis points)",
          col = "black",
          cex.lab =2.3,
          cex = 2.3,
          cex.axis = 2.3,
          main = "",
          xaxt='n',
          yaxt='n',
          lwd=3,
          xlim = c(0,80),
          ylim = c(0,1)))
axis(side=1, at=seq(0,max(F_xi$xi_Rmin[F_xi$date=="2003-06-30"]), by =10),cex.axis = 2.3)
axis(side=2, at=seq(0,1, by =0.1),cex.axis = 2.3)
par(new=T)
with(F_xi[F_xi$date=="2010-06-30",],
     lines(xi_Rmin,Fxi_wm,type="l",
           lty=2,
           lwd =3, col ="red"))


with(F_xi[F_xi$date=="2003-06-30",],abline(v=delta1_Rmin_wm,lwd=0.5,lty=1,
                                           col = "black"))


with(F_xi[F_xi$date=="2010-06-30",],abline(v=delta1_Rmin_wm,lwd=0.5,lty=2,
                                           col = "red"))


text(x =unique(F_xi$delta1_Rmin_wm[F_xi$date=="2003-06-30"]),
     y = 0.15,pos =4, col ="black",offset = -0.2,
     labels =TeX(paste0(" $\\leftarrow$ $\\Delta_1^{2003}(R_{min})$=",
                        round(unique(F_xi$delta1_Rmin_wm[F_xi$date=="2003-06-30"])),
                        " (bps)")),
     cex = 2.3)

text(x =unique(F_xi$delta1_Rmin_wm[F_xi$date=="2010-06-30"]),
     y = 0.3,pos =4, col ="red",offset = -0.2,
     labels =TeX(paste0(" $\\leftarrow$ $\\Delta_1^{2010}(R_{min})$=",
                        round(unique(F_xi$delta1_Rmin_wm[F_xi$date=="2010-06-30"])),
                        " (bps)")),
     cex = 2.3)

legend("topleft",c("2003","2010"),
       col = c("black","red"), cex = 2.3,
       lwd = c(3,3), lty = c(1,2),bty="n")

dev.off()



# unadjusted xi


pdf(paste0(figures_dir,"Fxi_xi_2000_2006.pdf"), height =10,width=16)

with(F_xi[F_xi$date=="2000-06-30",],
     plot(xi,Fxi_xi_wm,type="l",
          ylab = "Cumulative probability",
          xlab = "Search cost (utils)",
          col = "black",
          cex.lab =1.7,
          cex = 1,
          main = "",
          xaxt='n',
          yaxt='n',
          lwd=2,
          #xlim = c(0,120),
          ylim = c(0,1)))
axis(side=1, at=seq(0,max(F_xi$xi[F_xi$date=="2006-06-30"]), by =1))
axis(side=2, at=seq(0,1, by =0.1))
par(new=T)
with(F_xi[F_xi$date=="2006-06-30",],
     lines(xi,Fxi_xi_wm,type="l",
           lty=2,
           lwd =2, col ="red"))

with(F_xi[F_xi$date=="2006-06-30",],abline(v=delta1_wm,lwd=0.5,lty=2,
                                           col = "red"))


with(F_xi[F_xi$date=="2000-06-30",],abline(v=delta1_wm,lwd=0.5,lty=1,
                                           col = "black"))

text(x =unique(F_xi$delta1_wm[F_xi$date=="2000-06-30"]),
     y = 0.15,pos =4, col ="black",offset = 0.1,
     labels =TeX(paste0("$\\leftarrow$ $\\Delta_1$ (2000) ")),
     cex = 1.5)

text(x =unique(F_xi$delta1_wm[F_xi$date=="2006-06-30"]),
     y = 0.3,pos =4, col ="red",offset = 0.1,
     labels =TeX(paste0(" $\\leftarrow$ $\\Delta_1$ (2006)")),
     cex = 1.5)

dev.off()




pdf(paste0(figures_dir,"Fxi_xi_2003_2010.pdf"), height =10,width=16)

with(F_xi[F_xi$date=="2003-06-30",],
     plot(xi,Fxi_xi_wm,type="l",
          ylab = "Cumulative probability",
          xlab = "Search cost (utils)",
          col = "black",
          cex.lab =1.7,
          cex = 1,
          main = "",
          xaxt='n',
          yaxt='n',
          lwd=2,
          #xlim = c(0,120),
          ylim = c(0,1)))
axis(side=1, at=seq(0,max(F_xi$xi[F_xi$date=="2003-06-30"]), by =1))
axis(side=2, at=seq(0,1, by =0.1))
par(new=T)
with(F_xi[F_xi$date=="2010-06-30",],
     lines(xi,Fxi_xi_wm,type="l",
           lty=2,
           lwd =2, col ="red"))

with(F_xi[F_xi$date=="2003-06-30",],abline(v=delta1_wm,lwd=0.5,lty=2,
                                           col = "red"))


with(F_xi[F_xi$date=="2010-06-30",],abline(v=delta1_wm,lwd=0.5,lty=1,
                                           col = "black"))

text(x =unique(F_xi$delta1_wm[F_xi$date=="2003-06-30"]),
     y = 0.15,pos =4, col ="red",offset = -0.3,
     labels =TeX(paste0("$\\leftarrow$ $\\Delta_1$ (2003) ")),
     cex = 1.5)

text(x =unique(F_xi$delta1_wm[F_xi$date=="2010-06-30"]),
     y = 0.3,pos =4, col ="black",offset = -0.4,
     labels =TeX(paste0(" $\\leftarrow$ $\\Delta_1$ (2010)")),
     cex = 1.5)

dev.off()



# search cost distribution moments
date_axis=unique(xi_median_sd_agg$date)
date_axis = date_axis[order(date_axis)]
date_axis=date_axis[month(date_axis)==5]

par(mar=c(7,7,0.1,0.1))

pdf(paste0(figures_dir,"SearchCosts_med_sd.pdf"), height =10,width=16)

with(xi_median_sd_agg,
     plot(date, xi_Rmin_median,
          ylim = c(0,55),
          xaxt = "n",
          ylab = "basis points",
          xlab = "",
          cex =2,
          cex.lab =1.7,
          cex.axis = 1.7,
          lwd =3,
          type = "l"))
axis.Date(side = 1, xi_median_sd_agg$date,
          at = date_axis,cex.axis = 1.7,
          format = "%Y", las = 2)
par(new=T)
with(xi_median_sd_agg,
     lines(date, xi_Rmin_sd,
           type = "l", col = "red", lwd =2))

legend("topright",c("Median","Standard deviation"),
       col = c("black","red"), cex = 2,
       lwd = c(3,3), lty = c(1,1),bty="n")

dev.off()



pdf(paste0(figures_dir,"SearchCosts_med_sd_utils.pdf"), height =10,width=16)

with(xi_median_sd_agg,
     plot(date, xi_median,
          ylim = c(0,0.035),
          xaxt = "n",
          ylab = "utils",
          xlab = "",
          cex =2,
          cex.lab =1.7,
          cex.axis = 1.7,
          lwd =3,
          type = "l"))
axis.Date(side = 1, xi_median_sd_agg$date,
          at = date_axis,cex.axis = 1.7,
          format = "%Y", las = 2)
par(new=T)
with(xi_median_sd_agg,
     lines(date, xi_sd,
           type = "l", col = "red", lwd =2))

legend("topright",c("Median","Standard deviation"),
       col = c("black","red"), cex = 2,
       lwd = c(3,3), lty = c(1,1),bty="n")

dev.off()



#-------------------------------------------------------------------------------->
# Construct search cost distribution stats by market----
#-------------------------------------------------------------------------------->
source(paste0(code_proj_dir,"structural_est_functions.R"))
## construct mean and standard deviation of the search cost by market and date
msa_fxi = unique(F_xi_msa_m$msa_code)

#-------------------------------------------------------------------------------->
cluster = makeCluster(20, type = "SOCK")
#registerDoSNOW(cluster)
registerDoParallel(cl = cluster, cores =20)
#-------------------------------------------------------------------------------->
require(GoFKernel)
market_Fxi = foreach(m =msa_fxi, .combine ="rbind",
               .packages = c("foreach","lubridate","data.table","GoFKernel","nleqslv" ),
               .verbose = T,
               .errorhandling ="remove",
               .export = c("phi_prime","h"))%dopar%{

               dates_m = unique(F_xi_msa_m$date[F_xi_msa_m$msa_code==m]) # unique dates
               dates_m = dates_m[order(dates_m)]

               market_Fxi_m = foreach(d =dates_m, .combine ="rbind",
                                  .packages = c("foreach","nleqslv",
                                                "lubridate","GoFKernel","data.table")
                                  )%do%{
                                    print(d)

                                    data_d = F_xi_msa_m[F_xi_msa_m$msa_code==m &
                                                        F_xi_msa_m$date==d,]
                                    data_qvec_d = qvec_msa_m[qvec_msa_m$msa_code==m &
                                                             qvec_msa_m$date==d,]
                                    Nbanks = data_d$nbanks[1]
                                    R_min = data_d$R_min[1]
                                    R_max = data_d$R_max[1]
                                    R_tilde = data_d$R_t[1]
                                    qvec = data_qvec_d$qvec

                                    # construct support of cost function / basis points conversion
                                    phi_Rmin = phi_prime(unique(data_d$R_min),
                                                         sigma = 1.276, beta = 0.99, tau=1)
                                    xi =10^4*data_d$xi/phi_Rmin
                                    Fxi = data_d$Fxi
                                    xi_min = min(xi)
                                    xi_max = max(xi)
                                    # grid for the search costs
                                    xi_grid =  seq(0,1.5*xi_max,
                                                   length.out = 1000) # grid

                                    fxi_grid = seq(0,1,length.out = 1000)


                                    # user functions
                                    h = function(R, sigma, beta, tau){
                                      if(sigma==1){
                                        hh = rep(1/(1+beta^tau), length(R))
                                        return(hh)
                                      }
                                      if(sigma!=1){
                                        return(1/(1+beta^(tau*sigma)*R^(sigma-1)))
                                      }
                                    }

                                    Fzero = function(z,R,qvec,R_min,R_tilde,sigma,beta,tau){
                                      N = length(qvec)
                                      q1 = qvec[1]
                                      sum_q_F = 0
                                      for(i in 1:N){
                                        sum_q_F = sum_q_F+ i*qvec[i]*z^(i-1)
                                      }
                                      y = ( (R_tilde - R)*(1-h(R,sigma,beta,tau)))*sum_q_F - (R_tilde - R_min)*(1-h(R_min,sigma,beta,tau))*q1
                                      return(y)
                                    }

                                    F_R <-function(R){
                                      FFzero = function(z){return(Fzero(z,R,qvec,R_min,R_tilde,sigma,beta,tau))}
                                      FF = nleqslv(1/2,FFzero)$x
                                      if(R>R_max){FF=1}
                                      if(R<R_min){FF=0}
                                      return(FF)
                                    }
                                    F_R = Vectorize(F_R)

                                    f_R = function(R){
                                      FF = F_R(R)
                                      q_sum_1 = 0
                                      q_sum_2 = 0
                                      N=length(qvec)
                                      for(i in 1:N){
                                        q_sum_1 = q_sum_1+i*qvec[i]*FF^(i-1)
                                        q_sum_2 = q_sum_2+i*(i-1)*qvec[i]*FF^(i-2)
                                      }
                                      ff = ((-1)*psi_prime(R,R_tilde, sigma, beta, tau)/psi(R,R_tilde, sigma, beta, tau))*q_sum_1/q_sum_2
                                      if(R>R_max){ff=0}
                                      if(R<R_min){ff=0}
                                      return(ff)
                                    }
                                    f_R = Vectorize(f_R)

                                    V_k = function(k){
                                      int = function(R){
                                        return(phi(R,sigma, beta, tau)*k*F_R(R)^(k-1)*f_R(R))
                                      }
                                      VV = integrate(int,R_min,R_max)$value
                                      return(VV)
                                    }
                                    V_k = Vectorize(V_k)

                                    V_all = foreach(k=1:Nbanks, .combine ="sum")%do%{
                                      VV=V_k(k)*qvec[k]
                                      return(VV)
                                    }
                                    V_R_t =phi(R_tilde,sigma, beta, tau)
                                    Fxi_fit = splinefun(xi, Fxi, method = "monoH.FC")
                                    Fxi_fit = Vectorize(Fxi_fit)

                                    Fxi_inv = inverse(Fxi_fit, lower = xi_min, upper = xi_max)
                                    Fxi_inv = Vectorize(Fxi_inv)

                                    Fxi_inv2 = function(x){

                                      if(x<min(Fxi)){y =min(xi)}
                                      if(x>=min(Fxi)){y =Fxi_inv(x)}
                                      return(y)}

                                    Fxi_fit2 = function(x){
                                      y = Fxi_fit(x)

                                      for(i in 1:length(y)){
                                        if(y[i]>1){y[i]=1}
                                        if(y[i]<0){y[i]=0}
                                      }
                                      return(y)}

                                    plot(xi, Fxi)
                                    par(new=T)
                                    lines(xi_grid, Fxi_fit2(xi_grid))

                                    int_fun1 = function(x){ x*(1- Fxi_fit2(x))}
                                    int_fun2 = function(x){ (1- Fxi_fit2(x))}

                                    # compute variance/standard deviation of search cost
                                    xi_sd = sqrt(2*integrate(int_fun1,lower = 0, upper =10*xi_max )$value-
                                                   integrate(int_fun2,lower = 0, upper = 10*xi_max)$value^2)
                                    # compute mean
                                    xi_mean = integrate(int_fun2,lower = 0, upper =10*xi_max )$value
                                    # median and interquartile range
                                    xi_median = Fxi_inv2(0.5)
                                    xi_p25 = Fxi_inv2(0.25)
                                    xi_p75 = Fxi_inv2(0.75)

                                    market_Fxi_m_d = data.frame(msa_code = m,
                                                                date = d,
                                                                ks_p = unique(data_d$ks_p),
                                                                R_min = unique(data_d$R_min),
                                                                R_max = unique(data_d$R_max),
                                                                xi_sd = xi_sd,
                                                                xi_median = xi_median,
                                                                xi_mean = xi_mean,
                                                                xi_p25=xi_p25,
                                                                xi_p75=xi_p75)


                                    return(market_Fxi_m_d)}

               market_Fxi_m = data.table(market_Fxi_m)
               market_Fxi_m$year = year(market_Fxi_m$date)

               market_Fxi_collapse_m = market_Fxi_m[,list(xi_sd = mean(xi_sd,na.rm=T),
                                                 xi_median = median(xi_median,na.rm=T),
                                                 xi_mean = median(xi_mean,na.rm=T),
                                                 ks_p = mean(ks_p,na.rm=T),
                                                 R_min = median(R_min),
                                                 R_max = median(R_max),
                                                 xi_p25 = mean(xi_p25,na.rm=T),
                                                 xi_p75 = mean(xi_p75,na.rm=T)),
                                           by = c("msa_code","year")]

               return(market_Fxi_collapse_m)}



stopCluster(cluster)
#-------------------------------------------------------------------------------->
# Save market Fxi----
setwd(final_data_dir)
summary(market_Fxi)
save(market_Fxi, file = "market_Fxi.rda")
#-------------------------------------------------------------------------------->





#-------------------------------------------------------------------------------->
# Marginal cost estimates----
#-------------------------------------------------------------------------------->
qvec_agg_m[,R_t:=100*(R_t-1)]
qvec_agg_m[,R_t_w:=100*(R_t_w-1)]
qvec_agg_m[,R_t_p5:=100*(R_t_p5-1)]
qvec_agg_m[,R_t_p95:=100*(R_t_p95-1)]
qvec_agg_m[,R_max:=100*(R_max-1)]
qvec_agg_m[,R_min:=100*(R_max-R_range-1)]
qvec_agg_m[,m:=month(date)]
qvec_agg_m[,y:=year(date)]
qvec_treas_m = merge(qvec_agg_m,Treasm,by = c("y","m"), all.x=T)
qvec_treas_m = merge(qvec_treas_m,primcredit,by = c("y","m"), all.x=T)
qvec_treas_m=qvec_treas_m[order(qvec_treas_m$date),]
#-------------------------------------------------------------------------------->
# Plot the distribution of yields

date_axis = unique(qvec_treas_m$date)
date_axis = date_axis[order(date_axis)]
date_axis = date_axis[which(month(date_axis)==6)]
date_axis2 = seq(min(date_axis), max(date_axis), by = 365)
date_axis2 = c(date_axis2,max(qvec_treas_m$date))


pdf(file = paste0(figures_dir,"MC_estimate.pdf"),width = w , height = w/h)
with(qvec_treas_m,plot(date, R_t_w, type="l", col = "red",
                       cex.lab=2,
                       ylim = c(0,8),
                       lwd=2,
                       cex=2,
                       cex.axis =2,
                       xaxt = "n",
                       ylab = "",
                       xlab=""))
par(new =T)
axis.Date(side = 1, qvec_treas_m$date, at =date_axis2 ,cex.axis =2,
          format = "%Y", las = 2)
grid(NULL,NULL, col = "black", lty = "dotted", lwd = 1, equilogs = F)
with(qvec_treas_m,lines(date, R_max, type="l", lwd=2,
                        col = "black"))
with(qvec_treas_m,lines(date, libor12mo, type="l", lwd=2,
                        col = "blue"))
with(qvec_treas_m,lines(date, prcredit8, type="l", lwd=2,
                        col = "green"))
with(qvec_treas_m,lines(date, dff, type="l", lwd=2,
                        col = "cyan"))
abline(h=0)

legend('topright',ncol=1,cex =1.2,
       bg = "white",
       box.lwd = 0,
       c(TeX("Marginal cost $\\tilde{R}$"),
         TeX("Maximal rate $R_{max}$"),
         TeX("12-month LIBOR"),
         "Discount window primary credit","Effective fed funds rate"),
       lty=c(1,1,1,1),lwd =2,col=c("red","black","blue","green","cyan"))

dev.off()




with(qvec_agg_m,plot(date,100*R_sd,type ="l", col = "red"))
with(qvec_agg_m,plot(date,search_avg,type ="l", ylim = c(0,0.4)))
with(qvec_agg_m,plot(date,100*(R_t-1),type ="l"))
par(new =T)
with(qvec_agg_m,lines(date,100*(R_max-1),type ="l", col="red"))
with(qvec_agg_m,plot(date,100*(R_t-R_max),type ="l"))


# marginal cost estimates and q_k shares
qvec_delta_m = qvec_delta_m[qvec_delta_m$ks_p>0.05]
with(qvec_delta_m[qvec_delta_m$q_k==1], plot(qvec,R_t))
with(qvec_delta_m[qvec_delta_m$q_k==2], plot(qvec,R_t))
with(qvec_delta_m[qvec_delta_m$q_k==3], plot(qvec,R_t))
with(qvec_delta_m[qvec_delta_m$q_k==4], plot(qvec,R_t))
with(qvec_delta_m, plot(nbanks,R_t))

summary(qvec_delta_m$ks_p[qvec_delta_m$R_t>1.1])
summary(qvec_delta_m$ks_p)

