function Score_f  = Score(sigma,qvec,R,R_min,R_max,R_t,F, betta,tau)
% The score function: Score(sigma,qvec,R,F)
N = max(size(qvec));
qvec = qvec/sum(qvec);

h_f = @(R) 1./(1+betta.^(tau*sigma).*R.^(sigma-1));
h_sigma_f = @(R) -h_f(R).*(1-h_f(R)).*log(betta^tau*R);
psi_f = @(R) (R_t - R).*(1-h_f(R));
psi_R_f = @(R)-(1-h_f(R))*(1+(1-sigma)*(R_t-R)./R*h_f(R));




%% different sums of q_k
s0 = 0;
s1=0;
s2=0;
s3=0;
for k= 1:N
   s0 = s0+k*qvec(k);
   s1 = s1+k*qvec(k)*F^(k-1);
   s2 = s2+k*(k-1)*qvec(k)*F^(k-2);
   s3 = s3+k*(k-1)*(k-2)*qvec(k)*F^(k-3);
end


%% marginal cost of funds -- derivatives wrt (sigma,qvec)
% sigma
NN = R_max*(1-h_f(R_max))*s0-R_min*(1-h_f(R_min))*qvec(1);
DD = (1-h_f(R_max))*s0-(1-h_f(R_min))*qvec(1);

NN_sigma= -h_sigma_f(R_max)*R_max*s0+R_min*h_sigma_f(R_min)*qvec(1);
DD_sigma= -h_sigma_f(R_max)*s0+h_sigma_f(R_min)*qvec(1);

R_tilde_sigma = (NN_sigma*DD-NN*DD_sigma)/DD^2;
% qvec:
R_tilde_qvec = zeros(N,1);
NN_q1= R_max*(1-h_f(R_max))-R_min*(1-h_f(R_min));
DD_q1= (1-h_f(R_max))-(1-h_f(R_min));
R_tilde_qvec(1) = (NN_q1*DD-NN*DD_q1)/DD^2;

for k = 2:N
    NN_qk = R_max*(1-h_f(R_max))*k;
    DD_qk = (1-h_f(R_max))*k;
    R_tilde_qvec(k)= (NN_qk*DD-NN*DD_qk)/DD^2;
end

%% profit function psi
% sigma
psi_sigma_f = @(R) -h_sigma_f(R)*(R_t-R)+R_tilde_sigma*(1-h_f(R));

psi_R_sigma_f =@(R) h_sigma_f(R)*(1+(1-sigma)*(R_t-R)./R*h_f(R))-...
(1-h_f(R))*( (1-sigma)*(R_t-R)./R*h_sigma_f(R)+(1-sigma)*h_f(R)*R_tilde_sigma/R...
-(R_t-R)./R*h_f(R));

% R_tilde
psi_Rtilde_f =@(R) (1-h_f(R));
psi_R_Rtilde_f =@(R) -(1-h_f(R))*(1-sigma)*(1/R)*h_f(R);


%% Equilibrium F_R
% sigma
F_sigma = (1/s2)*( ( psi_sigma_f(R_min)*psi_f(R)-psi_sigma_f(R)*psi_f(R_min))/psi_f(R)^2);

F_qvec = zeros(N,1);

F_qvec(1) = - (1/s2)*(1- psi_f(R_min)/psi_f(R) -...
((psi_Rtilde_f(R_min)*psi_f(R)-psi_Rtilde_f(R)*psi_f(R_min))/psi_f(R)^2)*R_tilde_qvec(1)*qvec(1));

% qvec
for k =2:N
   F_qvec(k) =  - (1/s2)*(k*F^(k-1)-...
((psi_Rtilde_f(R_min)*psi_f(R)-psi_Rtilde_f(R)*psi_f(R_min))/psi_f(R)^2)*R_tilde_qvec(k)*qvec(1));
end

%% Score function 
Score_f = zeros(N+1,1);

% sigma 
Score_f(1) = (s1/s2)*(-psi_R_sigma_f(R)*psi_f(R)+psi_R_f(R)*psi_sigma_f(R))/psi_f(R)^2-(psi_R_f(R)/psi_f(R))*(1- s1*s3/s2^2)*F_sigma;

for k = 1:N
Score_f(k+1) = -(psi_R_f(R)/psi_f(R))*( (1/s2)*k*F^(k-1)-(s1/s2^2)*k*(k-1)*F^(k-2)+(1-s1*s3/s2^2)*F_qvec(k))-...
    (s1/s2)*(psi_R_Rtilde_f(R)*psi_f(R)-psi_R_f(R)*psi_Rtilde_f(R))/psi_f(R)^2*R_tilde_qvec(k);
    
end

end

