function [ S ] = Score_analytic(sigma,qvec,R,R_min,R_max,F,betta,tau)
%Score_analytic(theta,R,F ): Compute analytic derivative wrt to
%sigma,{q_k}
N = max(size(qvec));
s0=0;
s1=0;
for k= 1:N
    s0 = s0+k*qvec(k);
    s1 = s1+k*qvec(k)*F^(k-1);
end
s2=0;
for k= 2:N
    s2 = s2+k*(k-1)*qvec(k)*F^(k-2);
end
s3=0;
for k= 3:N
    s3 = s3+k*(k-1)*(k-2)*qvec(k)*F^(k-3);
end

% marginal propensity to consume
h_RR = @(R) 1./(1+betta.^(tau*sigma).*R.^(sigma-1));
h_RR_sigma = @(R) -h_RR(R).*(1-h_RR(R)).*log(betta^tau*R);

% marginal utility of wealth
phi_RR = @(R) h_RR(R).^(1/(1-sigma));


%% marginal cost
N_R_t = (R_max*(1-h_RR(R_max))*s0-R_min*(1-h_RR(R_min))*qvec(1));
N_R_t_sigma = -R_max*h_RR_sigma(R_max)*s0+R_min*h_RR_sigma(R_min)*qvec(1);

D_R_t = ((1-h_RR(R_max))*s0-(1-h_RR(R_min))*qvec(1));
D_R_t_sigma = -h_RR_sigma(R_max)*s0+h_RR_sigma(R_min)*qvec(1);


R_t = (R_max*(1-h_RR(R_max))*s0-R_min*(1-h_RR(R_min))*qvec(1))./...
    ((1-h_RR(R_max))*s0-(1-h_RR(R_min))*qvec(1));

% derivatives wrt sigma
R_t_sigma = (N_R_t_sigma.*D_R_t-N_R_t.*D_R_t_sigma)./D_R_t.^2;
% derivatives wrt qk
R_t_qvec = zeros(N,1);
N_R_t_q1 = (R_max*(1-h_RR(R_max))-R_min*(1-h_RR(R_min)));
D_R_t_q1 = -h_RR(R_max)+h_RR(R_min);

R_t_qvec(1) = (N_R_t_q1.*D_R_t-N_R_t.*D_R_t_q1)./D_R_t.^2;
for k = 2:N
    N_R_t_qk = R_max*(1-h_RR(R_max))*k;
    D_R_t_qk = (1-h_RR(R_max))*k;
    R_t_qvec(k) = (N_R_t_qk.*D_R_t-N_R_t.*D_R_t_qk)./D_R_t.^2;
end



%% profit function
psi_RR =@(R) (R_t - R).*(1-h_RR(R));
psi_prime_RR = @(R) -(1-h_RR(R)).*(1+(1-sigma).*(R_t-R)./R.*h_RR(R));

% derivatives wrt sigma
psi_RR_sigma = @(R) -(R_t-R).*h_RR_sigma(R)+R_t_sigma.*(1-h_RR(R));
psi_prime_RR_sigma = @(R) h_RR_sigma(R).*(1+(1-sigma).*(R_t-R)./R.*h_RR(R))-...
 (1-h_RR(R)).*(-(R_t-R)./R.*h_RR(R)+(1-sigma).*(R_t-R)./R.*h_RR_sigma(R)+...
 (1-sigma).*h_RR(R).*R_t_sigma./R);   


psi_RR_R_t = @(R) (1-h_RR(R));
psi_prime_RR_R_t = @(R) -(1-h_RR(R)).*(1-sigma).*(h_RR(R)./R);


%% Derivatives of equilibrium F wrt sigma, qk
F_RR_sigma = -(1/(s2*psi_RR(R))).*(s1*psi_RR_sigma(R)-psi_RR_sigma(R_min)*qvec(1));

F_RR_qvec =zeros(N,1);
F_RR_qvec(1) = -(1/(s2*psi_RR(R))).*(psi_RR(R)+s1*psi_RR_R_t(R)*R_t_qvec(1) - psi_RR(R_min)-qvec(1)*psi_RR_R_t(R_min)*R_t_qvec(1));

for k = 2:N
    F_RR_qvec(k) = -(1/(s2*psi_RR(R))).*(k*F^(k-1)*psi_RR(R)+s1*psi_RR_R_t(R)*R_t_qvec(k)-qvec(1)*psi_RR_R_t(R_min)*R_t_qvec(k));
end
%% Likelihood function

f_RR = -(psi_prime_RR(R)/psi_RR(R))*s1/s2;

%% derivatives of the likelihood function
% wrt sigma
f_RR_sigma = - ((psi_prime_RR_sigma(R)*psi_RR(R)-psi_RR_sigma(R)*psi_prime_RR(R))./psi_RR(R)^2)*(s1/s2)-...
    (psi_prime_RR(R)/psi_RR(R))*(1-s1*s3/s2^2)*F_RR_sigma;

% wrt qk
f_RR_qvec = zeros(N,1);
for k = 1:N;
 f_RR_qvec(k) = - ((psi_prime_RR_R_t(R)*psi_RR(R)-psi_RR_R_t(R)*psi_prime_RR(R))./psi_RR(R)^2)*(s1/s2)*R_t_qvec(k)-...
     (psi_prime_RR(R)/psi_RR(R))*((1/s2)*k*F^(k-1)- (s1/s2^2)*k*(k-1)*F^(k-2)+(1-s1*s3/s2^2)*F_RR_qvec(k));
end
S =[f_RR_sigma/f_RR; f_RR_qvec/f_RR];
end

