function [ Score_eval ] = Score_symbolic(sigma,qvec,R,R_min,R_max,F,betta,tau)
%Score_symbolic(sigma,qvec,R,R_min,R_max,F,betta,tau):
% Constructs the score function using the Symbolic toolbox
N = max(size(qvec));
syms sigma_s R_s qvec_s s1_s F_s
        % create q1---qN as a symbolic array (cell)
        qvec_ss = sym('q',[N,1]); % store symbolic variables
        qvec_s = cell(N,1);
        theta_s = cell(N+1,1);
        theta_num = cell(N+1,1);
        theta_s{1} = sigma_s;
        theta_num{1} = sigma;
        for k = 1:N
            qvec_s{k} = qvec_ss(k);
            theta_s{k+1} = qvec_ss(k);
            theta_num{k+1} = qvec(k);
        end
        % define sums sum(kq_kF^(k-1))
        s1_s = sym(0);
        for k= 1:N
            s1_s = s1_s+k*qvec_ss(k)*F_s^(k-1);
        end
        
        
        sumq_s = sym(0);
        for k = 1:N
            sumq_s = sumq_s+k*qvec_ss(k);
        end
        % marginal propensity to consume
        h_s(R_s,sigma_s)=1./(1+betta.^(tau*sigma_s).*R_s.^(sigma_s-1));
         % marginal cost
        R_t_s(theta_s{:}) = (R_max*(1-h_s(R_max,sigma_s))*sumq_s-R_min*(1-h_s(R_min,sigma_s))*qvec_ss(1))./...
            ((1-h_s(R_max,sigma_s))*sumq_s-(1-h_s(R_min,sigma_s))*qvec_ss(1));
        
        
        
         
        % equlibrium condition
        G(theta_s{:},R_s,F_s) = (R_t_s-R_s)*(1-h_s(R_s,sigma_s))*s1_s-(R_t_s-R_min)*(1-h_s(R_min,sigma_s))*qvec_ss(1);
        
        % likelihood function
        f_R_s(theta_s{:},R_s,F_s) = -(1/diff(G,F_s))*diff(G,R_s);
        
       
        
        % score of the log-likelihood function        
        Score_s = sym([]);      
        for k = 1:N+1
            %disp(k);
            diff_f_theta_k = diff(f_R_s,theta_s{k});
            diff_f_F = diff(f_R_s,F_s);
            diff_F_theta_k = -(1/diff(G,F_s))*diff(G,theta_s{k});
            Score_s = [Score_s;(diff_f_theta_k+diff_f_F*diff_F_theta_k)./f_R_s];
        end
    
         
        Score_fun = matlabFunction(Score_s);       
       Score_eval = Score_fun(theta_num{:},R,F);
end

