# -----------------------------------------------------------------------------
# Log-likelihood function
# -----------------------------------------------------------------------------

## Auxiliary functions
# -----------------------------------------------------------------------------
# Solve for the equilibrium distribution of rates F(R) given q_k, k=1,2..N
# -----------------------------------------------------------------------------

Fzero = function(z,R,qvec,R_min,R_tilde,sigma,beta,tau){
  N = length(qvec)
  q1 = qvec[1]
  sum_q_F = 0
  for(i in 1:N){
    sum_q_F = sum_q_F+ i*qvec[i]*z^(i-1)
  }
  y = ( (R_tilde - R)*(1-h(R,sigma,beta,tau)))*sum_q_F - (R_tilde - R_min)*(1-h(R_min,sigma,beta,tau))*q1
  return(y)
}

FFzero = function(z){return(Fzero(z,R,qvec,R_min,R_tilde,sigma,beta,tau))}



##--Likelihood function
loglik <-function(qvec,sigma,beta,tau,Rvec, R_min, R_max){
  N = length(qvec)
  q1 = qvec[1]


  # compute the implied marginal cost of funds
  sum_kq = 0; # sum k x q_k k = 1,...,N
  for(i in 1:N){
    sum_kq = sum_kq+i*qvec[i]
  }

  # Marginal cost of funds
  R_tilde = (R_max*(1-h(R_max, sigma, beta, tau))*sum_kq -
               R_min*(1-h(R_min, sigma, beta, tau))*q1)/((1-h(R_max, sigma, beta, tau))*sum_kq -
                                                           (1-h(R_min, sigma, beta, tau))*q1)
  # Solve for the likelihood of observing R
  fvec = foreach(R = Rvec, .combine = "c")%do%{
    FFzero = function(z){return(Fzero(z,R,qvec,R_min,R_tilde,sigma,beta,tau))}
    FF = nleqslv(1/2,FFzero)$x
    q_sum_1 = 0
    q_sum_2 = 0
    for(i in 1:N){
      q_sum_1 = q_sum_1+i*qvec[i]*FF^(i-1)
      q_sum_2 = q_sum_2+i*(i-1)*qvec[i]*FF^(i-2)
    }
    ff = ((-1)*psi_prime(R,R_tilde, sigma, beta, tau)/psi(R,R_tilde, sigma, beta, tau))*q_sum_1/q_sum_2
    return(ff)}

  # negative of the log-likelihood for minimization
  loglk =-sum(log(fvec))

  # check if returned value is infinite --> assign very large positive value
  if(is.nan(loglk) | is.infinite(loglk)){loglk=10^16}

  loglk = as.numeric(loglk)

  return(loglk)
}





# -----------------------------------------------------------------------------
#
# -----------------------------------------------------------------------------
