function [loglk,fvec_score] = loglik_sigmafixed_sym(qvec,sigma,betta,tau,Rvec, R_min, R_max)
% Log-likelihood function
% Author: Vladimir Yankov (vladimir.yankov@gmail.com)
% Description: Estimates the search cost distributions
% Codes provided for replication purposes for 
% "In Search of a Risk-free Asset" forthcoming at the JMCB
%% extract the parameters of interest
options = optimset('Display','off','TolFun',1e-16);

%% remove missing zeros and R_min and R_max from Rvec
Rvec = Rvec(Rvec~=R_max & Rvec~=R_min & Rvec~=0 & ~isnan(Rvec));
N = max(size(Rvec)); % number of observations
M= max(size(qvec)); % number of firms
theta_num=cell(M,1); % parameter vector
for k = 1:M
    theta_num{k}=qvec(k);
end
%% Marginal cost of funds
R_t = R_tilde(qvec,sigma,R_min,R_max,betta, tau);
% profit function and the derivative of the profit function
psi_RR = @(R) psi_R(R,R_t, sigma, betta, tau);
psi_prime_RR = @(R) psi_prime(R,R_t, sigma, betta, tau);
% Evaluate the likelihood function 
fvec = zeros(N,1); %initialize the likelihood
fvec_theta = zeros(N,M+1); %initialize the score (N x M+1)
fvec_theta2 = zeros(N,M+1); %initialize the score (N x M+1)


%% Evaluate the equilibrium f_R, F_R,, f_theta, and score for each R_i
for i=1:N;
    %% solve for the equilibrium distribution of rates F_R 
    if Rvec(i)==R_min
        FF = 0;
    end
    if Rvec(i)==R_max
        FF = 1;
    end
    if Rvec(i) ~= R_min && Rvec(i)~=R_max    
    FF = fzero(@(z)Fzero(z,Rvec(i),qvec,R_min,R_t,sigma,betta,tau),0.5,options);
    end
    % the solution needs to be between 0 and 1
    if FF <0;
        FF=0;
    end
    if FF >1;
        FF=1;
    end
    % define sums used in computations below
     q_sum_1 = 0;
     q_sum_2 = 0;
     for j=1:M;
        q_sum_1 = q_sum_1+j*qvec(j)*FF^(j-1);
        q_sum_2 = q_sum_2+j*(j-1)*qvec(j)*FF^(j-2);
     end
     % likelihood
    fvec(i) = ((-1)*psi_prime_RR(Rvec(i))./psi_RR(Rvec(i)))*q_sum_1/q_sum_2;

    %% Calculate the gradient of the likelihood function
    % analytical derivation
    fvec_theta(i,:) = -Score_analytic(sigma,qvec,Rvec(i),R_min,R_max,FF,betta,tau)';

end;
% negative of the log-likelihood for minimization
loglk =-sum(log(fvec));

if isnan(loglk)||~isfinite(loglk)
    loglk = 1e32;
end

% compute the gradient 
% note drop derivative wrt to sigma as here it is fixed
fvec_theta = fvec_theta(:,2:end);
fvec_score = zeros(M,1);
for ii = 1:M
    fvec_score(ii) = sum(fvec_theta(:,ii));
end



end

